#!/usr/bin/env python

import os, sys,re
from PIL import Image, ImageDraw

dimensions = dict()
linestodraw = []
circlestodraw = []

#If num is not a number, look it up.
def getnumber(num):
    if num[0] == '-':
        return -getnumber(num[1:])
    try:
        return int(num)
    except ValueError:
        return dimensions[num]

def readdimensions(line):
    sline = line.split()
    try:
        dimensions[sline[0]] = int(sline[1])
    except ValueError:
        if sline[1][0] == '-':
            dimensions[sline[0]] = -dimensions[sline[1][1:]]
        else:
            dimensions[sline[0]] = dimensions[sline[1]]

def readfieldline(line):
    m = re.match(r"[(]\s*(-*\w+)\s*(-*\w+)\s*[)]\s*-\s*[(]\s*(-*\w+)\s*(-*\w+)\s*[)]", line)
    if m:
        xs = getnumber(m.group(1).strip())
        ys = getnumber(m.group(2).strip())
        xe = getnumber(m.group(3).strip())
        ye = getnumber(m.group(4).strip())
        l = ((xs, ys), (xe, ye))
        linestodraw.append(l)
        return
    m = re.match(r"circle\s*\(\s*(-*\w+)\s*(-*\w+)\s*(-*\w+)\s*(-*\w+)\s*\)", line)
    if m:
        x = getnumber(m.group(1).strip())
        y = getnumber(m.group(2).strip())
        r = getnumber(m.group(3).strip())
        magicnumber = getnumber(m.group(4).strip())
        c = (x,y,r,magicnumber)
        circlestodraw.append(c)

def readcarpetBorder(line):
    readfieldline(line)

def readfieldBorder(line):
    readfieldline(line)

def readfieldLines(line):
    readfieldline(line)

# We ignore the .Corner so far and nothing seems to be missing.
def readxCorner(line):
    None

def readtCorner0(line):
    None

def readtCorner90(line):
    None

def readtCorner180(line):
    None

def readtCorner270(line):
    None

def readlCorner0(line):
    None

def readlCorner90(line):
    None

def readlCorner180(line):
    None

def readlCorner270(line):
    None


if len(sys.argv) < 2:
    print ("Usage: field.py <field.cfg>")
    exit(-1)

lines = open(sys.argv[1]).readlines()
lines = map(lambda x: x.lstrip().rstrip(), lines)
lines = filter(lambda x: len(x) != 0 and x[0] != '/' and x[0] != '#', lines)

state = ""

for line in lines:
    m = re.match(r"\[(\w+)\]", line)
    if m:
        state = m.group(1)
    else:
        locals()["read"+state](line)

print(dimensions)
print(linestodraw)
print(circlestodraw)

#create a bitmap 
#center of orientation
cX = dimensions["xPosOpponentFieldBorder"]
cY = dimensions["yPosLeftFieldBorder"]

#centercircle radius
radius = dimensions["centerCircleRadius"]

#line width
lWidth = dimensions["fieldLinesWidth"]
hlWidth = lWidth / 2

#color of lines
cLine = (255, 255, 255) #RGB White
#color of field without lines
cField = (31, 148, 69) #RGB Green

iSizeX = 2 * cX
iSizeY = 2 * cY
im = Image.new("RGB", (iSizeX, iSizeY))

im.paste(cField, im.getbbox())

imd = ImageDraw.Draw(im)

#draw circle with given linewidth and radius
#imaging lacks of an option or parameter for width in ellipse() Function,
#therefore its necessary to firt draw a white filled circle with radius = radius +
#linewidth followed by a circle with field color and radius = radius
#
#ellipse() draws an ellipse regarding to its bounding box
imd.ellipse(((cX - radius - lWidth), (cY - radius - lWidth), (cX + radius + lWidth), (cY + radius + lWidth)), fill = cLine)
imd.ellipse(((cX - radius), (cY - radius), (cX + radius), (cY + radius)), fill = cField)

#draw read lines
for ((x1, y1),(x2, y2)) in linestodraw:
    d_x = x1 - x2
    if d_x > 0:
        d_x = 1
    elif d_x < 0:
        d_x = -1
    d_y = y1 - y2
    if d_y > 0:
        d_y = 1
    elif d_y < 0:
        d_y = -1
    imd.line([(x1 + cX + (hlWidth * d_x), y1 + cY + (hlWidth * d_y)),\
              (x2 + cX - (hlWidth * d_x), y2 + cY - (hlWidth * d_y))], fill = cLine, width = lWidth)

del imd

im.transform((iSizeX / 10, iSizeY / 10), Image.EXTENT, im.getbbox(), Image.BICUBIC).save("robo.bmp", "BMP")
#del im
im.save("robo.bmp", "BMP")
del im

#at the moment imaging does not write tga files, so we use extern unix-os convert tool
#os.system("convert robo.bmp field.tga")
#os.system("rm robo.bmp")
