#!/bin/bash

set -eu

export scriptPath=$(echo $0 | sed "s|^\.\./|`pwd`/../|" | sed "s|^\./|`pwd`/|")
export baseDir=$(dirname ${scriptPath})
export gtDir=$(dirname ${baseDir})
export includeDir="${baseDir}/include/"
defaultArchive="aldebaran-sdk-1.6.13-linux-i386.tar.gz"

source "${includeDir}/bhumanBase.sh"

includes="
include/alcommon/albroker.h \
include/alcommon/alfunctionmonitoring.h \
include/alcommon/alfunctorconvert.h \
include/alcommon/alfunctorcreate.h \
include/alcommon/alfunctor.h \
include/alcommon/alfunctorwithoutreturn.h \
include/alcommon/alfunctorwithreturn.h \
include/alcommon/almemoryfastaccess.h \
include/alcommon/almethodinfo.h \
include/alcommon/almodule.h \
include/alcommon/almoduleinfo.h \
include/alcommon/alproxy.h \
include/alcommon/alsharedclass.h \
include/alcommon/altaskmonitor.h \
include/alcommon/altools.h \
include/alcore/alerror.h \
include/alcore/alptr.h \
include/alcore/alsignal.hpp \
include/alcore/altypes.h \
include/alfactory/alfactory.h \
include/alfactory/basetype.h \
include/alproxies/almemoryproxy.h \
include/alproxies/almemoryproxyposthandler.h \
include/alproxies/dcmproxy.h \
include/alproxies/dcmproxyposthandler.h \
include/altools/altimeval.h \
include/altools/alxplatform.h \
include/altools/tools.h \
include/alvalue/alvalue.h \
include/configcore.h \
include/libthread/alcriticalsection.h \
include/libthread/alcriticalsectionread.h \
include/libthread/alcriticalsectionwrite.h \
include/libthread/almutex.h \
include/libthread/altask.h \
include/libthread/althreadpool.h \
include/libthread/althreadtypes.h"
usage() {
  echo "usage:"
  echo "${0} <sdk-archive>"
  echo ""
  echo " <sdk-archive> : The archive file with the aldebaran linux sdk"
  exit 1
}

if [ $# -ne 1 ]
then
  usage
else
  archive=$1
fi

tmpdir="$(mktemp -d)"
archiveDir="${archive}"
archiveBase="$(dirname ${archiveDir})"
archiveDir="${archiveDir#${archiveBase}}"
archiveDir="${archiveDir#/}"
if [ "${defaultArchive}" != "${archiveDir}" ]
then
  warning "sdk-archive (\"${archiveDir}\") does not match the expected name (\"${defaultArchive}\")."
fi
archiveDir="${archiveDir%\.tar\.gz}"

debug "tempdir: " "${tmpdir}"
debug "archive: " "${archive}"
debug "archiveBase: " "${archiveBase}"
debug "archiveDir: " "${archiveDir}"

for f in ${includes}
do
  alcommonIncludes+="${archiveDir}/${f} "
done
boostIncludes+="${archiveDir}/include/boost/ "

alcommonContent="${alcommonIncludes}${archiveDir}/lib/libalcommon.so"
echo -n "Unpacking required content..."
tar -xz -C "${tmpdir}" -f "${archive}" ${alcommonContent}
tar -xz -C "${tmpdir}" -f "${archive}" ${boostIncludes}
echo "Done"

echo -n "Copying alcommon headers to destination..."
mkdir -p "${baseDir}/../Util/alcommon/include" 
for f in ${includes}
do
  fEnd=${f##*/}
  sed -e 's/\r$//g' "${tmpdir}/${archiveDir}/${f}" > "${baseDir}/../Util/alcommon/include/${fEnd}"
done
echo "Done"

echo -n "Copying alcommon library to destination..."
mkdir -p "${baseDir}/../Util/alcommon/lib/linux" 
cp "${tmpdir}/${archiveDir}/lib/libalcommon.so" "${baseDir}/../Util/alcommon/lib/linux/libalcommon.so"
echo "Done"

echo -n "Copying boost headers to destination..."
for f in `find "${tmpdir}/${archiveDir}/include" -not -type d -regex '.*boost/.*' -printf '%P\n'`
do
  dirOfFile=$(dirname ${f})
  if [ ! -d "${baseDir}/../Util/boost/linux/${dirOfFile}" ]
  then
    mkdir -p "${baseDir}/../Util/boost/linux/${dirOfFile}"
  fi
  sed -e 's/\r$//g' "${tmpdir}/${archiveDir}/include/${f}" > "${baseDir}/../Util/boost/linux/${f}"
done
echo "Done"

echo -n "Cleaning up..."
rm -r "${tmpdir}"
echo "Done"
