#!/bin/bash

if [ -z "${gtDir:-""}" ]; then
  echo "This script is only useful when included into other scripts"
  exit 125
fi

declare -a wiredRobotAddresses
declare -i numOfWiredRobotAddresses=0
declare -a wirelessRobotAddresses
declare -i numOfWirelessRobotAddresses=0
declare -i numOfRobotAddresses=0

wiredRobotAddresses=($(cat ${fileDir}/interfaces_* 2> ${stderr} | grep -A3 eth0 | grep address | awk '{print $2}' | sort -n ))
numOfWiredRobotAddresses=${#wiredRobotAddresses[@]}
wirelessRobotAddresses=($(cat ${fileDir}/interfaces_* 2> ${stderr} | grep -A3 wlan0 | grep address | awk '{print $2}' | sort -n ))
numOfWirelessRobotAddresses=${#wirelessRobotAddresses[@]}
if [ ${numOfWiredRobotAddresses} -ne ${numOfWirelessRobotAddresses} ]; then
  warning "Numbers of wireless and wired ip addresses differ"
  warning "Perfering wired ip addresses"
fi
numOfRobotAddresses=${numOfWiredRobotAddresses}
message "Found ${numOfRobotAddresses} ip address"

function sendCommandToAll()
{
  if [ -z "${1:-""}" ]; then
    fatal "No command given"
  fi
  command="${1}"
  for ((i=0;i<${numOfRobotAddresses};i++)); do
    wired="${wiredRobotAddresses[${i}]}"
    wireless="${wirelessRobotAddresses[${i}]}"
    message "Sending command to robot ${wired}/${wireless}"
    if ping -c1 "${wired}" >"${stdout}" 2>"${stderr}"; then
      message "Trying wired connection to ${wired}"
      ssh -i "${configDir}/Keys/id_rsa_nao" -o StrictHostKeyChecking=no root@"${wired}" "${command}" || echo "Failed"
    elif ping -c1 "${wireless}" >"${stdout}" 2>"${stderr}"; then
      message "Trying wireless connection to ${wireless}"
      ssh -i "${configDir}/Keys/id_rsa_nao" -o StrictHostKeyChecking=no root@"${wireless}" "${command}" || echo "Failed"
    else
      message "Robot ${wired}/${wireless} is not reachable"
    fi
  done
}
