from ConfigParser import SafeConfigParser
from util import *
import re
import functools

MAX_ROBOTCOUNT = 3

class TeamConfigParser(object):
  def __init__(self, defaults):
    self.__config = SafeConfigParser(defaults);
    self.__teams = None;
    self.__robotCounter = 0

  def read(self, filename):
    assert filename != None
    assert type(filename) == str
    self.__config.read(filename)

  def _getlist(self, section, option):
    return map(lambda x : x.strip(), self.__config.get(section, option).split(","))

  def _toList(self, string):
    return map(lambda x : x.strip(), string.split(","))

  def __fillRobots(self, robotDetails, attr):
    ret = dict()
    robotsTmp = list()
    count = 0
    for r in self._toList(attr):
      if robotDetails.has_key(r) or r == "_":
        count = (count % 3) + 1
        robotsTmp.append((count, r))
      else:
        error("Robot \"%s\" not defined" % r)
        sys.exit(1)
    autoNumber = any(map((lambda (n, r): r == '_' or robotDetails[r]["player"] == None), robotsTmp))
    for pn, rt in robotsTmp:
      if rt == "_":
        continue
      ret[rt] = robotDetails[rt]
      if autoNumber:
        info("Setting player number of %s to %i" % (rt, pn))
        ret[rt]["player"] = pn
    return ret

  def __makePort(self, teams, s):
    return int("10" + str(teams[s]["number"]) + "01")

  def __parseSections(self, secPrefix, inOut, attrs):
    for s, detail in inOut.iteritems():
      sec = secPrefix + s
      for a in attrs:
        if self.__config.has_option(sec, a[0]):
          detail[a[0]] = a[1](self.__config.get(sec, a[0]))
        elif len(a) == 3:
          if callable(a[2]):
            from inspect import getargspec
            argCount = 0
            if type(a[2]) == functools.partial:
              argCount = len(filter(lambda x : x != 'self', getargspec(a[2].func)[0])) - len(a[2].args)
            else:
              argCount = len(filter(lambda x : x != 'self', getargspec(a[2])[0]))
            if argCount == 0:
              detail[a[0]] = a[2]()
            elif argCount == 1:
              detail[a[0]] = a[2](s)
          else:
            detail[a[0]] = a[2]
          info("%s \"%s\" has no \"%s\", using default: \"%s\"" \
                  % (secPrefix, s, a[0], str(detail[a[0]])))
        else:
          error("%s \"%s\" has no \"%s\" and no default value!" % (secPrefix, s, a[0]))
          sys.exit(1)

  def getTeams(self):
    if self.__teams == None:
      sections = self.__config.sections()
      teams = dict()
      robots = dict()
      reTeam = re.compile("^Team:.*")
      reRobot = re.compile("^Robot:.*")

      # read which teams and robots are there
      for s in sections:
        if reTeam.match(s):
          teamName = s.split(":")[1].strip()
          teams[teamName] = dict()
        elif reRobot.match(s):
          robotName = s.split(":")[1].strip()
          robots[robotName] = dict()

      # read details of robots
      attrs = [
          ("ip", str),
          ("build", str),
          ("colorTable", str),
          ("player", int, None),
          ]
      self.__parseSections("Robot:", robots, attrs)

      # read details of teams
      from functools import partial
      attrs = [
          ("location", str),
          ("color", str),
          ("number" , int),
          ("port", int, partial(self.__makePort, teams)),
          ("robots", partial(self.__fillRobots, robots)),
          ("wlanConfig", str, None)
          ]
      self.__parseSections("Team:", teams, attrs)

      self.__teams = teams
    return self.__teams
