/**
* @file ControllerQt/Views/ViewBike/ViewBike.h
*
* Declaration of class ViewBike
*
* @author <a href="mailto:judy@tzi.de">Judith Mueller</a>
*/

#ifndef ViewBike_H
#define ViewBike_H

#include <QGLWidget>
#include <QWidget>
#include "Tools/Math/Vector2.h"
#include "Tools/Math/Vector3.h"

#include "Controller/DirectView.h"
#include "RobotConsole.h"
#include "OpenGL/ObjectRenderer.h"

#include "Representations/Configuration/JointCalibration.h"
#include "Representations/Configuration/RobotDimensions.h"
#include "Representations/MotionControl/MotionRequest.h"

class RobotConsole;
class QGLWidget;

class Point
{
public:
  Point(int x, int y) : x(x), y(y) {}
  Point() : x(0), y(0) {}
  int x;
  int y;
};

class ViewBike: public DirectView
{
public:
  /**
  * Constructor.
  *
  */
  ViewBike(RobotConsole& console, const MotionRequest& motionRequest, const JointData& jointData, const JointCalibration& jointCalibration, const RobotDimensions& robotDimensions, const std::string& mr, Simulation* simulation, std::string robot, SimObject* origin);

  RobotConsole& console;
  const MotionRequest& motionRequest;
  const JointData& jointData;
  const JointCalibration& jointCalibration;
  const RobotDimensions& robotDimensions;
  //const std::string& consoleText;
  const std::string& motionRequestCommand;
  ObjectRenderer renderer;
  QGLWidget* viewBikeWidget; /**< The Xabsl widget that displays the info. */
  SimObject* origin;

private:
  /**
  * The method returns a new instance of a widget for this direct view.
  * The caller has to delete this instance. (Qt handles this)
  * @return The widget.
  */
  QWidget* createWidget();

  /**
  * The function is called to let the direct view update the widget. Do so if necessary.
  * @param widget The widget
  */
  void updateWidget(QWidget& widget);
};

#endif //ViewBike_H


