/**
* @file ControllerQt/Views/ViewBike/ViewBikeWidget.cpp
*
* Implementation of class ViewBike
*
* @author <a href="mailto:judy@tzi.de">Judith Müller</a>
*/


#include "ViewBikeWidget.h"
#include "TabWidget.h"
#include "BikeMenuBar.h"
#include "Tools/Streams/InStreams.h"
#include "Platform/GTAssert.h"
#include "Tools/Math/Vector3.h"
#include "Tools/Math/Vector2.h"
#include "Tools/Math/Vector_n.h"
#include <vector>
#include <sstream>

#include <QGLWidget>
#include <QMouseEvent>
#include <QApplication>
#include <QKeyEvent>
#include <QHeaderView>
#include <QFileDialog>
#include <QGridLayout>
#include <QPalette>
#include <QColor>
#include <QPushButton>
#include <QLineEdit>
#include <QTreeView>
#include <QStandardItem>
#include <QTabWidget>
#include <QObject>
#include <QContextMenuEvent>
#include <QLabel>
#include <QCheckBox>

ViewBikeWidget::ViewBikeWidget(ViewBike& viewBike, QWidget* parent) : 
QGLWidget(parent),
viewBike(viewBike), 
phaseDrawings(true),
singleDraw(false),
reachedDraw(false),
tra2dWindows(false), 
tra1dWindows(false),
velocityWindows(false),
accelWindows(false),
getString(4),
ghost(0),
dragPlane(0,1,0),
moveDrag(false),
moveViewOfViewDragXY(false),
moveViewOfViewDragXZ(false),
moveViewOfViewDragYZ(false),
moveViewOfViewDragXP(false),
moveViewOfViewDragYP(false),
moveViewOfViewDragZP(false),
mirror(false)
{
  //Vertical Layouts
  QVBoxLayout *vbox = new QVBoxLayout(this);
  QVBoxLayout *vbox2 = new QVBoxLayout();
  QVBoxLayout *vbox3 = new QVBoxLayout();

  //horizontal Layouts
  QHBoxLayout *hbox = new QHBoxLayout();
  QHBoxLayout *hbox2 = new QHBoxLayout();

  QCheckBox *mirrorCB = new QCheckBox("Mirror", this);
  mirrorCB->setCheckState(Qt::Unchecked);
  //make the Buttons
  QPushButton* play = new QPushButton("Play Motion", this);
  QPushButton* play2 = new QPushButton("Play Motion until Active", this);
  QPushButton* reset = new QPushButton("Reset Robot", this);
  QPushButton* stop = new QPushButton("Stand", this);
  QPushButton* undoStep = new QPushButton("Undo", this);
  QPushButton* redoStep = new QPushButton("Redo", this);

  save = new QPushButton("Save", this);
  saveAs = new QPushButton("Save as...", this);
  load = new QPushButton("Load", this);
  newMove = new QPushButton("New", this);
  drawings = new QPushButton("Hide Drawings", this);
  singleDrawing = new QPushButton("Single Drawing");
  editor = new QPushButton("Hide Editor", this);
  tra2d = new QPushButton("Show 2D", this);
  tra1d = new QPushButton("Show 1D", this);
  velocity = new QPushButton("Show Velocity", this);
  accel = new QPushButton("Show Acceleration", this);
  addPhase = new QPushButton("Add Phase", this);
  deletePhase = new QPushButton("Delete Phase", this);

  QSlider* slider1 = new QSlider(Qt::Horizontal, this);
  QLabel* label1 = new QLabel(tr("<font color=white size=2>Robot Opacity </font>"), this);

  //Set Sliderrange
  QPalette pal = slider1->palette();
  pal.setColor(QPalette::Background, Qt::darkGray);
  slider1->setPalette(pal);
  slider1->setAutoFillBackground(true);
  slider1->setRange(0, 16);

  pal = label1->palette();
  pal.setColor(QPalette::Background, Qt::darkGray);
  label1->setPalette(pal);
  label1->setAutoFillBackground(true);

  mirrorCB->setAutoFillBackground(true);
  play->setAutoFillBackground(true);
  play2->setAutoFillBackground(true);
  reset->setAutoFillBackground(true);
  stop->setAutoFillBackground(true);
  save->setAutoFillBackground(true);
  saveAs->setAutoFillBackground(true);
  load->setAutoFillBackground(true);
  newMove->setAutoFillBackground(true);
  undoStep->setAutoFillBackground(true);
  redoStep->setAutoFillBackground(true);

  drawings->setAutoFillBackground(true);
  singleDrawing->setAutoFillBackground(true);
  editor->setAutoFillBackground(true);
  tra2d->setAutoFillBackground(true);
  tra1d->setAutoFillBackground(true);
  velocity->setAutoFillBackground(true);
  accel->setAutoFillBackground(true);
  addPhase->setAutoFillBackground(true);
  deletePhase->setAutoFillBackground(true);

  hbox->addWidget(newMove, 0, Qt::AlignLeft);
  hbox->addWidget(load, 0, Qt::AlignLeft);
  hbox->addWidget(save, 0, Qt::AlignLeft); 
  hbox->addWidget(saveAs, 1, Qt::AlignLeft);
  hbox->addWidget(mirrorCB, 1, Qt::AlignRight);
  hbox->addWidget(play, 0, Qt::AlignRight);
  hbox->addWidget(play2, 0, Qt::AlignRight);
  hbox->addWidget(reset, 0, Qt::AlignRight);
  hbox->addWidget(stop, 0, Qt::AlignRight);

  vbox3->addWidget(singleDrawing, 0, Qt::AlignTop);
  /*vbox3->addWidget(reachedDrawing, 0, Qt::AlignTop);*/
  vbox3->addWidget(drawings, 0, Qt::AlignTop);
  vbox3->addWidget(editor, 0, Qt::AlignTop);
  vbox3->addWidget(label1,0,Qt::AlignTop);
  vbox3->addWidget(slider1,0,Qt::AlignTop);
  vbox3->addWidget(tra1d, 0, Qt::AlignTop);
  vbox3->addWidget(tra2d, 0, Qt::AlignTop);
  vbox3->addWidget(velocity, 0, Qt::AlignTop);
  vbox3->addWidget(accel, 1, Qt::AlignTop);

  vbox3->addWidget(addPhase, 1, Qt::AlignBottom);
  vbox3->addWidget(deletePhase, 0, Qt::AlignBottom);

  vbox3->addWidget(undoStep, 0, Qt::AlignTop);
  vbox3->addWidget(redoStep, 0, Qt::AlignTop);
  //Make the tabs
  tabber = new TabWidget(this);
  tabber->setAutoFillBackground(true);

  QPalette palette = tabber->palette();
  palette.setColor(QPalette::Background, Qt::darkGray);
  tabber->setPalette(palette);

  treeViewCommon = new QTreeView(this);
  treeViewCommon->setDragDropMode(QTreeView::DragOnly);
  treeViewCommon->setSelectionMode(QTreeView::ExtendedSelection);
  treeViewCommon->header()->setVisible(true);
  treeViewCommon->setRootIsDecorated(true);
  treeViewCommon->setDragEnabled(false);
  treeViewCommon->setAcceptDrops(false);

  //initialize Tabs with some working stuff
  modelCommon = new QStandardItemModel();

  QStandardItem *parentItem = modelCommon->invisibleRootItem();

  QList <QStandardItem*> nameLabels;
  nameLabels.append(makeLabel(QString("Name")));
  nameLabels.append(makeLabel(QString("NoName")));
  parentItem->appendRow(nameLabels);

  QList <QStandardItem*> phaseNumber;
  phaseNumber.append(makeLabel(QString("Number of Phases")));
  phaseNumber.append(makeLabel(QString::number(0)));
  parentItem->appendRow(phaseNumber);

  QStandardItem* footOrigin = makeLabel(QString("Foot Origin"));
  parentItem->appendRow(addXYZLabel(footOrigin));

  QList <QStandardItem*> list2;
  list2.append(makeLabel(QString("")));
  list2.append(makeLabel(QString("")));
  list2.append(makeValue((float) 0.f));
  list2.append(makeValue((float) 50.f));
  list2.append(makeValue((float) -210.f));
  footOrigin->appendRow(list2);

  QStandardItem* armOrigin = makeLabel(QString("Arm Origin"));
  parentItem->appendRow(addXYZLabel(armOrigin));

  QList <QStandardItem*> list3;
  list3.append(makeLabel(QString("")));
  list3.append(makeLabel(QString("")));
  list3.append(makeValue((float) 0.f));
  list3.append(makeValue((float) 100.f));
  list3.append(makeValue((float) 30.f));
  armOrigin->appendRow(list3);

  QStandardItem* pidx = makeLabel(QString("COM Balance X"));
  QList <QStandardItem*> pidxlist;
  pidxlist.append(pidx);
  pidxlist.append(makeLabel(QString("")));
  pidxlist.append(makeLabel(QString("p")));
  pidxlist.append(makeLabel(QString("i")));
  pidxlist.append(makeLabel(QString("d")));
  parentItem->appendRow(pidxlist);
  QList <QStandardItem*> list4;
  list4.append(makeLabel(QString("")));
  list4.append(makeLabel(QString("")));
  list4.append(makeValue((float) 0.f));
  list4.append(makeValue((float) 0.f));
  list4.append(makeValue((float) 0.f));
  pidx->appendRow(list4);

  QStandardItem* pidy = makeLabel(QString("COM Balance Y"));
  QList <QStandardItem*> pidylist;
  pidylist.append(pidy);
  pidylist.append(makeLabel(QString("")));
  pidylist.append(makeLabel(QString("p")));
  pidylist.append(makeLabel(QString("i")));
  pidylist.append(makeLabel(QString("d")));
  parentItem->appendRow(pidylist);
  QList <QStandardItem*> list5;
  list5.append(makeLabel(QString("")));
  list5.append(makeLabel(QString("")));
  list5.append(makeValue((float) 0.f));
  list5.append(makeValue((float) 0.f));
  list5.append(makeValue((float) 0.f));
  pidy->appendRow(list5);

  parentItem->appendRow(makeStringAndValueRow(QString("Preview (in Frames)"), (int) 0));

  parentItem->appendRow(makeStringAndValueRow(QString("Loop"), (bool) 0));

  treeViewCommon->setModel(modelCommon);
  modelCommon->setColumnCount(5);
  modelCommon->setHeaderData(0, Qt::Horizontal,  QVariant(""));
  modelCommon->setHeaderData(1, Qt::Horizontal,  QVariant(""));
  modelCommon->setHeaderData(2, Qt::Horizontal,  QVariant(""));
  modelCommon->setHeaderData(3, Qt::Horizontal,  QVariant(""));
  modelCommon->setHeaderData(4, Qt::Horizontal,  QVariant(""));

  treeViewCommon->header()->setResizeMode(0, QHeaderView::ResizeToContents);
  treeViewCommon->header()->setResizeMode(1, QHeaderView::Stretch);
  treeViewCommon->header()->setResizeMode(2, QHeaderView::Stretch);
  treeViewCommon->header()->setResizeMode(3, QHeaderView::Stretch);
  treeViewCommon->header()->setResizeMode(4, QHeaderView::Stretch);

  tabber->addTab(treeViewCommon, "Common");

  hbox2->addWidget(tabber,1);
  hbox2->addLayout(vbox3);
  hbox2->addStretch(2);

  vbox2->addStretch(1);
  hbox2->addLayout(vbox2);
  vbox->addLayout(hbox2);
  vbox->addLayout(hbox);

  treeViewCommon->expandAll();
  treeViewCommon->header()->setResizeMode(0, QHeaderView::ResizeToContents);

  bikeMenuBar = new BikeMenuBar(this);
  bikeMenuBar->hide();

  QMenu shortcutMenu;
  shortcutMenu.addMenu(bikeMenuBar->dragPlaneMenu);
  shortcutMenu.hide();

  //Connect Buttons and Stuff with 
  connect(newMove, SIGNAL(clicked()), this ,SLOT(newButtonClicked()));
  connect(load, SIGNAL(clicked()), this ,SLOT(loadButtonClicked()));
  connect(saveAs, SIGNAL(clicked()), this ,SLOT(saveAsButtonClicked()));
  connect(save, SIGNAL(clicked()), this ,SLOT(saveButtonClicked()));
  connect(drawings, SIGNAL(clicked()), this, SLOT(setDrawings()));
  connect(singleDrawing, SIGNAL(clicked()), this, SLOT(setSingleDrawing()));
  connect(editor, SIGNAL(clicked()), this, SLOT(setEditor()));
  connect(tra2d, SIGNAL(clicked()), this, SLOT(setTra2d()));
  connect(tra1d, SIGNAL(clicked()), this, SLOT(setTra1d()));
  connect(velocity, SIGNAL(clicked()), this, SLOT(setVelocity()));
  connect(accel, SIGNAL(clicked()), this, SLOT(setAccel()));
  connect(slider1, SIGNAL(valueChanged(int)), this, SLOT(transparencyChanged(int)));
  connect(deletePhase, SIGNAL(clicked()), this, SLOT(removePhase()));
  connect(addPhase, SIGNAL(clicked()), this, SLOT(addPhaseAfterActual()));
  connect(mirrorCB, SIGNAL(stateChanged(int)), this, SLOT(setMirrored(int)));
  connect(play, SIGNAL(clicked()), this, SLOT(playWholeMotion()));
  connect(reset, SIGNAL(clicked()), this, SLOT(resetRobot()));
  connect(stop, SIGNAL(clicked()), this, SLOT(standRobot()));
  connect(play2, SIGNAL(clicked()), this, SLOT(playMotionTilActive()));
  connect(undoStep, SIGNAL(clicked()), this, SLOT(undoChanges()));
  connect(redoStep, SIGNAL(clicked()), this, SLOT(redoChanges()));

  connect(modelCommon, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updateCommonParameters(QStandardItem*)));
  connect(tabber->getTabBar(), SIGNAL(tabMoveRequested(int, int)), this, SLOT(movePhase(int, int)));
  connect(tabber, SIGNAL(currentChanged(int)), this, SLOT(setSelectedFromEditor(int)));

  connect(bikeMenuBar->splitPhase, SIGNAL(triggered()), this, SLOT(splitPhase()));

  dragPlaneMapper.setMapping(bikeMenuBar->xy_plane, XY_PLANE);
  dragPlaneMapper.setMapping(bikeMenuBar->xz_plane, XZ_PLANE);
  dragPlaneMapper.setMapping(bikeMenuBar->yz_plane, YZ_PLANE);
  connect(&dragPlaneMapper, SIGNAL(mapped(int)), SLOT(setDragPlane(int)));

  connect(bikeMenuBar->xy_plane, SIGNAL(triggered()), &dragPlaneMapper, SLOT(map()));
  connect(bikeMenuBar->xz_plane, SIGNAL(triggered()), &dragPlaneMapper, SLOT(map()));
  connect(bikeMenuBar->yz_plane, SIGNAL(triggered()), &dragPlaneMapper, SLOT(map()));

  qobject_cast<QAction*>(dragPlaneMapper.mapping(XZ_PLANE))->setChecked(true);

  //initialize actual Parameters
  parameters.footOrigin = Vector3 <>(0.f,60.f,-210.f);
  parameters.armOrigin = Vector3 <>(0.f,100.f,30.f);
  parameters.numberOfPhases = 0;
  parameters.loop = false;
  parameters.kpx = 0.f;
  parameters.kdx = 0.f;
  parameters.kix = 0.f;
  parameters.kpy = 0.f;
  parameters.kdy = 0.f;
  parameters.kiy = 0.f;
  parameters.preview = 150;
  strcpy(parameters.name, "noName");

  selectedPoint.limb = -1;
  selectedPoint.phaseNumber = -1;
  selectedPoint.xzRot = false;

  GLubyte test[17][128] = 
  {
    {0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

    {0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00},

    {0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0x88, 0x88, 0x88, 0x88, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00},

    {0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0x22, 0x22, 0x22, 0x22, 0x00, 0x00, 0x00, 0x00},

    {0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00},

    {0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x00, 0x00, 0x00, 0x00},

    {0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x44, 0x44, 0x44, 0x44, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11},

    {0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x11, 0x11, 0x11, 0x11},

    {0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55},

    {0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xaa, 0xaa, 0xaa, 0xaa, 0x55, 0x55, 0x55, 0x55},

    {0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xee, 0xee, 0xee, 0xee, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55},

    {0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xbb, 0xbb, 0xbb, 0xbb, 0x55, 0x55, 0x55, 0x55},

    {0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55},

    {0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x55, 0x55, 0x55, 0x55},

    {0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xdd, 0xdd, 0xdd, 0xdd, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77},

    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x77, 0x77, 0x77, 0x77},

    {0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff,
    0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff}
  };
  for (int i =0; i < 17; i++)
  {
    for (int k=0; k < 128; k++)
    {
      stippleMask[i][k] = test[i][k];
    }
  }

  if (!commands.empty()) commands.clear();
}

void ViewBikeWidget::setDragPlane(const int& plane)
{

  switch(plane)
  {
  case XY_PLANE:
    dragPlane = Vector3d(0,0,1);
    break;
  case XZ_PLANE:
    dragPlane = Vector3d(0,1,0);
    break;
  case YZ_PLANE:
    dragPlane = Vector3d(1,0,0); 
    break;
  default:
    break;
  }
  qobject_cast<QAction*>(dragPlaneMapper.mapping(plane))->setChecked(true);
}

void ViewBikeWidget::removePhase()
{
  int phaseNumber = tabber->currentIndex()-1;

  if (phaseNumber > -1)
  {
    undo.push_back(parameters);
    if(undo.size() >= 20)
    {
      undo.erase(undo.begin());
    }

    std::vector<Phase>::iterator it;
    it = parameters.phaseParameters.begin() + phaseNumber;
    if (it != parameters.phaseParameters.end()) 
    {
      disconnect(*(phaseTab.begin()+phaseNumber), SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));
      tabber->setCurrentIndex(phaseNumber);
      tabber->removeTab(phaseNumber+1);
      for (int i=phaseNumber+1; i<tabber->count(); i++)
      {
        tabber->setTabText(i, QString("Phase %1").arg(i-1));
      }
      deleteKids(phaseTab[phaseNumber]->invisibleRootItem());
      delete *(treeView.begin()+phaseNumber);
      delete *(phaseTab.begin()+phaseNumber);
      treeView.erase(treeView.begin()+phaseNumber);
      phaseTab.erase(phaseTab.begin()+phaseNumber);   
      parameters.phaseParameters.erase(it);
      parameters.numberOfPhases-=1;

      for (int limb=0; limb < Phase::numOfLimbs; limb++)
      {
        if (phaseNumber <  parameters.numberOfPhases-1)
        {
          float factor =  (float)parameters.phaseParameters[phaseNumber].duration /  
            (float)parameters.phaseParameters[phaseNumber+1].duration;

          parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] = 
            parameters.phaseParameters[phaseNumber].controlPoints[limb][2] - 
            parameters.phaseParameters[phaseNumber].controlPoints[limb][1];

          parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] *= factor; 

          parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] += 
            parameters.phaseParameters[phaseNumber].controlPoints[limb][2];
        }
      }

      parameters.init();
      disconnect(modelCommon, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updateCommonParameters(QStandardItem*)));
      modelCommon->invisibleRootItem()->child(1,1)->setData(QString::number(parameters.numberOfPhases), Qt::DisplayRole);
      updateCommon();
    }
  }
}

void ViewBikeWidget::movePhase(const int fromIndex, int toIndex)
{ 
  if (parameters.phaseParameters.begin() + fromIndex-1 != parameters.phaseParameters.end()) 
  {
    toIndex = (fromIndex < toIndex)? toIndex-1 : toIndex;

    QStandardItemModel* temp = phaseTab[fromIndex-1];
    phaseTab.erase(phaseTab.begin()+fromIndex-1);
    phaseTab.insert(phaseTab.begin()+toIndex-1, temp);

    Phase object = parameters.phaseParameters[fromIndex-1];
    parameters.phaseParameters.erase(parameters.phaseParameters.begin() + fromIndex-1);
    parameters.phaseParameters.insert(parameters.phaseParameters.begin()+toIndex-1, object);
    parameters.init();
    updateEditorView();
    tabber->setCurrentIndex(toIndex);
  }
}

void ViewBikeWidget::makeNewPhaseWithModelAndTree(const int& phaseNumber)
{
  Phase newPhase;

  parameters.numberOfPhases++;
  disconnect(modelCommon, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updateCommonParameters(QStandardItem*)));
  modelCommon->invisibleRootItem()->child(1,1)->setData(QString::number(parameters.numberOfPhases), Qt::DisplayRole);
  updateCommon();


  newPhase.duration = 1000;
  for (int limb = 0; limb < Phase::numOfLimbs; limb++)
  {
    for (int point=0; point < NUM_OF_POINTS; point++)
    {
      newPhase.controlPoints[limb][point] = Vector3 <> (0.f,0.f,0.f);
    }
  } 

  parameters.phaseParameters.insert(parameters.phaseParameters.begin()+phaseNumber, newPhase);


  QStandardItemModel* model = makeNewModelWithLabels();
  phaseTab.insert(phaseTab.begin()+phaseNumber, model);
  fillModelWithPhaseData(phaseTab[phaseNumber]->invisibleRootItem(), phaseNumber);

  QTreeView* newTree = new QTreeView();
  newTree->setModel(phaseTab[phaseNumber]);
  newTree->setDragDropMode(QTreeView::DragOnly);
  newTree->setSelectionMode(QTreeView::ExtendedSelection);
  newTree->header()->setVisible(true);
  newTree->setRootIsDecorated(true);
  newTree->setDragEnabled(false);
  newTree->setAcceptDrops(false);
  newTree->expandToDepth(3);
  newTree->header()->setResizeMode(0, QHeaderView::ResizeToContents);
  newTree->header()->setResizeMode(1, QHeaderView::Stretch);
  newTree->header()->setResizeMode(2, QHeaderView::Stretch);
  newTree->header()->setResizeMode(3, QHeaderView::Stretch);
  newTree->header()->setResizeMode(4, QHeaderView::Stretch);
  newTree->header()->setResizeMode(5, QHeaderView::Stretch);
  treeView.insert(treeView.begin()+phaseNumber, newTree);

}
void ViewBikeWidget::addPhaseAfterActual()
{
  int phaseNumber = tabber->currentIndex(); // we want to insert after actual Phase

  undo.push_back(parameters);
  if(undo.size() >= 20)
  {
    undo.erase(undo.begin());
  }

  makeNewPhaseWithModelAndTree(phaseNumber);
  //connect(phaseTab[phaseNumber], SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));
  tabber->insertTab(phaseNumber+1, treeView[phaseNumber], QString("new"));

  if (phaseNumber > 0) //if there is a phase before copy last parameters
  {
    for (int limb=0; limb < Phase::numOfLimbs; limb++)
    {
      //Set all Points from new Phase to the End of last Phase
      for (int point = 0; point < (int)NUM_OF_POINTS; point++)
      {
        parameters.phaseParameters[phaseNumber].controlPoints[limb][point] = parameters.phaseParameters[phaseNumber-1].controlPoints[limb][NUM_OF_POINTS-1];
      }

      //correct controlPoints 
      // äquidistant und kolinear p3-p2 = q1-q0 
      //p3 = q0 => p3 - p2 = q1 - p3 => 2*p3 - p2 = q1;

      //q1
      if (phaseNumber <  parameters.numberOfPhases-1)
      {
        float factor =  (float)parameters.phaseParameters[phaseNumber].duration /  
          (float)parameters.phaseParameters[phaseNumber+1].duration;

        parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] = 
          parameters.phaseParameters[phaseNumber].controlPoints[limb][2] - 
          parameters.phaseParameters[phaseNumber].controlPoints[limb][1];

        parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] *= factor; 

        parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] += 
          parameters.phaseParameters[phaseNumber].controlPoints[limb][2];
      }
      //p1
      parameters.phaseParameters[phaseNumber].controlPoints[limb][0] = 
        parameters.phaseParameters[phaseNumber-1].controlPoints[limb][2] - 
        parameters.phaseParameters[phaseNumber-1].controlPoints[limb][1];

      float factor =  (float)parameters.phaseParameters[phaseNumber].duration /  
        (float)parameters.phaseParameters[phaseNumber-1].duration;

      parameters.phaseParameters[phaseNumber].controlPoints[limb][0] *= factor; 

      parameters.phaseParameters[phaseNumber].controlPoints[limb][0] += 
        parameters.phaseParameters[phaseNumber-1].controlPoints[limb][2];
    }
  }
  parameters.init();
  updateEditorView();  
  tabber->setCurrentIndex(phaseNumber+1);

}

std::string ViewBikeWidget::floatToStr(const float& f)
{
  char temp[100];
  sprintf(temp,"%.3f",f);
  return temp;
}

std::string ViewBikeWidget::intToStr(const int& i)
{
  char temp[100];
  sprintf(temp,"%i",i);
  return temp;
}

std::string ViewBikeWidget::boolToStr(const bool& b)
{
  if (b)
    return "true";
  else
    return "false";
}

void ViewBikeWidget::setMirrored(int state) {
  mirror = (state == Qt::Checked);
}

void ViewBikeWidget::playWholeMotion()
{
  if (parameters.numberOfPhases > 0)  playMotion(parameters.numberOfPhases);
}

void ViewBikeWidget::playMotionTilActive()
{
  int numberOfPhases = tabber->currentIndex();
  if (numberOfPhases > 0)  playMotion(numberOfPhases);
}

void ViewBikeWidget::playMotion(int phase)
{
  std::string setNewBMotion = "set module:BIKE:newBMotion { footOrigin = { x = "
    + floatToStr(parameters.footOrigin.x) +"; y = "
    + floatToStr(parameters.footOrigin.y) +"; z = "
    + floatToStr(parameters.footOrigin.z) +"; }; armOrigin = { x = "
    + floatToStr(parameters.armOrigin.x)  +"; y = "
    + floatToStr(parameters.armOrigin.y)  +"; z = "
    + floatToStr(parameters.armOrigin.z)  +"; }; kpx = "
    + floatToStr(parameters.kpx)         +"; kix = "
    + floatToStr(parameters.kix)         +"; kdx = "
    + floatToStr(parameters.kdx)         +"; kpy = "
    + floatToStr(parameters.kpy)         +"; kiy = "
    + floatToStr(parameters.kiy)         +"; kdy = "
    + floatToStr(parameters.kdy)         +"; preview = "
    + floatToStr(parameters.preview)     +"; loop = "
    + boolToStr(parameters.loop)         +"; phaseParameters["+ intToStr(phase) +"] = {";

  for (int i=0; i<phase; i++)
  {
    setNewBMotion += " { duration = "
      + intToStr(parameters.phaseParameters[i].duration) +"; controlPoints[Phase::leftFootTra][3] = { { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][0].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][0].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][0].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][1].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][1].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][1].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][2].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][2].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][2].z)+"; } }; controlPoints[Phase::leftFootRot][3] = { { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][0].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][0].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][0].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][1].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][1].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][1].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][2].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][2].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][2].z)+"; } }; controlPoints[Phase::rightFootTra][3] = { { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][0].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][0].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][0].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][1].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][1].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][1].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][2].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][2].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][2].z)+"; } }; controlPoints[Phase::rightFootRot][3] = { { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][0].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][0].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][0].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][1].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][1].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][1].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][2].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][2].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][2].z)+"; } }; controlPoints[Phase::leftArmTra][3] = { { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][0].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][0].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][0].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][1].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][1].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][1].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][2].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][2].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][2].z)+"; } }; controlPoints[Phase::rightArmTra][3] = { { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][0].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][0].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][0].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][1].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][1].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][1].z)+"; } { x = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][2].x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][2].y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][2].z)+"; } }; odometryOffset = { x = "
      + floatToStr(parameters.phaseParameters[i].odometryOffset.x)+"; y = "
      + floatToStr(parameters.phaseParameters[i].odometryOffset.y)+"; z = "
      + floatToStr(parameters.phaseParameters[i].odometryOffset.z)+"; }; }";
  }

  setNewBMotion += " }; supportLegMode = auto; }";
  commands.push_back(setNewBMotion);

  std::stringstream todo(std::stringstream::in | std::stringstream::out);
  int idxMotion = viewBike.motionRequestCommand.find("motion");
  int idxSpecialAction = viewBike.motionRequestCommand.find("specialActionRequest", idxMotion);
  int idxBMotion = viewBike.motionRequestCommand.find("bMotionType", idxSpecialAction);
  int idxDynamical = viewBike.motionRequestCommand.find("dynamical", idxBMotion);
  todo << viewBike.motionRequestCommand.substr(0, idxMotion + 9) << "bike; ";
  todo << viewBike.motionRequestCommand.substr(idxSpecialAction, idxBMotion + 14 - idxSpecialAction) << "noName; mirror = ";
  todo << (mirror ? "true" : "false") << "; " << viewBike.motionRequestCommand.substr(idxDynamical); 
  commands.push_back(todo.str());

}


void ViewBikeWidget::resetRobot()
{
  std::string moveRobot = "mv 0 0 300 0 0 180";
  commands.push_back(moveRobot);
  commands.push_back(" ");
}

void ViewBikeWidget::standRobot()
{
  std::string command = viewBike.motionRequestCommand;
  std::string firstOfcom = command.substr(0,command.find("motion")+ 9);
  std::string temp = command.substr(command.find("motion")+ 10, command.size()-command.find("motion")+ 10);
  std::string endOfcom = temp.substr(temp.find_first_of(";"), temp.size());
  std::string stand = firstOfcom + "stand" + endOfcom;
  commands.push_back(stand);
  commands.push_back(" "); 
}

void ViewBikeWidget::setSelectedFromEditor(const int& index)
{
  selectedPoint.phaseNumber = index-1;
}

void ViewBikeWidget::setDrawings()
{
  if(!phaseDrawings)
  { 
    phaseDrawings = true;
    drawings->setText("Hide Drawings");
  }
  else
  {
    phaseDrawings = false;
    drawings->setText("Show Drawings");
  }
}

void ViewBikeWidget::setSingleDrawing()
{
  if(!singleDraw)
  { 
    singleDraw = true;
    singleDrawing->setText(" All Drawings  ");
  }
  else
  {
    singleDraw = false;
    singleDrawing->setText("Single Drawing");
  }
}
void ViewBikeWidget::setReachedDrawing()
{
  if(!reachedDraw)
  { 
    reachedDraw = true;
    reachedDrawing->setText("Hide Reached");
  }
  else
  {
    reachedDraw = false;
    reachedDrawing->setText("Show Reached");
  }
}

void ViewBikeWidget::setTra2d()
{
  if(!tra2dWindows)
  { 
    tra2dWindows = true;
    tra2d->setText("Hide 2D");
    if(tra1dWindows)
    {
      tra1dWindows = false;
      tra1d->setText("Show 1D");
    }
    if(accelWindows)
    {
      accelWindows = false;
      accel->setText("Show Acceleration");
    }
    if(velocityWindows)
    {
      velocityWindows = false;
      velocity->setText("Show Velocity");
    }
  }
  else
  {
    tra2dWindows = false;
    tra2d->setText("Show 2D");
  }
}
void ViewBikeWidget::setTra1d()
{
  if(!tra1dWindows)
  { 
    tra1dWindows = true;
    tra1d->setText("Hide 1D");
    if(tra2dWindows)
    {
      tra2dWindows = false;
      tra2d->setText("Show 2D");
    }
    if(accelWindows)
    {
      accelWindows = false;
      accel->setText("Show Acceleration");
    }
    if(velocityWindows)
    {
      velocityWindows = false;
      velocity->setText("Show Velocity");
    }
  }
  else
  {
    tra1dWindows = false;
    tra1d->setText("Show 1D");
  }

}

void ViewBikeWidget::setVelocity()
{
  if(!velocityWindows)
  { 
    velocityWindows = true;
    velocity->setText("Hide Velocity");
    if(tra2dWindows)
    {
      tra2dWindows = false;
      tra2d->setText("Show 2D");
    }
    if(tra1dWindows)
    {
      tra1dWindows = false;
      tra1d->setText("Show 1D");
    }
    if(accelWindows)
    {
      accelWindows = false;
      accel->setText("Show Acceleration");
    }
  }
  else
  {
    velocityWindows = false;
    velocity->setText("Show Velocity");
  }
}

void ViewBikeWidget::setAccel()
{
  if(!accelWindows)
  { 
    accelWindows = true;
    accel->setText("Hide Acceleration");
    if(tra2dWindows)
    {
      tra2dWindows = false;
      tra2d->setText("Show 2D");
    }
    if(tra1dWindows)
    {
      tra1dWindows = false;
      tra1d->setText("Show 1D");
    }
    if(velocityWindows)
    {
      velocityWindows = false;
      velocity->setText("Show Velocity");
    }
  }
  else
  {
    accelWindows = false;
    accel->setText("Show Acceleration");
  }

}

void ViewBikeWidget::undoChanges()
{

  if (!undo.empty())
  {
    BIKEParameters last;
    last = undo.back();
    undo.pop_back();
    redo.push_back(parameters);

    if(redo.size() >= 20) redo.erase(redo.begin());
    if (parameters.numberOfPhases == last.numberOfPhases)
    {
      parameters = last;
      updateCommon();
      for (int i=0; i < parameters.numberOfPhases; i++)
      {
        fillModelWithPhaseData(phaseTab[i]->invisibleRootItem(),i);
        treeView[i]->expandToDepth(3);
      }
    }
    else
    {
      parameters = last;
      updateEditorView();
    }

    updateGL();
  }

}


void ViewBikeWidget::redoChanges()
{

  if (!redo.empty())
  {
    BIKEParameters last;
    last = redo.back();
    redo.pop_back();

    undo.push_back(parameters);
    if(undo.size() >= 20) undo.erase(undo.begin());

    if (parameters.numberOfPhases == last.numberOfPhases)
    {
      parameters = last;
      updateCommon();
      for (int i=0; i < parameters.numberOfPhases; i++)
      {
        fillModelWithPhaseData(phaseTab[i]->invisibleRootItem(),i);
        treeView[i]->expandToDepth(3);
      }
    }
    else
    {
      parameters = last;
      updateEditorView();
    }
    updateGL();
  }

}


void ViewBikeWidget::transparencyChanged(const int& i)
{
  ghost = i;
}

void ViewBikeWidget::setEditor()
{
  if(tabber->isHidden())
  { 
    tabber->setHidden(false);
    newMove->setHidden(false);
    load->setHidden(false);
    save->setHidden(false);
    saveAs->setHidden(false);
    editor->setText("Hide Editor");
    deletePhase->setHidden(false);
    addPhase->setHidden(false);

  }
  else
  {
    tabber->hide();
    newMove->hide();
    load->hide();
    saveAs->hide();
    save->hide();
    deletePhase->hide();
    addPhase->hide();
    editor->setText("Show Editor");
  }
}

void ViewBikeWidget::updateCommon()
{
  disconnect(modelCommon, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updateCommonParameters(QStandardItem*)));
  QStandardItem *rootItem  = modelCommon->invisibleRootItem();
  rootItem->child(0,1)->setData(QString(parameters.name), Qt::DisplayRole);
  rootItem->child(1,1)->setData(QString::number(parameters.numberOfPhases), Qt::DisplayRole);

  rootItem->child(2,0)->child(0,2)->setData(QVariant(parameters.footOrigin.x), Qt::DisplayRole);
  rootItem->child(2,0)->child(0,3)->setData(QVariant(parameters.footOrigin.y), Qt::DisplayRole);
  rootItem->child(2,0)->child(0,4)->setData(QVariant(parameters.footOrigin.z), Qt::DisplayRole);

  rootItem->child(3,0)->child(0,2)->setData(QVariant(parameters.armOrigin.x), Qt::DisplayRole);
  rootItem->child(3,0)->child(0,3)->setData(QVariant(parameters.armOrigin.y), Qt::DisplayRole);
  rootItem->child(3,0)->child(0,4)->setData(QVariant(parameters.armOrigin.z), Qt::DisplayRole);

  rootItem->child(4,0)->child(0,2)->setData(QVariant(parameters.kpx), Qt::DisplayRole);
  rootItem->child(4,0)->child(0,3)->setData(QVariant(parameters.kix), Qt::DisplayRole);
  rootItem->child(4,0)->child(0,4)->setData(QVariant(parameters.kdx), Qt::DisplayRole);

  rootItem->child(5,0)->child(0,2)->setData(QVariant(parameters.kpy), Qt::DisplayRole);
  rootItem->child(5,0)->child(0,3)->setData(QVariant(parameters.kiy), Qt::DisplayRole);
  rootItem->child(5,0)->child(0,4)->setData(QVariant(parameters.kdy), Qt::DisplayRole);

  rootItem->child(6,1)->setData(QVariant(parameters.preview), Qt::DisplayRole);
  rootItem->child(7,1)->setData(QVariant(parameters.loop), Qt::DisplayRole);

  connect(modelCommon, SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updateCommonParameters(QStandardItem*)));
}


void ViewBikeWidget::updateEditorView()
{
  //Change Common Tab

  updateCommon();

  //Clear and Destroy and Resize
  for (int i=0; i<tabber->count(); i++)
  {
    tabber->removeTab(i);
  }
  //this->repaint();
  tabber->addTab(treeViewCommon, "Common");
  treeViewCommon->expandAll();
  std::vector<QTreeView*>::iterator itTree;
  std::vector<QStandardItemModel*>::iterator itModel;

  if (!treeView.empty())
  {

    for (itTree = treeView.begin(); itTree < treeView.end(); itTree++)
    {
      delete *itTree;
    }
    treeView.clear();
  }

  if (!phaseTab.empty())
  {
    for (itModel = phaseTab.begin(); itModel < phaseTab.end(); itModel++)
    {
      deleteKids(((QStandardItemModel*)(*itModel))->invisibleRootItem());
      disconnect(((QStandardItemModel*)(*itModel)), SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));
      delete *itModel;
    }
    phaseTab.clear();
  }


  treeView.resize(parameters.numberOfPhases);
  phaseTab.resize(parameters.numberOfPhases);

  //readIn possible Phases
  for (int i = 0; i < parameters.numberOfPhases; i++)
  {
    treeView[i] = new QTreeView();
    treeView[i]->setDragDropMode(QTreeView::DragOnly);
    treeView[i]->setSelectionMode(QTreeView::ExtendedSelection);
    treeView[i]->header()->setVisible(true);
    treeView[i]->setRootIsDecorated(true);
    phaseTab[i] = makeNewModelWithLabels();
    fillModelWithPhaseData(phaseTab[i]->invisibleRootItem(),i);
    treeView[i]->setDragEnabled(false);
    treeView[i]->setAcceptDrops(false);
    treeView[i]->setModel(phaseTab[i]);
    treeView[i]->expandToDepth(3);
    treeView[i]->header()->setResizeMode(0, QHeaderView::ResizeToContents);
    treeView[i]->header()->setResizeMode(1, QHeaderView::Stretch);
    treeView[i]->header()->setResizeMode(2, QHeaderView::Stretch);
    treeView[i]->header()->setResizeMode(3, QHeaderView::Stretch);
    treeView[i]->header()->setResizeMode(4, QHeaderView::Stretch);
    tabber->addTab(treeView[i], QString("Phase %1").arg(i));

    //connect(phaseTab[i], SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));
  }

  updateGL();
}

void ViewBikeWidget::newButtonClicked()
{
  //  parameters.destruct();
  parameters.phaseParameters.clear();
  strcpy(parameters.name, "noName");
  parameters.numberOfPhases = 0;
  parameters.footOrigin = Vector3<> (0.f,60.f,-210.f);
  parameters.armOrigin = Vector3<>(0.f,100.f,30.f);
  parameters.loop = false;
  undo.clear();
  redo.clear();
  updateEditorView();
}

void ViewBikeWidget::loadButtonClicked()
{
  char dirname[260]; 
  sprintf(dirname, "%s/Config/Bike/", File::getGTDir());
  fileName = QFileDialog::getOpenFileName(this,
    tr("Open Bike Motion"), dirname, tr("Bike Motion Config Files (*.bmc)"));
  QString name;
  name = fileName.remove(0,fileName.lastIndexOf("/",fileName.lastIndexOf("/")-1)+1);

  InConfigFile stream(name.toAscii().constData());
  if (stream.exists())
  {
    stream >> parameters;
    name = name.remove(0,name.lastIndexOf("/")+1);
    strcpy(parameters.name,name.remove(name.lastIndexOf("."), name.length()).toAscii().constData());

    parameters.init();
    updateEditorView();
    undo.clear();
    redo.clear();
  }
}

QList <QStandardItem*> ViewBikeWidget::makeStringAndValueRow(QString string, QVariant value)
{
  QList <QStandardItem*> list;
  QStandardItem *item1, *item2;
  item1 = makeLabel(string);
  item2 = makeValue(value);
  list.append(item1);
  list.append(item2);
  return list;
}

void ViewBikeWidget::addStringAndValueToList(QList <QStandardItem*> &list, QString string, QVariant value)
{
  QStandardItem *item1, *item2;
  item1 = makeLabel(string);
  item2 = makeValue(value);
  list.append(item1);
  list.append(item2);
}

QStandardItem* ViewBikeWidget::makeLabel(QString string)
{
  QStandardItem* item = new QStandardItem(string);
  item->setEditable(false);
  item->setDragEnabled(false);
  item->setDropEnabled(false);    
  return item;
}

QStandardItem* ViewBikeWidget::makeValue(QVariant var)
{
  QStandardItem* item = new QStandardItem();
  item->setData(var, Qt::DisplayRole);
  item->setDragEnabled(false);
  item->setDropEnabled(false);
  return item;
}

void ViewBikeWidget::addControlPoints(QStandardItem* item)
{
  for (unsigned int k = 0; k < NUM_OF_POINTS; k++)
  {
    QList <QStandardItem*> list;
    list.append(makeLabel(QString("Point%1").arg(k)));
    list.append(makeLabel(QString("")));
    for (int i=0; i < 3; i++)
    {
      list.append(makeValue((double) 0.0));
    }
    item->appendRow(list);
  }
}

QList <QStandardItem*> ViewBikeWidget::addXYZLabel(QStandardItem* item)
{
  QList <QStandardItem*> list;
  list.append(item);
  list.append(makeLabel(QString("")));
  list.append(makeLabel(QString("x")));
  list.append(makeLabel(QString("y")));
  list.append(makeLabel(QString("z")));

  return list;
}

QStandardItemModel* ViewBikeWidget::makeNewModelWithLabels()
{
  QStandardItemModel* model = new QStandardItemModel();
  model->setColumnCount(4);
  model->setHeaderData(0, Qt::Horizontal,  QVariant(" "));
  model->setHeaderData(1, Qt::Horizontal,  QVariant(" "));
  model->setHeaderData(2, Qt::Horizontal,  QVariant(" "));
  model->setHeaderData(3, Qt::Horizontal,  QVariant(" "));
  model->setHeaderData(4, Qt::Horizontal,  QVariant(" "));
  model->setHeaderData(5, Qt::Horizontal,  QVariant(" "));


  QStandardItem *parentItem = model->invisibleRootItem();

  parentItem->appendRow(makeStringAndValueRow(QString("Duration in ms"), (int) 0));

  QStandardItem* lFoot = makeLabel(QString("Left Foot"));
  parentItem->appendRow(lFoot);

  QStandardItem* lFootControlPointsTra = makeLabel(QString("Translation"));
  addControlPoints(lFootControlPointsTra);
  lFoot->appendRow(addXYZLabel(lFootControlPointsTra));

  QStandardItem* lFootControlPointsRot = makeLabel(QString("Rotation"));
  addControlPoints(lFootControlPointsRot);
  lFoot->appendRow(addXYZLabel(lFootControlPointsRot));

  QStandardItem* rFoot = makeLabel(QString("Right Foot"));
  parentItem->appendRow(rFoot);

  QStandardItem* rFootControlPointsTra = makeLabel(QString("Translation"));
  addControlPoints(rFootControlPointsTra);
  rFoot->appendRow(addXYZLabel(rFootControlPointsTra));


  QStandardItem* rFootControlPointsRot = makeLabel(QString("Rotation"));
  addControlPoints(rFootControlPointsRot);
  rFoot->appendRow(addXYZLabel(rFootControlPointsRot));

  QStandardItem* lArm = makeLabel(QString("Left Arm"));
  parentItem->appendRow(lArm);

  QStandardItem* lArmTra = makeLabel(QString("Translation"));
  addControlPoints(lArmTra);
  lArm->appendRow(addXYZLabel(lArmTra));


  QStandardItem* rArm = makeLabel(QString("Right Arm"));
  parentItem->appendRow(rArm);

  QStandardItem* rArmTra = makeLabel(QString("Translation"));
  addControlPoints(rArmTra);
  rArm->appendRow(addXYZLabel(rArmTra));

  QList <QStandardItem*> list;
  QStandardItem* odometry = makeLabel(QString("Odometry Change"));
  list.append(odometry);
  list.append(makeLabel(QString("")));
  list.append(makeLabel(QString("x")));
  list.append(makeLabel(QString("y")));
  list.append(makeLabel(QString("rotZ")));
  parentItem->appendRow(list);

  QList <QStandardItem*> list2;
  list2.append(makeLabel(QString("")));
  list2.append(makeLabel(QString("")));
  list2.append(makeValue(0.f));
  list2.append(makeValue(0.f));
  list2.append(makeValue(0.f));
  odometry->appendRow(list2);

  return model;
}


bool ViewBikeWidget::deleteKids(QStandardItem* rootItem)
{
  if(rootItem->hasChildren())
  {
    for(int i=0; i<rootItem->rowCount(); i++)
    {
      deleteKids(rootItem->child(i,0));
    }
    rootItem->removeRows(0, rootItem->rowCount());
  }
  return true;
}


void ViewBikeWidget::fillModelWithPhaseData(QStandardItem* rootItem, int i)
{
  disconnect(phaseTab[i], SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));

  //Duration im ms
  rootItem->child(0,1)->setData(QVariant(parameters.phaseParameters[i].duration), Qt::DisplayRole);
  rootItem->child(5,0)->child(0,2)->setData(QVariant(parameters.phaseParameters[i].odometryOffset.x), Qt::DisplayRole);
  rootItem->child(5,0)->child(0,3)->setData(QVariant(parameters.phaseParameters[i].odometryOffset.y), Qt::DisplayRole);
  rootItem->child(5,0)->child(0,4)->setData(QVariant(parameters.phaseParameters[i].odometryOffset.z), Qt::DisplayRole);

  int limb = 0, child1 = 1, child2 = 0;
  QStandardItem *item;
  while (limb < Phase::numOfLimbs)
  {
    switch(limb)
    {
    case 0: child1 = 1; child2 = 0; break;
    case 1: child1 = 1; child2 = 1; break;
    case 2: child1 = 2; child2 = 0; break;
    case 3: child1 = 2; child2 = 1; break;
    case 4: child1 = 3; child2 = 0; break;
    case 5: child1 = 4; child2 = 0; break;
    default: limb=100; break;
    }

    for (unsigned int k = 0; k < NUM_OF_POINTS; k++)
    {
      item = rootItem->child(child1,0)->child(child2,0);
      item->child(k,2)->setData(QVariant(parameters.phaseParameters[i].controlPoints[limb][k].x), Qt::DisplayRole);
      item->child(k,3)->setData(QVariant(parameters.phaseParameters[i].controlPoints[limb][k].y), Qt::DisplayRole);
      item->child(k,4)->setData(QVariant(parameters.phaseParameters[i].controlPoints[limb][k].z), Qt::DisplayRole);
    } 
    limb++;
  }
  connect(phaseTab[i], SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));
}

void ViewBikeWidget::writeParametersToFile(const std::string& name)
{
  OutTextFile file(name, false);
  file << "#****" << parameters.name <<  "****"<< endl;
  file <<  "#Common" << "Parameters"<< endl << endl;
  file <<  "#FootOrigin" << "XYZ"<< endl;
  file << parameters.footOrigin << endl << endl;
  file << "#ArmOrigin" << "XYZ" << endl;
  file << parameters.armOrigin << endl << endl;
  file << "#COMBalanceX" << endl;
  file << parameters.kpx << parameters.kix << parameters.kdx << endl << endl;
  file << "#COMBalancY" << endl;
  file << parameters.kpy << parameters.kiy  << parameters.kdy << endl << endl;
  file << "#Preview" <<  "(in" << "Frames)" << endl;
  file << parameters.preview << endl << endl;
  file << "#loop" << endl;
  file << parameters.loop << endl << endl;
  file << "#NumberOfPhases" << endl;
  file << parameters.numberOfPhases << endl << endl;
  for (int i=0; i < parameters.numberOfPhases; i++)
  {
    file << "#######" << "Phase" << i <<"########################################" << endl << endl;
    file << "#duration" << "Timings" << "in" << "ms" << endl;
    file << parameters.phaseParameters[i].duration << endl << endl;

    file << "#Feet" << endl << endl;

    file << "#LeftFoot" << endl << endl;

    file << "#LFootTranslation" << endl;
    file << "#Point2" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][0] << endl;
    file << "#Point3" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][1] << endl;
    file << "#Point4" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftFootTra][2] << endl << endl;

    file << "#LFootRotation" << endl;
    file << "#Point2" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][0] << endl;
    file << "#Point3" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][1] << endl;
    file << "#Point4" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftFootRot][2] << endl << endl;

    file << "#RightFoot" << endl << endl;
    file << "#RFootTranslation" << endl;
    file << "#Point2" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][0] << endl;
    file << "#Point3" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][1] << endl;
    file << "#Point4" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightFootTra][2] << endl << endl;

    file << "#RFootRotation" << endl;
    file << "#Point2" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][0] << endl;
    file << "#Point3" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][1] << endl;
    file << "#Point4" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightFootRot][2] << endl << endl;

    file << "#LeftArm" << endl;
    file << "#Point2" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][0] << endl;
    file << "#Point3" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][1] << endl;
    file << "#Point4" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::leftArmTra][2] << endl << endl;

    file << "#RightArm" << endl;
    file << "#Point2" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][0] << endl;
    file << "#Point3" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][1] << endl;
    file << "#Point4" << endl;
    file << parameters.phaseParameters[i].controlPoints[Phase::rightArmTra][2] << endl << endl;

    file << "#odometryOffset" << "x" << "y" << "rotation" << endl;
    file << parameters.phaseParameters[i].odometryOffset << endl << endl;
  }
}
void ViewBikeWidget::saveAsButtonClicked()
{
  char dirname[260]; 
  sprintf(dirname, "%s/Config/Bike/", File::getGTDir());
  fileName = QFileDialog::getSaveFileName(this,
    tr("Save Bike Motion as..."), dirname, tr("Bike Motion Config Files (*.bmc)"));

  if (fileName.begin() != fileName.end())
  {
    QString temp  = fileName.remove(0,fileName.lastIndexOf("/",fileName.lastIndexOf("/")-1)+1);
    temp = temp.remove(0,temp.lastIndexOf("/")+1);
    strcpy(parameters.name,temp.remove(temp.lastIndexOf("."), temp.length()).toAscii().constData());
    writeParametersToFile(fileName.toAscii().constData());
  }
}
void ViewBikeWidget::saveButtonClicked()
{
  if(fileName.begin() != fileName.end())
  {
    QString name;
    name = fileName.remove(0,fileName.lastIndexOf("/",fileName.lastIndexOf("/")-1)+1);
    writeParametersToFile(name.toAscii().constData());
  }
  else
  {
    saveAsButtonClicked();
  }
}



void ViewBikeWidget::updateCommonParameters(QStandardItem* item)
{
  undo.push_back(parameters);
  if(undo.size() >= 20)
  {
    undo.erase(undo.begin());
  }

  QStandardItem* mamaItem = item->parent();
  int row = item->row();
  if (!mamaItem)
  {
    if (row == 6) parameters.preview = item->data(Qt::DisplayRole).toInt();
    if (row == 7) parameters.loop = item->data(Qt::DisplayRole).toInt();
  }
  else
  {
    int col = item->column();
    if (QString("Foot Origin") == mamaItem->data(Qt::DisplayRole))
    {
      if (col == 2) parameters.footOrigin.x = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 3) parameters.footOrigin.y = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 4) parameters.footOrigin.z = (float)item->data(Qt::DisplayRole).toDouble();
    }
    if (QString("Arm Origin") == mamaItem->data(Qt::DisplayRole))
    {
      if (col == 2) parameters.armOrigin.x = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 3) parameters.armOrigin.y = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 4) parameters.armOrigin.z = (float)item->data(Qt::DisplayRole).toDouble();
    }
    if (QString("COM Balance X") == mamaItem->data(Qt::DisplayRole))
    {
      if (col == 2) parameters.kpx = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 3) parameters.kix = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 4) parameters.kdx = (float)item->data(Qt::DisplayRole).toDouble();
    }
    if (QString("COM Balance Y") == mamaItem->data(Qt::DisplayRole))
    {
      if (col == 2) parameters.kpy = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 3) parameters.kiy = (float)item->data(Qt::DisplayRole).toDouble();
      if (col == 4) parameters.kdy = (float)item->data(Qt::DisplayRole).toDouble();
    }
  }

  parameters.init();
}

void ViewBikeWidget::updatePhaseParameters(QStandardItem* item)
{
  QStandardItem* mamaItem = item->parent();
  int row = item->row();
  int phaseNumber = tabber->currentIndex()-1;
  int col = item->column();
  int limb = -1;

  undo.push_back(parameters);
  if(undo.size() >= 20)
  {
    undo.erase(undo.begin());
  }

  if (!mamaItem)
  {
    if (row == 0) parameters.phaseParameters[phaseNumber].duration = item->data(Qt::DisplayRole).toInt();
  }
  else
  {
    if (QString("Left Foot") == mamaItem->parent()->data(Qt::DisplayRole))
    {
      if (QString("Translation") == mamaItem->data(Qt::DisplayRole))
      {
        limb = Phase::leftFootTra;
      }
      if (QString("Rotation") == mamaItem->data(Qt::DisplayRole))
      {
        limb = Phase::leftFootRot;
      }
    }

    if (QString("Right Foot") == mamaItem->parent()->data(Qt::DisplayRole))
    {
      if (QString("Translation") == mamaItem->data(Qt::DisplayRole))
      {
        limb = Phase::rightFootTra;

      }
      if (QString("Rotation") == mamaItem->data(Qt::DisplayRole))
      {
        limb = Phase::rightFootRot;
      }
    }
    if (QString("Right Arm") == mamaItem->parent()->data(Qt::DisplayRole))
    {
      if (QString("Translation") == mamaItem->data(Qt::DisplayRole))
      {
        limb = Phase::rightArmTra;
      }
    }
    if (QString("Left Arm") == mamaItem->parent()->data(Qt::DisplayRole))
    {
      if (QString("Translation") == mamaItem->data(Qt::DisplayRole))
      {
        limb = Phase::leftArmTra;

      }
    }

    if (QString("Odometry Change") == mamaItem->parent()->data(Qt::DisplayRole))
    {
      switch(col)
      {
      case 2: parameters.phaseParameters[phaseNumber].odometryOffset.x = (float)item->data(Qt::DisplayRole).toDouble(); break;
      case 3: parameters.phaseParameters[phaseNumber].odometryOffset.y = (float)item->data(Qt::DisplayRole).toDouble(); break;
      case 4: parameters.phaseParameters[phaseNumber].odometryOffset.z = (float)item->data(Qt::DisplayRole).toDouble(); break;
      default: break;
      }
    }
  }

  selectedPoint.limb = limb;
  if (limb > -1)
  {
    if (row < NUM_OF_POINTS)
    {
      selectedPoint.pointNumber = row;
      switch(col)
      {
      case 2: parameters.phaseParameters[phaseNumber].controlPoints[limb][row].x = (float)item->data(Qt::DisplayRole).toDouble(); break;
      case 3: parameters.phaseParameters[phaseNumber].controlPoints[limb][row].y = (float)item->data(Qt::DisplayRole).toDouble(); break;
      case 4: parameters.phaseParameters[phaseNumber].controlPoints[limb][row].z = (float)item->data(Qt::DisplayRole).toDouble(); break;
      default: break;
      } 
    }
  }

  parameters.init();
}


void ViewBikeWidget::initializeGL()
{
  viewBike.renderer.init(isSharing());
  viewBike.renderer.setCameraMode(VisualizationParameterSet::TARGETCAM);
  viewBike.renderer.setSurfaceStyle(VisualizationParameterSet::CAMERA_SIMULATION);
  viewBike.renderer.setSnapToRoot(false);
  viewBike.renderer.resetCamera();
  targetPosOffset.init(0.3,0,0);
  cameraPosOffset.init(0.3,0,0);
  viewBike.renderer.zoom(0.3);
}

void ViewBikeWidget::resizeGL(int newWidth, int newHeight)
{
  viewBike.renderer.setSize(newWidth, newHeight);
}


bool ViewBikeWidget::clickControlPoint(const int& x, const int& y)
{
  Vector3d vecNear, vecFar;

  gluUnProjectClick(x,y, vecFar, vecNear);

  for (int phaseNumber=0; phaseNumber < parameters.numberOfPhases; phaseNumber++)
  {
    if(!moveDrag)
    {
      if (!singleDraw || phaseNumber == selectedPoint.phaseNumber)
      {
        for (int limb=0; limb<Phase::numOfLimbs; limb++)
        {
          Vector3d point((double)parameters.phaseParameters[phaseNumber].originPos[limb].x, 
            (double)parameters.phaseParameters[phaseNumber].originPos[limb].y, 
            (double)parameters.phaseParameters[phaseNumber].originPos[limb].z);
          for(unsigned int i=0; i < NUM_OF_POINTS; i++)
          {             
            Vector3d cubePoint;
            cubePoint = point + Vector3d(
              (double)(parameters.phaseParameters[phaseNumber].controlPoints[limb][i].x - parameters.phaseParameters[phaseNumber].originOffset[limb].x),
              (double)(parameters.phaseParameters[phaseNumber].controlPoints[limb][i].y - parameters.phaseParameters[phaseNumber].originOffset[limb].y),
              (double)(parameters.phaseParameters[phaseNumber].controlPoints[limb][i].z - parameters.phaseParameters[phaseNumber].originOffset[limb].z));

            if (limb== Phase::leftFootRot)
            {
              Vector3d drawPos(0.,150., 0.);
              drawPos.rotateX(cubePoint.v[0]);
              drawPos.rotateZ(cubePoint.v[2]);
              drawPos.v[0]+= (double)parameters.footOrigin.x;
              drawPos.v[1]+= (double)parameters.footOrigin.y;
              drawPos.v[2]+= (double)parameters.footOrigin.z;

              Vector3d intersection;
              if (SimGeometry::intersectRayAndBox(vecNear, vecFar, drawPos, originRot,15.0, 15.0, 15.0, intersection))
              {
                tabber->setCurrentIndex(phaseNumber+1);
                selectedPoint.phaseNumber = phaseNumber;
                selectedPoint.pointNumber = i;
                selectedPoint.limb = limb;
                selectedPoint.xzRot = true;
                moveDrag = true;
                return true;
              }
              Vector3d drawPos2(150.,0., 0.);
              drawPos2.rotateY(cubePoint.v[1]);
              drawPos2.rotateZ(cubePoint.v[2]);
              drawPos2.v[0]+= (double)parameters.footOrigin.x;
              drawPos2.v[1]+= (double)parameters.footOrigin.y;
              drawPos2.v[2]+= (double)parameters.footOrigin.z;
              cubePoint = drawPos2;
            }

            if (limb== Phase::rightFootRot)
            {
              Vector3d drawPos(0.,-150., 0.);
              drawPos.rotateX(cubePoint.v[0]);
              drawPos.rotateZ(cubePoint.v[2]);
              drawPos.v[0]+= (double)parameters.footOrigin.x;
              drawPos.v[1]+= (double)-parameters.footOrigin.y;
              drawPos.v[2]+= (double)parameters.footOrigin.z;

              Vector3d intersection;
              if (SimGeometry::intersectRayAndBox(vecNear, vecFar, drawPos, originRot,15.0, 15.0, 15.0, intersection))
              {
                tabber->setCurrentIndex(phaseNumber+1);
                selectedPoint.phaseNumber = phaseNumber;
                selectedPoint.pointNumber = i;
                selectedPoint.limb = limb;
                selectedPoint.xzRot = true;
                moveDrag = true;
                return true;
              }

              Vector3d drawPos2(150.,0.,0.);
              drawPos2.rotateY(cubePoint.v[1]);
              drawPos2.rotateZ(cubePoint.v[2]);
              drawPos2.v[0]+= (double)parameters.footOrigin.x;
              drawPos2.v[1]+= (double)-parameters.footOrigin.y;
              drawPos2.v[2]+= (double)parameters.footOrigin.z;
              cubePoint = drawPos2;
            }

            Vector3d intersection;
            if (SimGeometry::intersectRayAndBox(vecNear, vecFar, cubePoint, originRot,15.0, 15.0, 15.0, intersection))
            {
              tabber->setCurrentIndex(phaseNumber+1);
              selectedPoint.phaseNumber = phaseNumber;
              selectedPoint.pointNumber = i;
              selectedPoint.limb = limb;
              selectedPoint.xzRot = false;
              moveDrag = true;
              return true;
            }
          }
        }
      }
    }

    if ((tra2dWindows || tra1dWindows) && !moveViewOfViewDragXY && !moveViewOfViewDragXZ && !moveViewOfViewDragYZ
      && !moveViewOfViewDragXP && !moveViewOfViewDragYP && !moveViewOfViewDragZP)
    {
      if (selectedPoint.phaseNumber > -1 && selectedPoint.limb > -1)
      {
        unsigned int width, height;
        viewBike.renderer.getSize(width, height);

        float mini_window_width = width/3.0f;
        float mini_window_height = (height-150.0f)/3.0f;

        Vector3<> p = parameters.getPosition(0.0, selectedPoint.phaseNumber, selectedPoint.limb)
          + parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb] - 
          parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb];

         float maxXrange=100, minXrange=-100, maxYrange=150, minYrange=-150, maxZrange=100, minZrange=-250;

        std::vector<Vector2<> > cpWindow1, cpWindow2, cpWindow3;
        cpWindow1.resize(NUM_OF_POINTS);
        cpWindow2.resize(NUM_OF_POINTS);
        cpWindow3.resize(NUM_OF_POINTS);

        for (unsigned int k=0; k < NUM_OF_POINTS; k++)
        {
          Vector3<> controlPoint = parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][k];

          minXrange = std::min<float>(minXrange, controlPoint.x);
          maxXrange = std::max<float> (maxXrange, controlPoint.x);
          minYrange = std::min<float>(minYrange, controlPoint.y);
          maxYrange = std::max<float>(maxYrange, controlPoint.y);
          minZrange = std::min<float>(minZrange, controlPoint.z);
          maxZrange = std::max<float>(maxZrange, controlPoint.z);
          if (tra2dWindows)
          {
            cpWindow1[k]=(Vector2<>(controlPoint.x, controlPoint.y));
            cpWindow2[k]=(Vector2<>(controlPoint.x, controlPoint.z));
            cpWindow3[k]=(Vector2<>(controlPoint.y, controlPoint.z));
          }
          else
          {
            cpWindow1[k]=(Vector2<>((1.0f/3.0f)*(float)(k+1), controlPoint.x));
            cpWindow2[k]=(Vector2<>((1.0f/3.0f)*(float)(k+1), controlPoint.y));
            cpWindow3[k]=(Vector2<>((1.0f/3.0f)*(float)(k+1), controlPoint.z));
          }
        }
        //calculate zoomfactor
        float
          scaleFactorX = mini_window_width/std::max<>(maxXrange-minXrange,1.0f),
          scaleFactorX1 = mini_window_height/std::max<>(maxXrange-minXrange,1.0f),
          scaleFactorY = mini_window_height/std::max<>(maxYrange-minYrange,1.0f),
          scaleFactorY1 = mini_window_width/std::max<>(maxYrange-minYrange,1.0f),
          scaleFactorZ = mini_window_height/std::max<>(maxZrange-minZrange,1.0f);


        for(unsigned int k=0; k<cpWindow1.size(); k++)
        {
          if (tra2dWindows)
          {

            //XY-Window
            float win1, win2;
            win1 = (float)width-20.f-(float)width/3.0f + ((cpWindow1[k].x-minXrange)*scaleFactorX);
            win2 = mini_window_height+20.0f -((cpWindow1[k].y-minYrange)*scaleFactorY);
            float localDist = sqrt(((float)x-win1)*((float)x-win1) + ((float)y-win2)*((float)y-win2));
            if (localDist < 10.f)
            {
              selectedPoint.pointNumber = k;
              moveViewOfViewDragXY = true;
              return true;
            }
            //XZ-Window
            win1 = (float)width-20.f-(float)width/3.0f + ((cpWindow2[k].x-minXrange)*scaleFactorX);
            win2 = mini_window_height*2.f+55.0f -((cpWindow2[k].y-minZrange)*scaleFactorZ); 
            localDist = sqrt(((float)x-win1)*((float)x-win1) + ((float)y-win2)*((float)y-win2));
            if (localDist < 10.f)
            {
              selectedPoint.pointNumber = k;
              moveViewOfViewDragXZ = true;
              return true;
            }
            //XZ-Window
            win1 = (float)width-20.f-(float)width/3.0f + ((cpWindow3[k].x-minYrange)*scaleFactorY1); 
            win2 = mini_window_height*3.f+90 -((cpWindow3[k].y-minZrange)*scaleFactorZ); 
            localDist = sqrt(((float)x-win1)*((float)x-win1) + ((float)y-win2)*((float)y-win2));
            if (localDist < 10.f)
            {
              selectedPoint.pointNumber = k;
              moveViewOfViewDragYZ = true;
              return true;
            } 
          }
          else 
          {
            //XP-Window
            float win1, win2;
            win1 = (float)width-20.f-(float)width/3.0f + (cpWindow1[k].x*mini_window_width);
            win2 = mini_window_height+20.0f -((cpWindow1[k].y-minXrange)*scaleFactorX1);
            float localDist = sqrt(((float)x-win1)*((float)x-win1) + ((float)y-win2)*((float)y-win2));
            if (localDist < 10.f)
            {
              selectedPoint.pointNumber = k;
              moveViewOfViewDragXP = true;
              return true;
            }
            //YP-Window
            win1 = (float)width-20.f-(float)width/3.0f + (cpWindow1[k].x*mini_window_width);
            win2 = mini_window_height*2.f+55.0f -((cpWindow2[k].y-minYrange)*scaleFactorY); 
            localDist = sqrt(((float)x-win1)*((float)x-win1) + ((float)y-win2)*((float)y-win2));
            if (localDist < 10.0)
            {
              selectedPoint.pointNumber = k;
              moveViewOfViewDragYP = true;
              return true;
            }
            //ZP-Window
            win1 = (float)width-20.f-(float)width/3.0f + (cpWindow1[k].x*mini_window_width);
            win2 = mini_window_height*3.f+90.f -((cpWindow3[k].y-minZrange)*scaleFactorZ); 
            localDist = sqrt(((float)x-win1)*((float)x-win1) + ((float)y-win2)*((float)y-win2));
            if (localDist < 10.f)
            {
              selectedPoint.pointNumber = k;
              moveViewOfViewDragZP = true;
              return true;
            } 

          }
        } 
      }
    }
  }
  return false;
}

void ViewBikeWidget::gluUnProjectClick(int x, int y, Vector3d& vecFar, Vector3d& vecNear)
{
  glPushMatrix();
  setMatrix(viewBike.origin->getPosition(), originRot);
  glScaled(0.001, 0.001, 0.001);
  GLint viewport[4];
  glGetIntegerv(GL_VIEWPORT, viewport);
  GLdouble modelview[16];
  glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
  GLdouble projection[16];
  glGetDoublev(GL_PROJECTION_MATRIX, projection);
  GLfloat winX(0), winY(0);
  winX = (float) x;
  winY = (float) y;
  winY = (float)viewport[3] - winY;

  GLdouble tx, ty, tz;

  gluUnProject( winX, winY, 1.0, modelview, projection, viewport, &tx, &ty, &tz);

  vecFar.v[0] = tx;
  vecFar.v[1] = ty;
  vecFar.v[2] = tz;

  gluUnProject( winX, winY, 0.0, modelview, projection, viewport, &tx, &ty, &tz);

  vecNear.v[0] = tx;
  vecNear.v[1] = ty;
  vecNear.v[2] = tz;

  glPopMatrix();
}

void ViewBikeWidget::drawPhases()
{
  for (int phaseNumber=0; phaseNumber < parameters.numberOfPhases; phaseNumber++)
  {
    if (!singleDraw || phaseNumber == selectedPoint.phaseNumber)
    {
      drawBezierCurves(phaseNumber);
      //Draw Bezierlines
      Vector3<> oPoint;

      for (int j=0; j< Phase::numOfLimbs; j++)
      {
        if (j!= Phase::leftFootRot && j!= Phase::rightFootRot)
        {
          oPoint = parameters.phaseParameters[phaseNumber].originPos[j];
          for (unsigned int k=0; k < NUM_OF_POINTS; k++)
          {
            Vector3<> point1, point2;
            if (k == 0)
            {
              point1 = oPoint;
              point2 = point1 + parameters.phaseParameters[phaseNumber].controlPoints[j][k] - parameters.phaseParameters[phaseNumber].originOffset[j];
            }
            else
            {
              point1 = oPoint + parameters.phaseParameters[phaseNumber].controlPoints[j][k-1] - parameters.phaseParameters[phaseNumber].originOffset[j];
              point2 = oPoint + parameters.phaseParameters[phaseNumber].controlPoints[j][k] - parameters.phaseParameters[phaseNumber].originOffset[j];
            }
            glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
            glBegin(GL_LINES);
            glVertex3f(point1.x, point1.y, point1.z);
            glVertex3f(point2.x, point2.y, point2.z);
            glEnd();
          }
        }
      }
    }
  }
}

void ViewBikeWidget::drawBezierCurves(const int& phaseNumber)
{
  Vector3<> position, positions[6];

  float ps[1];
  glGetFloatv(GL_POINT_SIZE, ps);

  for (double phase = 0.0; phase < 1.0; phase+= 0.02)
  {
    for (int j=0; j<Phase::numOfLimbs; j++)
    {
      if (selectedPoint.phaseNumber == phaseNumber)
      {
        glColor4f(0.0f, 1.0f, 0.0f, 1.0f); //green selected Phase
        if (selectedPoint.limb == j)
        {
          glColor4f(1.0f, 1.0f, 0.0f, 1.0f); //selected Limb yellow
        }
      }
      else
      {
        glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
      }

      glPointSize(4);
      glBegin(GL_POINTS);
      positions[j] = parameters.getPosition(phase, phaseNumber, j);

      if (j== Phase::leftFootRot)
      {
        Vector3d drawPos(0.,150., 0.);
        drawPos.rotateX(positions[j].x);
        drawPos.rotateZ(positions[j].z);
        glVertex3f(parameters.footOrigin.x + drawPos.v[0],parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z + drawPos.v[2]);

        Vector3d drawPos2(150.,0., 0.);
        drawPos2.rotateY(positions[j].y);
        drawPos2.rotateZ(positions[j].z);
        position = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
        glVertex3f(position[0], position[1], position[2]);
      }

      if (j== Phase::rightFootRot)
      {
        Vector3d drawPos(0.,-150., 0.);
        drawPos.rotateX(positions[j].x);
        drawPos.rotateZ(positions[j].z);
        glVertex3f(parameters.footOrigin.x + drawPos.v[0], -parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z +drawPos.v[2]);

        Vector3d drawPos2(150.,0.,0.);
        drawPos2.rotateY(positions[j].y);
        drawPos2.rotateZ(positions[j].z);
        position = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], -parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
        glVertex3f(position[0], position[1], position[2]);
      }
      glEnd();
    }

    //clip legs
    double leg0, leg1, leg2, leg3, leg4, leg5;
    bool leftLegTooShort = calcLegJoints(positions[Phase::leftFootTra], positions[Phase::leftFootRot], true, leg0, leg1, leg2, leg3, leg4, leg5);
    if (clipLegJointsWithLimits(leg1, leg2, leg3, JointData::LHipYawPitch) || leftLegTooShort)
    {
      glPointSize(4);

      Vector3<> foot = calcFootPos(leg0,leg1,leg2, leg3, leg4, leg5, JointData::LHipYawPitch);
      glColor4f(1.0f, 0.5f, 0.f, 1.f);
      glBegin(GL_POINTS);
      glVertex3f(foot.x, foot.y, foot.z);
      glEnd();
    }
    else 
    {
      if (selectedPoint.phaseNumber == phaseNumber)
      {
        glColor4f(0.0f, 1.0f, 0.0f, 1.0f); //green selected Phase
        if (selectedPoint.limb == Phase::leftFootTra)
        {
          glColor4f(1.0f, 1.0f, 0.0f, 1.0f); //selected Limb yellow
        }
      }
      else
      {
        glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
      }

      glBegin(GL_POINTS);
      glVertex3f(positions[Phase::leftFootTra].x, positions[Phase::leftFootTra].y, positions[Phase::leftFootTra].z);
      glEnd();
    }

    bool rightLegTooShort = calcLegJoints(positions[Phase::rightFootTra], positions[Phase::rightFootRot], false, leg0, leg1, leg2, leg3, leg4, leg5);

    if (clipLegJointsWithLimits(leg1, leg2, leg3, JointData::RHipYawPitch) || rightLegTooShort)
    {
      glPointSize(4);

      Vector3<> foot = calcFootPos(leg0,leg1,leg2, leg3, leg4, leg5, JointData::RHipYawPitch);
      glColor4f(1.0f, 0.5f, 0.f, 1.f);
      glBegin(GL_POINTS);
      glVertex3f(foot.x, foot.y, foot.z);
      glEnd();
    }
    else
    {
      if (selectedPoint.phaseNumber == phaseNumber)
      {
        glColor4f(0.0f, 1.0f, 0.0f, 1.0f); //green selected Phase
        if (selectedPoint.limb == Phase::rightFootTra)
        {
          glColor4f(1.0f, 1.0f, 0.0f, 1.0f); //selected Limb yellow
        }
      }
      else
      {
        glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
      }
      glBegin(GL_POINTS);
      glVertex3f(positions[Phase::rightFootTra].x, positions[Phase::rightFootTra].y, positions[Phase::rightFootTra].z);
      glEnd();
    }
    if (selectedPoint.phaseNumber == phaseNumber)
    {
      glColor4f(0.0f, 1.0f, 0.0f, 1.0f); //green selected Phase
    }
    else
    {
      glColor4f(0.0f, 0.0f, 1.0f, 1.0f);
    }

    if (selectedPoint.limb == Phase::rightArmTra)
    {
      glColor4f(1.0f, 1.0f, 0.0f, 1.0f); //selected Limb yellow
    }
    glBegin(GL_POINTS);
    glVertex3f(positions[Phase::rightArmTra].x, positions[Phase::rightArmTra].y, positions[Phase::rightArmTra].z);
    glEnd();
    if (selectedPoint.limb == Phase::leftArmTra)
    {
      glColor4f(1.0f, 1.0f, 0.0f, 1.0f); //selected Limb yellow
    }
    glBegin(GL_POINTS);
    glVertex3f(positions[Phase::leftArmTra].x, positions[Phase::leftArmTra].y, positions[Phase::leftArmTra].z);
    glEnd();
    glPointSize(ps[0]);
  }


  //draw controlPoints
  for (int j=0; j<Phase::numOfLimbs; j++)
  {
    Vector3 <> point = parameters.phaseParameters[phaseNumber].originPos[j];

    for (unsigned int i=0; i < NUM_OF_POINTS; i++)
    {
      Vector3 <> cubePoint;
      cubePoint = point + parameters.phaseParameters[phaseNumber].controlPoints[j][i] - parameters.phaseParameters[phaseNumber].originOffset[j];

      if (selectedPoint.phaseNumber == phaseNumber)
      {
        glColor4f(0.0f, 1.0f, 0.0f, 1.0f); //green selected Phase
        if (selectedPoint.limb == j)
        {
          glColor4f(1.0f, 1.0f, 0.0f, 1.0f); //selected Limb yellow

          if ((unsigned int) selectedPoint.pointNumber == i)
          {
            glColor4f(0.0f, 1.0f, 1.0f, 1.0f); //cyan selected point
            if (moveDrag || moveViewOfViewDragYP || moveViewOfViewDragZP || moveViewOfViewDragXP
              || moveViewOfViewDragXY || moveViewOfViewDragXZ || moveViewOfViewDragYZ)
            {
              glColor4f(1.0f, 0.0f, 1.0f, 1.0f); //magenta moved selected point
            }
          }
        }
      }
      else
      {
        glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
      }

      if (j== Phase::leftFootRot)
      {
        Vector3d drawPos(0.f,150.f, 0.f);
        drawPos.rotateX(cubePoint.x);
        drawPos.rotateZ(cubePoint.z);
        drawControlPoint(Vector3<>(parameters.footOrigin.x + drawPos.v[0], parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z + drawPos.v[2]), 5.0f);

        Vector3d drawPos2(150.f,0.f, 0.f);
        drawPos2.rotateY(cubePoint.y);
        drawPos2.rotateZ(cubePoint.z);
        cubePoint = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
      }

      if (j== Phase::rightFootRot)
      {
        Vector3d drawPos(0.f,-150.f, 0.f);
        drawPos.rotateX(cubePoint.x);
        drawPos.rotateZ(cubePoint.z);
        drawControlPoint(Vector3<>(parameters.footOrigin.x + drawPos.v[0], -parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z + drawPos.v[2]), 5.0f);

        Vector3d drawPos2(150.f,0.f, 0.f);
        drawPos2.rotateY(cubePoint.y);
        drawPos2.rotateZ(cubePoint.z);
        cubePoint = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], -parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
      }
      drawControlPoint(cubePoint, 5.0f);
    }
  }
}

void ViewBikeWidget::drawControlPoint(const Vector3 <>& point, const float& cubeFaktor)
{
  glBegin(GL_QUADS);
  glVertex3f(point.x+ cubeFaktor, point.y+ cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y+ cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y+ cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y+ cubeFaktor, point.z+ cubeFaktor);

  glVertex3f(point.x+ cubeFaktor, point.y- cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y- cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y- cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y- cubeFaktor, point.z- cubeFaktor);

  glVertex3f(point.x+ cubeFaktor, point.y+ cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y+ cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y- cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y- cubeFaktor, point.z+ cubeFaktor);

  glVertex3f(point.x+ cubeFaktor, point.y- cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y- cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y+ cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y+ cubeFaktor, point.z- cubeFaktor);

  glVertex3f(point.x- cubeFaktor, point.y+ cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y+ cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y- cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x- cubeFaktor, point.y- cubeFaktor, point.z+ cubeFaktor);

  glVertex3f(point.x+ cubeFaktor, point.y+ cubeFaktor, point.z- cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y+ cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y- cubeFaktor, point.z+ cubeFaktor);
  glVertex3f(point.x+ cubeFaktor, point.y- cubeFaktor, point.z- cubeFaktor);
  glEnd();
}

void ViewBikeWidget::paintGL()
{ 

  originRot =  viewBike.origin->getRotation();
  double originZ = originRot.getZAngle();
  originRot.setXRotation(0);
  originRot.setYRotation(0);
  originRot.setZRotation(originZ);

  if (getString > 0) 
  {
    commands.push_back("_get2 representation:MotionRequest");
    getString--;
  }

  if (!commands.empty())
  {
    if (commands[0] == " ")
    {
      commands.erase(commands.begin());
    }
    else
    {
      viewBike.console.handleConsole(commands[0]);
      commands.erase(commands.begin());
    }
  }

  if (viewBike.motionRequest.motion == MotionRequest::bike && lastMotion != MotionRequest::bike)
  {
    for (int i=0; i<Phase::numOfLimbs; i++)
    {
      reachedPositions[i].clear();
    }
  }
  lastMotion = viewBike.motionRequest.motion;

  if (viewBike.motionRequest.motion == MotionRequest::bike && reachedDraw)
  {
    Vector3<> positions[Phase::numOfLimbs];

    positions[Phase::leftFootTra] = calculateFootPos(viewBike.jointData, JointData::LHipYawPitch);
    positions[Phase::rightFootTra] = calculateFootPos(viewBike.jointData, JointData::RHipYawPitch);

    positions[Phase::leftFootRot] = Vector3<>(viewBike.jointData.angles[JointData::LAnkleRoll], viewBike.jointData.angles[JointData::LAnklePitch], viewBike.jointData.angles[JointData::LHipYawPitch]);
    positions[Phase::rightFootRot] = Vector3<>(viewBike.jointData.angles[JointData::RAnkleRoll], viewBike.jointData.angles[JointData::RAnklePitch], viewBike.jointData.angles[JointData::RHipYawPitch]);

    positions[Phase::leftArmTra] = calculateHandPos(viewBike.jointData, JointData::LShoulderPitch);
    positions[Phase::rightArmTra] = calculateHandPos(viewBike.jointData, JointData::RShoulderPitch);

    for (int i=0; i<Phase::numOfLimbs; i++)
    {
      reachedPositions[i].push_back(positions[i]);
    }
  }

  Vector3d cameraPos,
           targetPos;
  viewBike.renderer.getCamera(cameraPos, targetPos);
  Vector3d newTargetPos = viewBike.origin->getPosition();
  cameraPos -= targetPos;

  Vector3d targetPosTempOffset(targetPosOffset.v[0], targetPosOffset.v[1], targetPosOffset.v[2]),
  cameraPosTempOffset(cameraPosOffset.v[0], cameraPosOffset.v[1], cameraPosOffset.v[2]);
  
  cameraPosTempOffset.rotateZ(-cameraPos.getZRotation());
  targetPosTempOffset.rotateZ(-cameraPos.getZRotation());
  
  cameraPos += newTargetPos;
  targetPos = newTargetPos;
  
  viewBike.renderer.setCamera(cameraPos+cameraPosTempOffset, targetPos+targetPosTempOffset);

  unsigned int width, height;
  viewBike.renderer.getSize(width, height);

  if(ghost)
  {
    glEnable(GL_POLYGON_STIPPLE);
    glPolygonStipple(stippleMask[16-ghost]);  
    viewBike.renderer.draw();
    glDisable(GL_POLYGON_STIPPLE);
  }
  else
    viewBike.renderer.draw();

  glDisable(GL_BLEND);
  glEnable(GL_DEPTH_TEST);


  glPushMatrix();
  setMatrix(viewBike.origin->getPosition(), originRot);
  glScaled(0.001, 0.001, 0.001);
  if (phaseDrawings)
  {
    drawPhases();
  }
  if (reachedDraw)
  {
    float ps[1];
    glGetFloatv(GL_POINT_SIZE, ps);
    glPointSize(4);
    glColor4f(1.f, 0.f, 1.f, 1.f);
    glBegin(GL_POINTS);
    for (int i=0; i<Phase::numOfLimbs; i++)
    {
      if (!reachedPositions[i].empty())
      {
        for(unsigned int j = 0; j  < reachedPositions[i].size(); ++j)
        {
          glVertex3d(reachedPositions[i][j].x, reachedPositions[i][j].y, reachedPositions[i][j].z);

        }
      }
    }
    glEnd();
    glPointSize(ps[0]);
  }
  glPopMatrix();
  glFlush();

  if(tra2dWindows || tra1dWindows || velocityWindows || accelWindows)
  {
    // draw background
    glDisable(GL_DEPTH_TEST);
    glColor4f(0.0f, 0.0f, 0.0f, 0.6f);
    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_BLEND);
    glMatrixMode(GL_PROJECTION);

    glPushMatrix();
    glLoadIdentity();
    glOrtho(0.0, width, 0.0, height, 0.0, 0.2);
    glMatrixMode(GL_MODELVIEW);

    glPushMatrix();

    glLoadIdentity();

    float offsetTop = 20.f;
    float offsetBetween = 35.f;

    float mini_window_width = width/3.0f;
    float mini_window_height = (height-150.0f)/3.0f;

    glTranslatef(width-width/3.0f-55, height, 0.f);

    //draw the backgrounds
    glBegin(GL_QUADS);
    glVertex2f(0.0f,  -(mini_window_height*3)-(offsetBetween*3)-10);
    glVertex2f(mini_window_width+55, -(mini_window_height*3)-(offsetBetween*3)-10);
    glVertex2f(mini_window_width+55, 0.0f);
    glVertex2f(0.0f,0.0f);
    glEnd();

    glBlendFunc(GL_DST_COLOR, GL_SRC_ALPHA);

       glTranslatef(35.f, -offsetTop, 0.f);

    glColor4f(1.0f,0.0f,0.0f,1.f);

    if (tra2dWindows)
    {
      renderText(0.5f,4.0f,0, "[C(phase) with P[x,y]", QFont("arial",10));
      renderText(0.5f,-mini_window_height-offsetBetween+4.0f,0, "[C(phase) with P[x,z]]", QFont("arial",10));
      renderText(0.5f,-mini_window_height*2-offsetBetween*2+4.0f,0, "[C(phase) with P[y,z]]", QFont("arial",10));
    }
    else if (tra1dWindows)
    {
      renderText(0.5f,4.0f,0, "[x:phase] with phase = [0..1]", QFont("arial",10));
      renderText(0.5f,-mini_window_height-offsetBetween+4.0f,0, "[y:phase] with phase = [0..1]", QFont("arial",10));
      renderText(0.5f,-mini_window_height*2-offsetBetween*2+4.0f,0, "[z:phase]  with phase = [0..1]", QFont("arial",10));
    }
    else if (velocityWindows)
    {
      renderText(0.5f,4.0f,0, "[x':phase] with phase = [0..1]", QFont("arial",10));
      renderText(0.5f,-mini_window_height-offsetBetween+4.0f,0, "[y':phase] with phase = [0..1]", QFont("arial",10));
      renderText(0.5f,-mini_window_height*2-offsetBetween*2+4.0f,0, "[z':phase]  with phase = [0..1]", QFont("arial",10));
    }
    else if (accelWindows)
    {
      renderText(0.5f,4.0f,0, "[x'':phase] with phase = [0..1]", QFont("arial",10));
      renderText(0.5f,-mini_window_height-offsetBetween+4.0f,0, "[y'':phase] with phase = [0..1]", QFont("arial",10));
      renderText(0.5f,-mini_window_height*2-offsetBetween*2+4.0f,0, "[z'':phase]  with phase = [0..1]", QFont("arial",10));
    }

    glColor4f(1.0f,0.0f,0.0f,0.6f);
    if (selectedPoint.phaseNumber > -1 && selectedPoint.limb > -1)
    {

      Vector3<> p = parameters.getPosition(0.f, selectedPoint.phaseNumber, selectedPoint.limb)
        + parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb] - 
        parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb];

      float maxXrange=100, minXrange=-100, maxYrange=150, minYrange=-150, maxZrange=100, minZrange=-250;

      std::vector<Vector2<> > cpWindow1, cpWindow2, cpWindow3, pWindow1, pWindow2, pWindow3;

      for (unsigned int k=0; k < NUM_OF_POINTS; k++)
      {
        Vector3<> controlPoint = parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][k];

        minXrange = std::min<float>(minXrange, controlPoint.x);
        maxXrange = std::max<float>(maxXrange, controlPoint.x);
        minYrange = std::min<float>(minYrange, controlPoint.y);
        maxYrange = std::max<float>(maxYrange, controlPoint.y);
        minZrange = std::min<float>(minZrange, controlPoint.z);
        maxZrange = std::max<float>(maxZrange, controlPoint.z);
        
        if(tra2dWindows)
        {
          cpWindow1.push_back(Vector2<>(controlPoint.x, controlPoint.y));
          cpWindow2.push_back(Vector2<>(controlPoint.x, controlPoint.z));
          cpWindow3.push_back(Vector2<>(controlPoint.y, controlPoint.z));
        }
        else
        {
          cpWindow1.push_back(Vector2<>((1.0f/3.0f)*(k+1), controlPoint.x));
          cpWindow2.push_back(Vector2<>((1.0f/3.0f)*(k+1), controlPoint.y));
          cpWindow3.push_back(Vector2<>((1.0f/3.0f)*(k+1), controlPoint.z));
        }

      }

      if (tra1dWindows || tra2dWindows)
      {
        for(float phase=0.f; phase <= 1.f;  phase+=0.01f)
        {
          Vector3<> point = parameters.getPosition(phase, selectedPoint.phaseNumber, selectedPoint.limb)
            + parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb] - 
            parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb];

          if (tra2dWindows)
          {
            pWindow1.push_back(Vector2<>(point.x,point.y));
            pWindow2.push_back(Vector2<>(point.x, point.z));
            pWindow3.push_back(Vector2<>(point.y,point.z));
          }
          else
          {
            pWindow1.push_back(Vector2<>(phase, point.x));
            pWindow2.push_back(Vector2<>(phase, point.y));
            pWindow3.push_back(Vector2<>(phase, point.z));
          }
        }
      }
      else
      {
        Vector3<> p0 = parameters.getPosition(0.f,selectedPoint.phaseNumber,selectedPoint.limb)
          + parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb] - 
          parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb];

        Vector3<> originOffset = parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb];
        Vector3<> originPos = parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb];
        for(float phase=0.f; phase <= 1.f; phase+=0.01f)
        {
          Vector3<> point;
          if(velocityWindows)
          {
            point.x = (-3.f+6.f*phase-3.f*phase*phase)*(p0.x-originOffset.x) + 3.f*(2*phase - 4.f*phase + 3.f*phase*phase)*(cpWindow1[0].y-originOffset.x) + 3.f*(2.f*phase - 3.f*phase*phase)*(cpWindow1[1].y-originOffset.x) + 3.f*phase*phase*(cpWindow1[2].y-originOffset.x);
            point.y = (-3.f+6.f*phase-3.f*phase*phase)*(p0.y-originOffset.y) + 3.f*(2*phase - 4.f*phase + 3.f*phase*phase)*(cpWindow2[0].y-originOffset.y) + 3.f*(2.f*phase - 3.f*phase*phase)*(cpWindow2[1].y-originOffset.y) + 3.f*phase*phase*(cpWindow2[2].y-originOffset.y);
            point.z = (-3.f+6.f*phase-3.f*phase*phase)*(p0.z-originOffset.z) + 3.f*(2*phase - 4.f*phase + 3.f*phase*phase)*(cpWindow3[0].y-originOffset.z) + 3.f*(2.f*phase - 3.f*phase*phase)*(cpWindow3[1].y-originOffset.z) + 3.f*phase*phase*(cpWindow3[2].y-originOffset.z);
          }
          else
          {
            point.x = (6.f - 6.f*phase)*(p0.x-originOffset.x) + 3.f*(-2.f+6.f*phase)*(cpWindow1[0].y-originOffset.x) + 3.f*(2.f-6.f*phase)*(cpWindow1[1].y-originOffset.x)+6.f*phase*(cpWindow1[2].y-originOffset.x);
            point.y = (6.f - 6.f*phase)*(p0.y-originOffset.y) + 3.f*(-2.f+6.f*phase)*(cpWindow2[0].y-originOffset.y) + 3.f*(2.f-6.f*phase)*(cpWindow2[1].y-originOffset.y)+6.f*phase*(cpWindow2[2].y-originOffset.y);
            point.z = (6.f - 6.f*phase)*(p0.z-originOffset.z) + 3.f*(-2.f+6.f*phase)*(cpWindow3[0].y-originOffset.z) + 3.f*(2.f-6.f*phase)*(cpWindow3[1].y-originOffset.z)+6.f*phase*(cpWindow3[2].y-originOffset.z);
          }
          
          point+=(originOffset-originPos);

          minXrange = std::min<float>(minXrange, point.x);
          maxXrange = std::max<float>(maxXrange, point.x);
          minYrange = std::min<float>(minYrange, point.y);
          maxYrange = std::max<float>(maxYrange, point.y);
          minZrange = std::min<float>(minZrange, point.z);
          maxZrange = std::max<float>(maxZrange, point.z);

          pWindow1.push_back(Vector2<>(phase, point.x));
          pWindow2.push_back(Vector2<>(phase, point.y));
          pWindow3.push_back(Vector2<>(phase, point.z));
        }
      }

      //calculate zoomfactor
      float
        scaleFactorX = mini_window_width/std::max<>(maxXrange-minXrange,1.0f),
        scaleFactorX1 = mini_window_height/std::max<>(maxXrange-minXrange,1.0f),
        scaleFactorY = mini_window_height/std::max<>(maxYrange-minYrange,1.0f),
        scaleFactorY1 = mini_window_width/std::max<>(maxYrange-minYrange,1.0f),
        scaleFactorZ = mini_window_height/std::max<>(maxZrange-minZrange,1.0f);

      float colorX[] = {1.0f,0.0f,0.0f,1.0f}, colorY[] = {0.0f,1.0f,0.0f,1.0f}, colorZ[] = {0.0f,1.0f,1.0f,1.0f};
      if (tra2dWindows)
      {
        draw2dCurves(minXrange, minYrange, maxXrange, maxYrange, scaleFactorX, scaleFactorY, -minXrange, -minYrange,
          colorX, colorY, -mini_window_height, cpWindow1, pWindow1);
        draw2dCurves(minXrange, minZrange, maxXrange, maxZrange, scaleFactorX, scaleFactorZ, -minXrange, -minZrange, 
          colorX, colorZ, -mini_window_height*2-offsetBetween, cpWindow2, pWindow2);
        draw2dCurves(minYrange, minZrange, maxYrange, maxZrange, scaleFactorY1, scaleFactorZ, -minYrange, -minZrange,  
          colorY, colorZ, -mini_window_height*3-offsetBetween*2, cpWindow3, pWindow3);
      }
      else
      {
        float colorPhase[]={1.0f, 1.0f, 1.0f, 1.0f};
        draw2dCurves(0.0f, minXrange, 1.0f, maxXrange, mini_window_width, scaleFactorX1, 0.0f, -minXrange,
          colorPhase, colorX, -mini_window_height, cpWindow1, pWindow1);
        draw2dCurves(0.0f, minYrange, 1.0f, maxYrange, mini_window_width, scaleFactorY, 0.0f, -minYrange, 
          colorPhase, colorY, -mini_window_height*2-offsetBetween, cpWindow2, pWindow2);
        draw2dCurves(0.0f, minZrange, 1.0f, maxZrange, mini_window_width, scaleFactorZ, 0.0f, -minZrange, 
          colorPhase, colorZ, -mini_window_height*3-offsetBetween*2, cpWindow3, pWindow3);
      }
    }
    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();

    glDisable(GL_BLEND);
    glEnable(GL_DEPTH_TEST);
  }

  if(moveDrag || moveViewOfViewDragXY || moveViewOfViewDragXZ || moveViewOfViewDragYZ || moveViewOfViewDragXP || 
    moveViewOfViewDragYP || moveViewOfViewDragZP)
  {
    showPlane();
    Vector3<> point = parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber];
    glDisable(GL_DEPTH_TEST);
    glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
    glMatrixMode(GL_PROJECTION);
    glPushMatrix();
    glLoadIdentity();
    glOrtho(0.0, width, 0.0, height, 0.0, 0.2);
    glMatrixMode(GL_MODELVIEW);
    glPushMatrix();
    glLoadIdentity();

    glTranslatef((float)actualX-30, (float)(height-actualY-30), 0.0f);
    glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
    renderText(0.0,0.0,0.0, "point " + QString::number(selectedPoint.pointNumber), QFont("arial",10));
    glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
    renderText(0.0,-15.0,0.0, "x:" + QString::number(point.x), QFont("arial",10));
    glColor4f(0.0f, 1.0f, 0.0f, 1.0f);
    renderText(0.0,-30.0,0.0, "y:" + QString::number(point.y), QFont("arial",10));
    glColor4f(0.0f, 1.0f, 1.0f, 1.0f);
    renderText(0.0,-45.0,0.0, "z:" + QString::number(point.z), QFont("arial",10));

    glMatrixMode(GL_PROJECTION);
    glPopMatrix();

    glMatrixMode(GL_MODELVIEW);
    glPopMatrix();
    glEnable(GL_DEPTH_TEST);

  }
  glFlush();


}

void ViewBikeWidget::draw2dCurves(const float& minA, const float& minB, const float& maxA, const float& maxB, const float& scaleFactorA, const float& scaleFactorB, const float& transA, const float& transB, float *colorA, float *colorB,const float& window, const std::vector<Vector2<> >& cp, const std::vector<Vector2<> >& point)
{
  glPushMatrix();
  glTranslatef(0.0f, window, 0.0f);
  float ps[1], lw[1];
  glGetFloatv(GL_POINT_SIZE, ps);
  glGetFloatv(GL_LINE_WIDTH, lw);

  glScalef(scaleFactorA, scaleFactorB, 1.0f);

  //draw grid and axes
  glColor4fv(colorA);

  //draw zeroline
  if (minA <= 0.0f && maxA >= 0.0f) 
  {
    glLineWidth(4);
    glBegin(GL_LINES); 
    glVertex2f(transA,minB+transB);
    glVertex2f(transA,maxB+transB);
    glEnd();
    renderText(double(transA), double(minB+transB-15.0f/scaleFactorB),0.0 , QString::number(0), QFont("arial",8));
  }

  float tickA = floor(maxA-minA)/10.0f;
  glLineWidth(1);
  for (float w = ceil(minA/tickA)*tickA; w <= maxA+0.01; w+=tickA)
  { 
    if (fabs(w) > 0.001) //don't draw zeroline twice
    {
      glBegin(GL_LINES); 
      glVertex2f(w+transA,minB+transB);
      glVertex2f(w+transA,maxB+transB);
      glEnd();
      renderText(double(w+transA), double(minB+transB-15.0f/scaleFactorB),0.0 , QString::number(w), QFont("arial",8));
    }
  }

  glColor4fv(colorB);
  float tickB = floor(maxB-minB)/10.0f;

  if(minB <= 0.0f && maxB >= 0.0f)
  {
    glLineWidth(4);
    glBegin(GL_LINES);
    glVertex2f(minA+transA,transB);
    glVertex2f(maxA+transA,transB);
    glEnd();
    renderText((double)(minA+transA-30.0f/scaleFactorA) , (double)(transB),0.0, QString::number(0),  QFont("arial",8));
  }


  glLineWidth(1);
  for (float h=ceil(minB/tickB)*tickB; h <= maxB+0.01; h+=tickB)
  {
    if (fabs(h) > 0.001) //don't draw zeroline twice
    {
      glBegin(GL_LINES);
      glVertex2f(minA+transA,h+transB);
      glVertex2f(maxA+transA,h+transB);
      glEnd();
      renderText((double)(minA+transA-30.0f/scaleFactorA) , (double)(h+transB),0.0, QString::number(h),  QFont("arial",8));
    }
  }

  //draw controlPoints
  glColor4f(1.0,1.0,1.0,1.0);
  glPointSize(10);
  for (unsigned int k=0; k < cp.size(); k++)
  {
    glBegin(GL_POINTS);
    glVertex2f(cp[k].x+transA, cp[k].y+transB);
    glEnd();
  }

  //draw curve
  glPointSize(4);
  for (unsigned int k=0; k < point.size(); k++)
  {
    glBegin(GL_POINTS);
    glVertex2f(point[k].x+transA, point[k].y+transB);
    glEnd();
  }
  glPointSize(ps[0]);
  glLineWidth(lw[0]);
  glPopMatrix(); 
}

GLvoid ViewBikeWidget::setMatrix(const Vector3d& translation, const Matrix3d& rotation)
{
  GLdouble modelmatrix[16];
  modelmatrix[0] = rotation.col[0].v[0];
  modelmatrix[1] = rotation.col[0].v[1];
  modelmatrix[2] = rotation.col[0].v[2];
  modelmatrix[3] = modelmatrix[7] = modelmatrix[11] = 0.0f;
  modelmatrix[4] = rotation.col[1].v[0];
  modelmatrix[5] = rotation.col[1].v[1];
  modelmatrix[6] = rotation.col[1].v[2];
  modelmatrix[8] = rotation.col[2].v[0];
  modelmatrix[9] = rotation.col[2].v[1];
  modelmatrix[10] = rotation.col[2].v[2];
  modelmatrix[12] = translation.v[0];
  modelmatrix[13] = translation.v[1];
  modelmatrix[14] = translation.v[2];
  modelmatrix[15] = 1.0f;
  glMultMatrixd(modelmatrix);
}




void ViewBikeWidget::keyPressEvent(QKeyEvent* event)
{
  switch(event->key())
  {
  case Qt::Key_PageUp:
    event->accept();
    viewBike.renderer.zoom(0.05);
    updateGL();
    break;

  case Qt::Key_PageDown:
    event->accept();
    viewBike.renderer.zoom(-0.05);
    updateGL();
    break;

  case Qt::Key_Left:
    event->accept();   
    cameraPosOffset.v[0]-=0.1;
    targetPosOffset.v[0]-=0.1;
    updateGL();
    break;

  case Qt::Key_Right:
    event->accept();
    cameraPosOffset.v[0]+=0.1;
    targetPosOffset.v[0]+=0.1;
    updateGL();
    break;
  case Qt::Key_X:
    setDragPlane((int) YZ_PLANE);
    break;
  case Qt::Key_Y:
    setDragPlane((int) XZ_PLANE);
    break;
  case Qt::Key_Z:
    setDragPlane((int) XY_PLANE);
    break;

  default:
    QGLWidget::keyPressEvent(event);
    break;
  }
}

void ViewBikeWidget::wheelEvent(QWheelEvent* event)
{
  QWidget::wheelEvent(event);

  if(event->orientation() == Qt::Vertical)
  {
    viewBike.renderer.zoom(-0.05 * event->delta() / 160);
    updateGL();
    event->accept();
  }
}

void ViewBikeWidget::mouseMoveEvent(QMouseEvent* event)
{
  Vector3d newTarPos, camPos, camTarget, translationVec(0,0,0);
  viewBike.renderer.getCamera(camPos, camTarget);
  newTarPos = viewBike.renderer.getObjectPos();
  camPos -= camTarget;
  camPos += newTarPos;    
  viewBike.renderer.setCamera(camPos, newTarPos);

  actualX = event->x();  actualY = event->y();

  if((moveDrag || moveViewOfViewDragXY || moveViewOfViewDragXZ || moveViewOfViewDragYZ || moveViewOfViewDragXP || 
    moveViewOfViewDragYP || moveViewOfViewDragZP) && selectedPoint.limb > -1)
  {
    Vector3d vecFar, vecNear;
    gluUnProjectClick(event->x(), event->y(), vecFar, vecNear);
    Vector3d planeIntersection;
    Vector3<> p;
    if (!moveViewOfViewDragXY && !moveViewOfViewDragXZ && !moveViewOfViewDragYZ && !moveViewOfViewDragXP && !moveViewOfViewDragYP && !moveViewOfViewDragZP)
    {
      p = parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb] +
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber]
      - parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb];

      if (selectedPoint.limb == Phase::leftFootRot)
      {
        if (selectedPoint.xzRot)
        {
          Vector3d drawPos(0.f,150.f, 0.f);
          drawPos.rotateX(p.x);
          drawPos.rotateZ(p.z);
          p = Vector3<>(parameters.footOrigin.x + drawPos.v[0],parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z + drawPos.v[2]);
        }
        else
        {
          Vector3d drawPos2(150.f,0.f, 0.f);
          drawPos2.rotateY(p.y);
          drawPos2.rotateZ(p.z);
          p = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
        }
      }

      if (selectedPoint.limb == Phase::rightFootRot)
      {
        if (selectedPoint.xzRot)
        {
          Vector3d drawPos(0.f,-150.f, 0.f);
          drawPos.rotateX(p.x);
          drawPos.rotateZ(p.z);
          p = Vector3<>(parameters.footOrigin.x + drawPos.v[0], -parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z +drawPos.v[2]);
        }
        else
        {
          Vector3d drawPos2(150.f,0.f,0.f);
          drawPos2.rotateY(p.y);
          drawPos2.rotateZ(p.z);
          p = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], -parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
        }
      }

      Vector3d position((double)p.x, (double)p.y, (double)p.z);
      if(SimGeometry::intersectRayAndPlane(vecNear, vecFar,position,
        dragPlane, planeIntersection))
      {
        translationVec = planeIntersection - position;
        //this is buggy
        if (selectedPoint.limb == Phase::leftFootRot)
        {
          if (selectedPoint.xzRot)
          {
            double temp = -sin((translationVec.v[0]/150.));
            translationVec.v[0] = sin((translationVec.v[2]/150.));
            translationVec.v[1] = 0.;
            translationVec.v[2] = temp;
          }
          else
          {
            double temp = sin((translationVec.v[1]/150.));
            translationVec.v[0] = 0.;
            translationVec.v[1] = -sin((translationVec.v[2]/150.));
            translationVec.v[2] = temp;
          }
        }
        if (selectedPoint.limb == Phase::rightFootRot)
        {
          if (selectedPoint.xzRot)
          {
            double temp = -sin((translationVec.v[0]/-150.));
            translationVec.v[0] = sin((translationVec.v[2]/-150.));
            translationVec.v[1] = 0.;
            translationVec.v[2] = temp;
          }
          else
          {
            double temp = sin((translationVec.v[1]/150.));
            translationVec.v[0] = 0.;
            translationVec.v[1] = -sin((translationVec.v[2]/150.));
            translationVec.v[2] = temp;
          }
        }
      }
    }
    else
    {
      unsigned int width, height;
      viewBike.renderer.getSize(width, height);

      float mini_window_width = width/3.0f;
      float mini_window_height = (height-150.0f)/3.0f;

     float maxXrange=100, minXrange=-100, maxYrange=150, minYrange=-150, maxZrange=100, minZrange=-250;

      for (unsigned int k=0; k < NUM_OF_POINTS; k++)
      {
        Vector3<> controlPoint = parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][k];
        minXrange = std::min<float>(minXrange, controlPoint.x);
        maxXrange = std::max<float>(maxXrange, controlPoint.x);
        minYrange = std::min<float>(minYrange, controlPoint.y);
        maxYrange = std::max<float>(maxYrange, controlPoint.y);
        minZrange = std::min<float>(minZrange, controlPoint.z);
        maxZrange = std::max<float>(maxZrange, controlPoint.z);
      }
      //calculate zoomfactor
      float
        scaleFactorX = mini_window_width/std::max<>(maxXrange-minXrange,1.0f),
        scaleFactorX1 = mini_window_height/std::max<>(maxXrange-minXrange,1.0f),
        scaleFactorY = mini_window_height/std::max<>(maxYrange-minYrange,1.0f),
        scaleFactorY1 = mini_window_width/std::max<>(maxYrange-minYrange,1.0f),
        scaleFactorZ = mini_window_height/std::max<>(maxZrange-minZrange,1.0f);

      p = parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber];

      Vector3d windowVec;
      if (moveViewOfViewDragXY)
      {
        windowVec = Vector3d(((event->x()-(width-20.f-width/3.0f))/scaleFactorX)+minXrange, 
          ((event->y()- mini_window_height-20.0)/-scaleFactorY)+minYrange,0.0);
        p.z = 0.f;
      }
      if (moveViewOfViewDragXZ)
      {
        windowVec = Vector3d(((event->x()-(width-20.f-width/3.0f))/scaleFactorX)+minXrange, 
          0.0,((event->y()- mini_window_height*2-55.0)/-scaleFactorZ)+minZrange);
        p.y = 0.f;
      }
      if (moveViewOfViewDragYZ)
      {
        windowVec = Vector3d(0.0,((event->x()-(width-20.f-width/3.0f))/scaleFactorY1)+minYrange, 
          ((event->y()- mini_window_height*3-90.0)/-scaleFactorZ)+minZrange);
        p.x = 0.f;
      }
      if (moveViewOfViewDragXP)
      {
        windowVec = Vector3d(((event->y()- mini_window_height-20.0)/-scaleFactorX1)+minXrange,0.0,0.0);
        p.y = 0.f;
        p.z = 0.f;
      }
      if (moveViewOfViewDragYP)
      {
        windowVec = Vector3d(0.0,((event->y()- mini_window_height*2-55.0)/-scaleFactorY)+minYrange,0.0);
        p.x = 0.f;
        p.z = 0.f;
      }
      if (moveViewOfViewDragZP)
      {
        windowVec = Vector3d(0.0, 0.0,((event->y()- mini_window_height*3-90.0)/-scaleFactorZ)+minZrange);
        p.y = 0.f;
        p.x = 0.f;
      }

      translationVec = windowVec - Vector3d(p.x, p.y, p.z);
    }

    Vector3<> oldPoint =  parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber];

    parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x += (float)translationVec.v[0];
    parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y += (float)translationVec.v[1];
    parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z += (float)translationVec.v[2];

    if (selectedPoint.limb == Phase::leftFootRot || selectedPoint.limb == Phase::rightFootRot)
    {
      if (parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x > pi_4)
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x = pi_4;
      if (parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y > pi_4)
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y = pi_4;
      if (parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z > pi_4)
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z = pi_4;
      if (parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x < -pi_4)
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x = -pi_4;
      if (parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y < -pi_4)
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y = -pi_4;
      if (parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z < -pi_4)
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z = -pi_4;
    }

    //clipping
    clipCurve(translationVec);

    // �quidistant und kolinear p3-p2 = q1-q0 
    //p3 = q0 => p3 - p2 = q1 - p3 => 2*p3 - p2 = q1;
    switch(selectedPoint.pointNumber)
    {
    case 0: 
      //p2
      if (selectedPoint.phaseNumber > 0)
      {

        float factor =  (float)parameters.phaseParameters[selectedPoint.phaseNumber-1].duration /  
          (float)parameters.phaseParameters[selectedPoint.phaseNumber].duration;

        parameters.phaseParameters[selectedPoint.phaseNumber-1].controlPoints[selectedPoint.limb][1] = 
          parameters.phaseParameters[selectedPoint.phaseNumber-1].controlPoints[selectedPoint.limb][2] - 
          parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][0];

        parameters.phaseParameters[selectedPoint.phaseNumber-1].controlPoints[selectedPoint.limb][1] *= factor; 

        parameters.phaseParameters[selectedPoint.phaseNumber-1].controlPoints[selectedPoint.limb][1] += 
          parameters.phaseParameters[selectedPoint.phaseNumber-1].controlPoints[selectedPoint.limb][2];
      }
      break;
    case 1:
      //q1
      if (selectedPoint.phaseNumber <  parameters.numberOfPhases-1)
      {

        parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[selectedPoint.limb][0] = 
          parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][2] - 
          parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][1];

        float factor =  (float)parameters.phaseParameters[selectedPoint.phaseNumber+1].duration /  
          (float)parameters.phaseParameters[selectedPoint.phaseNumber].duration;
        parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[selectedPoint.limb][0] *= factor; 

        parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[selectedPoint.limb][0] += 
          parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][2];
      }
      break;
    case 2:
      //p2
      parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][1].x += (float)translationVec.v[0];
      parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][1].y += (float)translationVec.v[1];
      parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][1].z += (float)translationVec.v[2];
      //q1
      if (selectedPoint.phaseNumber <  parameters.numberOfPhases-1)
      {
        parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[selectedPoint.limb][0].x += (float)translationVec.v[0];
        parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[selectedPoint.limb][0].y += (float)translationVec.v[1];
        parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[selectedPoint.limb][0].z += (float)translationVec.v[2];
      }
      break;
    default:
      break;
    }
    parameters.init();
    if (selectedPoint.phaseNumber > -1)
      fillModelWithPhaseData(phaseTab[selectedPoint.phaseNumber]->invisibleRootItem(),selectedPoint.phaseNumber);
    event->accept();
    updateGL();
  }
  else
  {
    if(viewBike.renderer.moveDrag(event->x(), event->y()))
    {
      event->accept();
      if (selectedPoint.phaseNumber > -1)
        fillModelWithPhaseData(phaseTab[selectedPoint.phaseNumber]->invisibleRootItem(),selectedPoint.phaseNumber);
      updateGL();
    }
  }
}

void ViewBikeWidget::showPlane()
{
  Vector3<> p1, p2, p3, p4;
  Vector3<> zero1, zero2, zero3, zero4;

  Vector3<> point = parameters.phaseParameters[selectedPoint.phaseNumber].originPos[selectedPoint.limb]
  + parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber] 
  - parameters.phaseParameters[selectedPoint.phaseNumber].originOffset[selectedPoint.limb];

  if (selectedPoint.limb == Phase::leftFootRot)
  {
    if (selectedPoint.xzRot)
    {
      Vector3d drawPos(0.f,150.f, 0.f);
      drawPos.rotateX(point.x);
      drawPos.rotateZ(point.z);
      point = Vector3<>(parameters.footOrigin.x + drawPos.v[0],parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z + drawPos.v[2]);
    }
    else
    {
      Vector3d drawPos2(150.f,0.f, 0.f);
      drawPos2.rotateY(point.y);
      drawPos2.rotateZ(point.z);
      point = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
    }
  }

  if (selectedPoint.limb == Phase::rightFootRot)
  {
    if (selectedPoint.xzRot)
    {
      Vector3d drawPos(0.f,-150.f, 0.f);
      drawPos.rotateX(point.x);
      drawPos.rotateZ(point.z);
      point = Vector3<>(parameters.footOrigin.x + drawPos.v[0], -parameters.footOrigin.y + drawPos.v[1], parameters.footOrigin.z +drawPos.v[2]);
    }
    else
    {
      Vector3d drawPos2(150.f,0.f,0.f);
      drawPos2.rotateY(point.y);
      drawPos2.rotateZ(point.z);
      point = Vector3<>(parameters.footOrigin.x + drawPos2.v[0], -parameters.footOrigin.y + drawPos2.v[1], parameters.footOrigin.z + drawPos2.v[2]);
    }
  }

  if (dragPlane.v[0] == 1) //YZ
  {
    p1 = Vector3<>(point.x, -300.f, -300.f);
    p2 = Vector3<>(point.x, 300.f, -300.f);
    p3 = Vector3<>(point.x, 300.f, 300.f);
    p4 = Vector3<>(point.x, -300.f, 300.f);
    zero1 = Vector3<>(point.x, 0.f, -300.f);
    zero2 = Vector3<>(point.x, 0.f, 300.f);
    zero3 = Vector3<>(point.x, -300.f, 0.f);
    zero4 = Vector3<>(point.x, 300.f, 0.f);

  }
  else if (dragPlane.v[1] == 1) //XZ
  {
    p1 = Vector3<>( -300.f,point.y, -300.f);
    p2 = Vector3<>( 300.f,point.y, -300.f);
    p3 = Vector3<>( 300.f,point.y, 300.f);
    p4 = Vector3<>( -300.f,point.y, 300.f);
    zero1 = Vector3<>(0.f, point.y, -300.f);
    zero2 = Vector3<>(0.f, point.y, 300.f);
    zero3 = Vector3<>(-300.f, point.y,  0.f);
    zero4 = Vector3<>(300.f, point.y, 0.f);
  }
  else if (dragPlane.v[2] == 1) //XY
  {
    p1 = Vector3<>( -300.f,-300.f, point.z);
    p2 = Vector3<>( 300.f,-300.f,point.z);
    p3 = Vector3<>( 300.f,300.f,point.z);
    p4 = Vector3<>( -300.f,300.f,point.z);
    zero1 = Vector3<>(0.f, -300.f, point.z);
    zero2 = Vector3<>(0.f, 300.f, point.z);
    zero3 = Vector3<>(-300.f, 0.f, point.z);
    zero4 = Vector3<>(300.f, 0.f, point.z);
  }

  glPushMatrix();
  setMatrix(viewBike.origin->getPosition(), originRot);
  glScaled(0.001, 0.001, 0.001);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
  glEnable(GL_BLEND);
  glColor4f(0.f, 0.5f, 0.12f, 0.6f);
  glBegin(GL_QUADS);
  glVertex3f(p1.x, p1.y, p1.z);
  glVertex3f(p2.x, p2.y, p2.z);
  glVertex3f(p3.x, p3.y, p3.z);
  glVertex3f(p4.x, p4.y, p4.z);
  glEnd();
  glColor4f(1.f, 1.f, 1.f, 1.f);

  float lw[1];
  glGetFloatv(GL_LINE_WIDTH, lw);
  glLineWidth(4);
  glBegin(GL_LINES);
  glVertex3f(zero1.x, zero1.y, zero1.z);
  glVertex3f(zero2.x, zero2.y, zero2.z);
  glEnd();
  glBegin(GL_LINES);
  glVertex3f(zero3.x, zero3.y, zero3.z);
  glVertex3f(zero4.x, zero4.y, zero4.z);
  glEnd();
  glLineWidth(lw[0]);
  glDisable(GL_BLEND);
  glPopMatrix();
  glFlush();
}

void ViewBikeWidget::clipCurve(Vector3d& translationVec)
{
  Vector3<> positions[2];
  parameters.init();
  for (float t=0.f; t < 1.01f; t+=0.01f)
  {
    double leg0, leg1, leg2, leg3, leg4, leg5;

    if (selectedPoint.limb == Phase::leftFootTra) 
    {

      positions[0] = parameters.getPosition(t, selectedPoint.phaseNumber, Phase::leftFootTra);
      positions[1] = parameters.getPosition(t, selectedPoint.phaseNumber, Phase::leftFootRot);

      bool leftLegTooShort = calcLegJoints(positions[0], positions[1], true, leg0, leg1, leg2, leg3, leg4, leg5);

      if (clipLegJointsWithLimits(leg1, leg2, leg3, JointData::LHipYawPitch) || leftLegTooShort)
      {
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x -= (float)translationVec.v[0];
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y -= (float)translationVec.v[1];
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z -= (float)translationVec.v[2];
        translationVec = Vector3d(0.,0.,0.);
      }
    }
    if (selectedPoint.limb == Phase::rightFootTra) 
    {

      positions[0] = parameters.getPosition(t, selectedPoint.phaseNumber, Phase::rightFootTra);
      positions[1] = parameters.getPosition(t, selectedPoint.phaseNumber, Phase::rightFootRot);

      bool rightLegTooShort = calcLegJoints(positions[0], positions[1], false, leg0, leg1, leg2, leg3, leg4, leg5);
      if (clipLegJointsWithLimits(leg1, leg2, leg3, JointData::RHipYawPitch) || rightLegTooShort)
      { 
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].x -= (float)translationVec.v[0];
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].y -= (float)translationVec.v[1];
        parameters.phaseParameters[selectedPoint.phaseNumber].controlPoints[selectedPoint.limb][selectedPoint.pointNumber].z -= (float)translationVec.v[2];
        translationVec = Vector3d(0.,0.,0.);
      }
    }
  }
}


bool ViewBikeWidget::calcLegJoints(const Vector3<>& footPos, const Vector3<>& footRotAng, const bool& left, double& leg0, double& leg1, double& leg2, double& leg3, double& leg4, double& leg5)
{
  float sign = (left) ? 1.f : -1.f;
  bool legTooShort = false;

  Vector3<> anklePos; //FLOAT
  anklePos = Vector3<>(footPos.x,  footPos.y, footPos.z+viewBike.robotDimensions.heightLeg5Joint);

  anklePos -= Vector3<>(0.f, sign * (viewBike.robotDimensions.lengthBetweenLegs / 2.f), 0.f);

  RotationMatrix rotateBecauseOfHip = RotationMatrix().rotateZ(sign*footRotAng.z).rotateX(-sign*pi_4);
  Vector3<> rotatedFootRotAng;


  anklePos = rotateBecauseOfHip * anklePos; 
  leg0 = footRotAng.z;
  rotatedFootRotAng = rotateBecauseOfHip * footRotAng;

  leg2 = -atan2f(anklePos.x, Vector2<>(anklePos.y, anklePos.z).abs());

  float diagonal = anklePos.abs();

  // upperLegLength, lowerLegLength, and diagonal form a triangle, use cosine theorem
  float a1 = (viewBike.robotDimensions.upperLegLength * viewBike.robotDimensions.upperLegLength - 
    viewBike.robotDimensions.lowerLegLength * viewBike.robotDimensions.lowerLegLength + diagonal * diagonal) / 
    (2 * viewBike.robotDimensions.upperLegLength * diagonal);
  a1 = fabs(a1) > 1.f ? 0 : acos(a1);

  float a2 = (viewBike.robotDimensions.upperLegLength * viewBike.robotDimensions.upperLegLength +
    viewBike.robotDimensions.lowerLegLength * viewBike.robotDimensions.lowerLegLength - diagonal * diagonal) / 
    (2 * viewBike.robotDimensions.upperLegLength * viewBike.robotDimensions.lowerLegLength);
  const Range<float> clipping(-1.f, 1.f);
  if(!clipping.isInside(a2))
  {
    legTooShort = true;
  }
  a2 = fabs(a2) > 1.f ? pi : acos(a2);



  leg1 = (-sign*atan2(anklePos.y, fabs(anklePos.z))) -(pi/4);
  leg2 -= a1;
  leg3 = pi - a2;

  RotationMatrix footRot = RotationMatrix().rotateX(leg1 * -sign).rotateY(leg2 + leg3);
  footRot = footRot.invert() * rotateBecauseOfHip;
  leg5 = asin(-footRot[2].y) * -sign;
  leg4 = -atan2(footRot[2].x, footRot[2].z) * -1;
  leg4 += footRotAng.y;
  leg5 += footRotAng.x;

  return legTooShort;
}


bool ViewBikeWidget::clipLegJointsWithLimits(double& leg1, double& leg2, double& leg3, const JointData::Joint& joint)
{
  bool clipped = false;

  //clip Leg1
  if (viewBike.jointCalibration.joints[joint + 1].maxAngle < leg1) 
  {
    leg1 = viewBike.jointCalibration.joints[joint + 1].maxAngle;
    clipped = true;
  }
  if (viewBike.jointCalibration.joints[joint + 1].minAngle > leg1) 
  {
    leg1 = viewBike.jointCalibration.joints[joint + 1].minAngle;
    clipped = true;
  }

  //clip Leg2
  if (viewBike.jointCalibration.joints[joint + 2].maxAngle < leg2) 
  {
    leg2 = viewBike.jointCalibration.joints[joint + 2].maxAngle;
    clipped = true;
  }
  if (viewBike.jointCalibration.joints[joint + 2].minAngle > leg2) 
  {
    leg2 = viewBike.jointCalibration.joints[joint + 2].minAngle;
    clipped = true;
  }

  //clip Leg3
  if (viewBike.jointCalibration.joints[joint + 3].maxAngle < leg3) 
  {
    leg3 = viewBike.jointCalibration.joints[joint + 3].maxAngle;
    clipped = true;
  }  
  if (viewBike.jointCalibration.joints[joint + 3].minAngle > leg3) 
  {
    leg3 = viewBike.jointCalibration.joints[joint + 3].minAngle;
    clipped = true;
  }
  return clipped;
}

Vector3<> ViewBikeWidget::calcFootPos(const double& leg0, const double& leg1, const double& leg2, const double& leg3, const double& leg4, const double& leg5, JointData::Joint joint)
{
  Pose3D footPos(0,0,0);
  footPos.translate(0, 0, -viewBike.robotDimensions.heightLeg5Joint);
  float sign = joint == JointData::LHipYawPitch ? 1.f : -1.f;
  footPos.translate(0, sign*(viewBike.robotDimensions.lengthBetweenLegs/2),0); 
  footPos.rotateX(-pi_4*sign);
  footPos.rotateZ(-leg0*sign);
  footPos.rotateX(pi_4*sign); 
  footPos.rotateX(-leg1*sign);
  footPos.rotateY(leg2);
  footPos.translate(0, 0, -viewBike.robotDimensions.upperLegLength);
  footPos.rotateY(leg3);
  footPos.translate(0, 0, -viewBike.robotDimensions.lowerLegLength);
  return Vector3<>(footPos.translation.x, footPos.translation.y, footPos.translation.z);
}

void ViewBikeWidget::mousePressEvent(QMouseEvent* event)
{
  QWidget::mousePressEvent(event);

  if(event->button() == Qt::LeftButton)
  {
    if (clickControlPoint(event->x(), event->y()))
    {
      undo.push_back(parameters);
      actualX = event->x();
      actualY = event->y();
      if(undo.size() >= 20)
      {
        undo.erase(undo.begin());
      }
      event->accept();
      updateGL();
    }
    if(viewBike.renderer.startDrag(event->x(), event->y(), QApplication::keyboardModifiers() & Qt::ShiftModifier ? ObjectRenderer::DRAG_ROTATE : ObjectRenderer::DRAG_NORMAL))
    {
      event->accept();
      updateGL();
    }
  }

  if(event->button() == Qt::RightButton)
  {
    actualX = event->x();
    actualY = event->y();
    event->accept();
  }
}

void ViewBikeWidget::mouseReleaseEvent(QMouseEvent* event)
{
  QWidget::mouseReleaseEvent(event);

  if(moveDrag || moveViewOfViewDragXY || moveViewOfViewDragXZ || moveViewOfViewDragYZ || moveViewOfViewDragXP || moveViewOfViewDragYP || moveViewOfViewDragZP)
  {
    moveDrag = false;
    moveViewOfViewDragXY = false;
    moveViewOfViewDragXZ = false;
    moveViewOfViewDragYZ = false;
    moveViewOfViewDragXP = false;
    moveViewOfViewDragYP = false;
    moveViewOfViewDragZP = false;
    event->accept();
    updateGL();
  }
  else
  {
    if(viewBike.renderer.releaseDrag(event->x(), event->y()))
    {
      event->accept();
      updateGL();
    }
  }
}

void ViewBikeWidget::contextMenuEvent(QContextMenuEvent* event)
{
  event->accept();
  QMenu menu;  
  menu.addMenu(bikeMenuBar->dragPlaneMenu);
  menu.addSeparator();
  menu.addMenu(bikeMenuBar->phaseMenu);
  actualX = event->x();
  actualY = event->y();
  menu.exec(mapToGlobal(QPoint(event->x(), event->y())));
}

void ViewBikeWidget::splitPhase()
{
  int phaseNumber = selectedPoint.phaseNumber;
  if(phaseNumber > -1)
  {
    glPushMatrix();
    setMatrix(viewBike.origin->getPosition(), originRot);
    glScaled(0.001, 0.001, 0.001);
    GLint viewport[4];
    glGetIntegerv(GL_VIEWPORT, viewport);
    GLdouble modelview[16];
    glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
    GLdouble projection[16];
    glGetDoublev(GL_PROJECTION_MATRIX, projection);

    actualY = (float) viewport[3] - actualY;

    for (float phase=0.f; phase < 1.f; phase+=0.01f)
    {
      Vector3<> position = parameters.getPosition(phase,phaseNumber,selectedPoint.limb);

      GLdouble winX, winY, winZ;
      gluProject(position.x, position.y, position.z, modelview, projection, viewport, &winX, &winY, &winZ);

      double localDist = sqrt((actualX-winX)*(actualX-winX) + (actualY-winY)*(actualY-winY));
      if (localDist < 20.0)
      {
        makeNewPhaseWithModelAndTree(phaseNumber+1);
        parameters.phaseParameters[phaseNumber+1].duration = parameters.phaseParameters[phaseNumber+1].duration;
        for (int limb=0; limb < Phase::numOfLimbs; limb++)
        {
          Vector3<> endPoint = parameters.getPosition(phase, phaseNumber,limb) 
            + parameters.phaseParameters[phaseNumber].originOffset[limb] - 
            parameters.phaseParameters[phaseNumber].originPos[limb];

          parameters.phaseParameters[phaseNumber+1].controlPoints[limb][NUM_OF_POINTS-1] = 
            parameters.phaseParameters[phaseNumber].controlPoints[limb][NUM_OF_POINTS-1];
          parameters.phaseParameters[phaseNumber+1].controlPoints[limb][0] = endPoint;
          if (phaseNumber+2 < parameters.numberOfPhases) 
          {
            float factor =  (float)parameters.phaseParameters[selectedPoint.phaseNumber+2].duration /  
              (float)parameters.phaseParameters[selectedPoint.phaseNumber+1].duration;

            parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[limb][1] = 
              parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[limb][2] - 
              parameters.phaseParameters[selectedPoint.phaseNumber+2].controlPoints[limb][0];

            parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[limb][1] *= factor; 

            parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[limb][1] += 
              parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[limb][2];
          }
          else
          {
            parameters.phaseParameters[selectedPoint.phaseNumber+1].controlPoints[limb][1] = 
              parameters.phaseParameters[phaseNumber+1].controlPoints[limb][NUM_OF_POINTS-1]; 
          }

          parameters.phaseParameters[phaseNumber].controlPoints[limb][NUM_OF_POINTS-1] = endPoint;
          parameters.phaseParameters[phaseNumber].controlPoints[limb][1] = endPoint;
        }

        connect(phaseTab[phaseNumber+1], SIGNAL(itemChanged(QStandardItem*)), this, SLOT(updatePhaseParameters(QStandardItem*)));
        tabber->insertTab(phaseNumber+2, treeView[phaseNumber], QString("split"));
        parameters.init();
        tabber->setCurrentIndex(phaseNumber+2);
        glPopMatrix();
        return;
      }
    }
    glPopMatrix();
  }
}


Vector3<> ViewBikeWidget::calculateFootPos(const JointData& jointData, JointData::Joint joint)
{
  Pose3D footPos;
  float sign = joint == JointData::LHipYawPitch ? 1 : -1;

  footPos.translate(0, sign*(viewBike.robotDimensions.lengthBetweenLegs/2),0); 
  footPos.rotateX(-pi_4*sign);
  footPos.rotateZ(-jointData.angles[joint]*sign);
  footPos.rotateX(pi_4*sign); 
  footPos.rotateX(-jointData.angles[joint + 1]*sign);
  footPos.rotateY(jointData.angles[joint + 2]);
  footPos.translate(0, 0, -viewBike.robotDimensions.upperLegLength);
  footPos.rotateY(jointData.angles[joint + 3]);
  footPos.translate(0, 0, -viewBike.robotDimensions.lowerLegLength);
  footPos.translate(0, 0, -viewBike.robotDimensions.heightLeg5Joint);

  return Vector3<>(footPos.translation.x, footPos.translation.y, footPos.translation.z);
}

Vector3<> ViewBikeWidget::calculateHandPos(const JointData& jointData, JointData::Joint joint)
{
  Pose3D handPos;
  double sign = joint == JointData::LShoulderPitch ? 1 : -1;

  handPos.translate(viewBike.robotDimensions.armOffset.x, sign*(viewBike.robotDimensions.armOffset.y), viewBike.robotDimensions.armOffset.z);  
  handPos.rotateY(-jointData.angles[joint]);
  handPos.rotateX(sign*jointData.angles[joint + 1]);
  handPos.translate(0,viewBike.robotDimensions.upperArmLength*sign,0);
  handPos.rotateX(sign*pi_2);
  handPos.rotateX(sign*-pi_2);
  handPos.rotateY(jointData.angles[joint+2]);
  handPos.rotateX(sign*jointData.angles[joint + 3]);
  handPos.translate(0,viewBike.robotDimensions.lowerArmLength*sign,0);
  handPos.rotateX(sign*pi_2);

  return Vector3<>(handPos.translation.x, handPos.translation.y, handPos.translation.z);
}



