/** 
* @file BH2010StableBIKESymbols.cpp
*
* Implementation of class BH2010StableBIKESymbols.
*
* @author Judith Mller
* @author Apurva HP
*/

#include "BH2010StableBIKESymbols.h"
#include "Tools/Xabsl/GT/GTXabslTools.h"
#include "Modules/MotionControl/BIKEParameters.h"

PROCESS_WIDE_STORAGE BH2010StableBIKESymbols* BH2010StableBIKESymbols::theInstance = 0;

void BH2010StableBIKESymbols::registerSymbols(xabsl::Engine& engine)
{

  char s[256];
  char* dest;
 
  dest = s + sprintf(s, "motion.bike.");
  for(int i = 0; i < BikeRequest::numOfBMotions; ++i)
  {    
    getXabslString(dest, BikeRequest::getBMotionName((BikeRequest::BMotionID)i));
    engine.registerEnumElement("motion.bike",s,i);
  }
  engine.registerEnumeratedOutputSymbol("motion.bike","motion.bike", &setKick, &getKick);

  engine.registerBooleanOutputSymbol("motion.bike.mirror", &motionRequest.bikeRequest.mirror);
  engine.registerBooleanOutputSymbol("motion.bike.dynamical", &motionRequest.bikeRequest.dynamical);

  engine.registerEnumeratedInputSymbol("executed_motion.bike","motion.bike", (const int*)&motionInfo.bikeRequest.bMotionType);
  engine.registerBooleanInputSymbol("executed_motion.bike.mirror", &motionInfo.bikeRequest.mirror);

  engine.registerBooleanInputSymbol("motion.bike.is_leaving_possible", &bikeEngineOutput.isLeavingPossible);

  engine.registerBooleanInputSymbol("motion.nao_try_kick", &getNaoTryKick);
  engine.registerBooleanInputSymbolDecimalParameter("motion.nao_try_kick", "motion.nao_try_kick.x", &ballX);
  engine.registerBooleanInputSymbolDecimalParameter("motion.nao_try_kick", "motion.nao_try_kick.y", &ballY);
  

}

void BH2010StableBIKESymbols::setKick(int kick)
{
  theInstance->motionRequest.bikeRequest.bMotionType = BikeRequest::BMotionID(kick);
}

int BH2010StableBIKESymbols::getKick()
{
  return int(theInstance->motionRequest.bikeRequest.bMotionType);
}

 bool BH2010StableBIKESymbols::getNaoTryKick()
{
  float x = theInstance->ballX;
  float y = theInstance->ballY;
  
  if( x < 10 || x > 190 || fabs(y) < 30.f || fabs(y) > 100.f ) return false;
  
  if(x > 110.f) x = 110.f;
  if(y > 0.f)
  {
    theInstance->motionRequest.bikeRequest.mirror = true;
  }
  else
  {
    theInstance->motionRequest.bikeRequest.mirror = false;
  }

  if ( theInstance->motionRequest.bikeRequest.mirror == true) y = -y;

  theInstance->motionRequest.bikeRequest.bMotionType = BikeRequest::trykick;
  
  Vector3<> strikeOut, kickTo, motionDirection(0,0,0);

  //kick power factor 

    strikeOut = Vector3<>(-90,y-40.f,-170.f);
    kickTo = Vector3<>(x+20,y-40.f,-170.f);
    
  
  if(!theInstance->motionRequest.bikeRequest.dynPoints.empty())
  {
    theInstance->motionRequest.bikeRequest.dynPoints.clear();
  }
  
    DynPoint dynRFoot2(Phase::rightFootTra, 2, strikeOut, motionDirection,  Vector3<> (0.f,0.f,0.f)), //strikeout
             dynRFoot3(Phase::rightFootTra, 3, kickTo, motionDirection,  Vector3<> (0.f,0.f,0.f)); //kickto
    //rFoot in Phase2
    theInstance->motionRequest.bikeRequest.dynPoints.push_back(dynRFoot2);
    //rFoot in Phase3
    theInstance->motionRequest.bikeRequest.dynPoints.push_back(dynRFoot3);

    theInstance->motionRequest.bikeRequest.dynamical = true;
  
  
  return true;
 }


