/** 
* @file BH2010StableBIKESymbols.h
*
* Declaration of class BH2010StableBIKESymbols.
*
* @author Judith Mller
* @author Apurva HP
*/

#ifndef __BH2010StableBIKESymbols_h_
#define __BH2010StableBIKESymbols_h_

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/MotionControl/BikeEngineOutput.h"
#include "Representations/MotionControl/MotionRequest.h"
#include "Representations/MotionControl/MotionInfo.h"

/**
* The Xabsl symbols that are defined in "bike_symbols.xabsl"
*
* @author Judith Mller
*/ 
class BH2010StableBIKESymbols : public Symbols
{
public:
  /* Constructor */
  BH2010StableBIKESymbols(MotionRequest& motionRequest, const BikeEngineOutput& bikeEngineOutput, const MotionInfo& motionInfo) : 
    motionRequest(motionRequest),
    bikeEngineOutput(bikeEngineOutput),
    motionInfo(motionInfo)
  {
    theInstance = this;
  }
  MotionRequest& motionRequest; /**< A reference to the MotionRequest */
  const BikeEngineOutput& bikeEngineOutput; /**< A reference to the bikeEngineOutput (to get when a bmotion is leavable) */
  const MotionInfo& motionInfo;

  /** Registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  /** Update function, called every cycle */
  void update(){};

private:
  /** Points to the only instance of this class in this process or is 0 if there is none. */
  PROCESS_WIDE_STORAGE_STATIC BH2010StableBIKESymbols* theInstance; 
 
  static void setKick(int kick);
  static int getKick();

  static bool getNaoTryKick();

  float ballX, ballY, hitZ, destAng, strikeOutFactor;
  bool mirror;
};


#endif // __LEDSymbols_h_
