/** 
* @file BH2010StableGameSymbols.cpp
*
* Implementation of class BH2010StableGameSymbols.
*
* @author Judith Mller
*/

#include "BH2010StableGameSymbols.h"

PROCESS_WIDE_STORAGE BH2010StableGameSymbols* BH2010StableGameSymbols::theInstance = 0;

void BH2010StableGameSymbols::registerSymbols(xabsl::Engine& engine)
{
  engine.registerEnumElement("game.team_color", "game.team_color.blue", TEAM_BLUE);
  engine.registerEnumElement("game.team_color", "game.team_color.red", TEAM_RED);
  engine.registerEnumElement("game.team_color", "game.team_color.none", TEAM_NONE);

  engine.registerEnumElement("game.state", "game.state._initial", STATE_INITIAL);
  engine.registerEnumElement("game.state", "game.state.ready", STATE_READY);
  engine.registerEnumElement("game.state", "game.state.set", STATE_SET);
  engine.registerEnumElement("game.state", "game.state.playing", STATE_PLAYING);
  engine.registerEnumElement("game.state", "game.state.finished", STATE_FINISHED);

  engine.registerEnumElement("game.secondary_state", "game.secondary_state.normal", STATE2_NORMAL);
  engine.registerEnumElement("game.secondary_state", "game.secondary_state.penaltyshoot", STATE2_PENALTYSHOOT);

  engine.registerEnumElement("game.penalty", "game.penalty.none", PENALTY_NONE);
  engine.registerEnumElement("game.penalty", "game.penalty.ball_holding", PENALTY_SPL_BALL_HOLDING);
  engine.registerEnumElement("game.penalty", "game.penalty.player_pushing", PENALTY_SPL_PLAYER_PUSHING);
  engine.registerEnumElement("game.penalty", "game.penalty.obstruction", PENALTY_SPL_OBSTRUCTION);
  engine.registerEnumElement("game.penalty", "game.penalty.inactive_player", PENALTY_SPL_INACTIVE_PLAYER);
  engine.registerEnumElement("game.penalty", "game.penalty.illegal_defender", PENALTY_SPL_ILLEGAL_DEFENDER);
  engine.registerEnumElement("game.penalty", "game.penalty.leaving_the_field", PENALTY_SPL_LEAVING_THE_FIELD);
  engine.registerEnumElement("game.penalty", "game.penalty.playing_with_hands", PENALTY_SPL_PLAYING_WITH_HANDS);
  engine.registerEnumElement("game.penalty", "game.penalty.request_for_pickup", PENALTY_SPL_REQUEST_FOR_PICKUP);
  engine.registerEnumElement("game.penalty", "game.penalty.manual", PENALTY_MANUAL);
  engine.registerEnumeratedOutputSymbol("game.team_color", "game.team_color", &setOwnTeamColor ,&getOwnTeamColor);

  engine.registerEnumeratedInputSymbol("game.opponent_team_color", "game.team_color", &getOppTeamColor);

  engine.registerEnumeratedInputSymbol("game.player_number", "role.role", &getPlayerNumber);

  engine.registerEnumeratedOutputSymbol("game.penalty", "game.penalty", &setPenalty, &getPenalty);

  engine.registerDecimalInputSymbol("game.penalty.remaining_time", &getPenaltyRemainingTime);

  engine.registerEnumeratedOutputSymbol("game.state", "game.state", &setState ,&getState);

  engine.registerEnumeratedOutputSymbol("game.kickoff_team", "game.team_color", &setKickoffTeam,&getKickoffTeam);

  engine.registerEnumeratedOutputSymbol("game.secondary_state", "game.secondary_state", &setSecondaryState,&getSecondaryState);

  engine.registerDecimalInputSymbol("game.own_score", &getOwnScore);

  engine.registerDecimalInputSymbol("game.opponent_score", &getOpponentScore);

  engine.registerDecimalInputSymbol("game.remaining_time", &getRemainingTime);

  engine.registerBooleanOutputSymbol("game.startup", &startup);

  engine.registerDecimalInputSymbol("game.time_since_playing_started", &staticGetTimeSincePlayingState);

  engine.registerBooleanInputSymbol("game.kickoff_in_progress", &behaviorData.kickoffInProgress);

  engine.registerEnumeratedOutputSymbol("game.drop_in_team", "game.team_color", &setDropInTeam,&getDropInTeam);

  engine.registerDecimalOutputSymbol("game.drop_in_time", &setDropInTime,&getDropInTime);

  engine.registerBooleanInputSymbol("game.kickoff_changed", &kickOffChanged);

  engine.registerDecimalInputSymbol("game.time_since_last_penalization", &getTimeSinceLastPenalization);

  engine.registerDecimalInputSymbol("game.time_since_initial", &getTimeSinceInitial);

  engine.registerDecimalInputSymbol("game.time_since_last_package_received", &getTimeSinceLastPackageReceived);

    engine.registerBooleanInputSymbol("game.disable_pre_initial", &disablePreInitialState);
}
void BH2010StableGameSymbols::init()
{
    kickOffTeamBegin = gameInfo.kickOffTeam;
    timeSincePen2Play = 0;

    #ifdef TARGET_SIM
  disablePreInitialState = true;
#else
  disablePreInitialState = Global::getSettings().recover;
#endif
}

void BH2010StableGameSymbols::update()
{
  if(gameInfo.state == STATE_INITIAL)
  {
    lastTimeInInitial = SystemCall::getCurrentSystemTime();
  }
  else if(gameInfo.state == STATE_PLAYING && lastGameState != STATE_PLAYING)
  {
    timeStamp = frameInfo.time;
    behaviorData.kickoffInProgress = gameInfo.kickOffTeam == ownTeamInfo.teamColor;
    counter = 0;
  }
  lastGameState = gameInfo.state;
  if(behaviorData.kickoffInProgress && frameInfo.getTimeSince(ballModel.timeWhenLastSeen) == 0)
  {
    if(ballModel.lastPerception.getPositionInFieldCoordinates(robotPose).abs() > fieldDimensions.centerCircleRadius + 100)
      counter++;
    else if(counter != 0)
      counter--;
    if(counter >= 10)
     behaviorData.kickoffInProgress = false;
  }
  if(ownTeamInfo.players[0].secsTillUnpenalised > 0)
  {
    timeSincePen2Play = frameInfo.time;
  }
  if(gameInfo.state == STATE_READY && gameInfo.kickOffTeam == kickOffTeamBegin)
  {
    kickOffChanged = false;
  }else if(gameInfo.state == STATE_READY && gameInfo.kickOffTeam != kickOffTeamBegin)
  {    
    kickOffChanged = true;
    kickOffTeamBegin = gameInfo.kickOffTeam;
  }
}

float BH2010StableGameSymbols::getTimeSincePlayingState()
{
  return (float) frameInfo.getTimeSince(timeStamp);
}
