/** 
* @file BH2010StableHeadSymbols.cpp
* Implementation of class BH2010StableHeadSymbols.
* @author Colin Graf
*/

#include "BH2010StableHeadSymbols.h"
#include "Tools/Math/Common.h"

PROCESS_WIDE_STORAGE BH2010StableHeadSymbols* BH2010StableHeadSymbols::theInstance = 0;

void BH2010StableHeadSymbols::registerSymbols(xabsl::Engine& engine)
{
  engine.registerEnumElement("head.mode", "head.mode.pan_tilt", HeadMotionRequest::panTiltMode);
  engine.registerEnumElement("head.mode", "head.mode._target", HeadMotionRequest::targetMode);
  engine.registerEnumElement("head.mode", "head.mode.target_on_ground", HeadMotionRequest::targetOnGroundMode);

  engine.registerEnumeratedOutputSymbol("head.mode", "head.mode", (int*)&headMotionProposal.mode);
  engine.registerDecimalOutputSymbol("head.tilt", &setTilt, &getTilt);
  engine.registerDecimalOutputSymbol("head.pan", &setPan, &getPan);
  engine.registerDecimalOutputSymbol("head.speed", &setSpeed, &getSpeed);
  engine.registerDecimalOutputSymbol("head.target.x", &headMotionProposal.target.x);
  engine.registerDecimalOutputSymbol("head.target.y", &headMotionProposal.target.y);
  engine.registerDecimalOutputSymbol("head.target.z", &headMotionProposal.target.z);
  engine.registerBooleanInputSymbol("head.is_moving", &headJointRequest.moving);
  engine.registerBooleanInputSymbol("head.is_reachable", &headJointRequest.reachable);
  engine.registerBooleanOutputSymbol("head.use_active_vision", &headMotionProposal.useActiveVision);
  engine.registerDecimalOutputSymbol("head.active_vision_ball_factor", &headMotionProposal.ballFactor);
}

void BH2010StableHeadSymbols::update()
{
  headMotionProposal.useActiveVision = false;
}

void BH2010StableHeadSymbols::setTilt(float tilt)
{
  theInstance->headMotionProposal.tilt = fromDegrees(tilt);
}

float BH2010StableHeadSymbols::getTilt()
{
  return toDegrees(theInstance->headMotionProposal.tilt);
}

void BH2010StableHeadSymbols::setPan(float pan)
{
  theInstance->headMotionProposal.pan = fromDegrees(pan);
}

float BH2010StableHeadSymbols::getPan()
{
  return toDegrees(theInstance->headMotionProposal.pan);
}

void BH2010StableHeadSymbols::setSpeed(float speed)
{
  theInstance->headMotionProposal.speed = fromDegrees(speed);
}

float BH2010StableHeadSymbols::getSpeed()
{
  return toDegrees(theInstance->headMotionProposal.speed);
}
