/** 
* @file BH2010StableHeadSymbols.h
* Declaration of class BH2009HeadSymbols.
* @author Colin Graf
*/

#ifndef BH2010StableHeadSymbols_H
#define BH2010StableHeadSymbols_H

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/BehaviorControl/HeadMotionProposal.h"
#include "Representations/MotionControl/HeadJointRequest.h"

/**
* The Xabsl symbols that are defined in "head_symbols.xabsl"
* @author Max Risler
*/ 
class BH2010StableHeadSymbols : public Symbols
{
public:
  /*
  * Constructor.
  * @param headMotionProposal A reference to the HeadMotionProposal.
  */
  BH2010StableHeadSymbols(HeadMotionProposal& headMotionProposal, const HeadJointRequest& headJointRequest) : 
    headMotionProposal(headMotionProposal),
    headJointRequest(headJointRequest)
  {
    theInstance = this;
  }

private:

  PROCESS_WIDE_STORAGE_STATIC BH2010StableHeadSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  static void setTilt(float tilt);
  static float getTilt();
  static void setPan(float pan);
  static float getPan();
  static void setSpeed(float speed);
  static float getSpeed();

  HeadMotionProposal& headMotionProposal; /**< A reference to the HeadMotionProposal */
  const HeadJointRequest& headJointRequest; /**< A reference to the HeadJointRequest */

  /** 
  * Registers the symbols at an engine.
  * @param engine The xabsl engine.
  */
  void registerSymbols(xabsl::Engine& engine);

  /** updates the symbols */
  void update();

};


#endif // BH2010StableHeadSymbols_H

