/** 
* @file BH2010StableLocatorSymbols.h
*
* Declaration of class BH2010StableLocatorSymbols.
*
* @author Judith Mller
*/

#ifndef __BH2010StableLocatorSymbols_h_
#define __BH2010StableLocatorSymbols_h_

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/Infrastructure/TeamInfo.h"
#include "Representations/Modeling/BallModel.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/Configuration/FieldDimensions.h"
#include "Representations/Sensing/GroundContactState.h"

/**
* 
*
*
*/ 
class BH2010StableLocatorSymbols : public Symbols
{
public:
  /*
  * Constructor.
  * @param robotPose A reference to the RobotPose.
  * @param teamInfo A reference to the TeamInfo.
  * @param ballModel A reference to the ballModel.
  */
  BH2010StableLocatorSymbols(const RobotPose &robotPose, const RobotPoseInfo& robotPoseInfo, const OwnTeamInfo& ownTeamInfo, const FrameInfo& frameInfo, BallModel const& ballModel, FieldDimensions const& fieldDimensions, const GroundContactState& groundContactState) : 
    robotPose(robotPose),
    robotPoseInfo(robotPoseInfo),
    ownTeamInfo(ownTeamInfo),
    frameInfo(frameInfo),
    ballModel(ballModel),
    fieldDimensions(fieldDimensions),
    groundContactState(groundContactState),
    angleToOpponentGoal(0),
    angleWidthToOpponentGoal(0),
    ballPosRelWithOffset(0,0),
    angleToBallPosRelWithOffset(0)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC BH2010StableLocatorSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** Updates the Locatorator symbols */
  void update();

  /** A reference to the RobotPose */
  const RobotPose &robotPose;
  
  const RobotPoseInfo& robotPoseInfo;

  /** A reference to the TeamInfo */
  OwnTeamInfo const& ownTeamInfo;

  /** A reference to the FrameInfo */
  const FrameInfo& frameInfo;

  /** A reference to the BallModel */
  BallModel const& ballModel;

  FieldDimensions const& fieldDimensions;

  const GroundContactState& groundContactState;

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  static float getPoseX();
  static float getPoseY();
  static float getPoseAngle();
  static float getAngleToLineThroughBallAndGoal();

  float angleToOpponentGoal;

  float angleWidthToOpponentGoal;  
  float angleToleranceToOpponentGoal;
 
  float angleToOwnGoal;
  float angleWidthToOwnGoal;

  /** calculate balloffset with angle to oppent goal */ 
  Vector2<> ballPosRelWithOffset;
  
  float angleToBallPosRelWithOffset;

  /** calculates the decimal input function "distance_to" */
  static float distanceTo();
  
  /** calculates the decimal input function "angle_to" */
  static float angleTo();

  /** calculates the decimal input function "field_to_relative.x" */
  static float getFieldToRelativeX();

  /** calculates the decimal input function "field_to_relative.y" */
  static float getFieldToRelativeY();

  static float getTimeSinceLastPoseReset();
  
  /** The parameter "x" of the functions "angle_to","distance_to","relative_to_field" */
  float x;
  
  /** The parameter "y" of the functions "angle_to","distance_to","relative_to_field" */
  float y;

};


#endif // __LocatorSymbols_h_

