/** 
* @file BH2010StableMotionSymbols.h
*
* Declaration of class BH2010StableMotionSymbols.
*
* @author Max Risler
*/

#ifndef __BH2010StableMotionSymbols_h_
#define __BH2010StableMotionSymbols_h_

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/MotionControl/MotionRequest.h"
#include "Representations/MotionControl/MotionInfo.h"
#include "Representations/Infrastructure/RobotInfo.h"
#include "Representations/Configuration/RobotName.h"
#include "Representations/Infrastructure/FrameInfo.h"

/**
* The Xabsl symbols that are defined in "motion_symbols.xabsl"
*
* @author Max Risler
*/ 
class BH2010StableMotionSymbols : public Symbols
{
public:
  BH2010StableMotionSymbols(MotionRequest& motionRequest, const MotionInfo& motionInfo, const RobotInfo& robotInfo, const RobotName& robotName, const FrameInfo& frameInfo) : 
    motionRequest(motionRequest),
    motionInfo(motionInfo),
    robotInfo(robotInfo),
    robotName(robotName),
    frameInfo(frameInfo),
    lastSpecialAction(SpecialActionRequest::playDead)
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC BH2010StableMotionSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  MotionRequest& motionRequest; /**< A reference to the MotionRequest */
  const MotionInfo& motionInfo; /**< A reference to the MotionInfo */  
  const RobotInfo& robotInfo; /**< A reference to the RobotInfo */ 
  const RobotName& robotName; /**< A reference to the RobotName */ 
  const FrameInfo& frameInfo;

  SpecialActionRequest::SpecialActionID lastSpecialAction; /**< The last requested not-specialized special action. */

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);
  void update();

  unsigned int timeStampOfLastKick;

  static void setMotionType(int type);
  static int getMotionType();
  static void setWalkSpeedX(float x);
  static float getWalkSpeedX();
  static void setWalkSpeedY(float y);
  static float getWalkSpeedY();
  static void setWalkSpeedRot(float rot);
  static float getWalkSpeedRot();
  static void setPercentageSpeedX(float percent);
  static float getPercentageSpeedX();
  static void setPercentageSpeedY(float percent);
  static float getPercentageSpeedY();
  static void setPercentageSpeedRot(float percent);
  static float getPercentageSpeedRot();
  static void setWalkTargetX(float x);
  static float getWalkTargetX();
  static void setWalkTargetY(float y);
  static float getWalkTargetY();
  static void setWalkTargetRot(float rot);
  static float getWalkTargetRot();
  static void setWalkSpeed(float speed);
  static float getWalkSpeed();
  static bool getWalkTargetReached();
  static bool getWalkTargetInProgress();
  static void setSpecialAction(int specialAction);
  static int getSpecialAction();
  static void setSIKE(int sikeId);
  static int getSIKE();
  static float getTimeSinceLastKick();

};

#endif // __BH2010StableMotionSymbols_h_

