/** 
* @file BH2010StableObstacleSymbols.h
*
* Declaration of class BH2010StableObstacleSymbols.
*
* @author <a href="mailto:Tim.Laue@dfki.de">Tim Laue</a>
* @author <a href="mailto:rieskamp@informatik.uni-bremen.de">Andrik Rieskamp</a>
* @author <a href="mailto:aschreck@informatik.uni-bremen.de">Andr Schreck</a>
*/

#ifndef __BH2010StableObstacleSymbols_h_
#define __BH2010StableObstacleSymbols_h_

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/Modeling/ObstacleModel.h"


/**
* @class BH2010StableObstacleSymbols
* Symbols for collision avoidence with ultrasound sensors:
*/
class BH2010StableObstacleSymbols : public Symbols
{
public:
  /** Constructor */
  BH2010StableObstacleSymbols(const ObstacleModel& obstacleModel):
    obstacleModel(obstacleModel)
  {}

  /** Registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  /** Updates some of the symbols */
  void update();

private:
  float distanceToClosestObstacle;      /**< As the name says... */
  float distanceToClosestObstacleLeft;  /**< As the name says... */
  float distanceToClosestObstacleRight; /**< As the name says... */
  const ObstacleModel& obstacleModel; /**< Reference to ObstacleModel which contains obstacle information */
};

#endif // __BH2010StableObstacleSymbols_h_
