/** 
* @file KeySymbols.h
*
* Declaration of class KeySymbols.
*
* @author Judith Mller
* @author Max Risler
* @author Michael Mester
*/

#ifndef __KeySymbols_h_
#define __KeySymbols_h_

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/Infrastructure/KeyStates.h"

class KeySymbols : public Symbols
{
public:
  KeySymbols(KeyStates const& keyStates): 
      keyStates(keyStates), leftFoot(false), rightFoot(false)
  {
    theInstance = this;
    for (int i = 0; i < KeyStates::numberOfKeys; ++i)
    {
      wasPressed[i] = false;
      pressedAndReleased[i] = false;
    }
  }

  PROCESS_WIDE_STORAGE_STATIC KeySymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);
  void update();

private:
  KeyStates const& keyStates;
  bool wasPressed[KeyStates::numberOfKeys];
  bool pressedAndReleased[KeyStates::numberOfKeys];
  bool leftFoot;
  bool rightFoot;
};


#endif // __KeySymbols_h_

