/** 
* @file MathSymbols.h
*
* Declaration of class MathSymbols.
*
* @author Max Risler
*/

#ifndef __MathSymbols_h_
#define __MathSymbols_h_

#include "Modules/BehaviorControl/Symbols.h"
#include "Platform/SystemCall.h"

/**
* The Xabsl symbols that are defined in "math_symbols.xabsl"
*
* @author Max Risler
*/ 
class MathSymbols : public Symbols
{
public:
  /*
  * Constructor.
  */
  MathSymbols()
  {
    theInstance = this;
  }

  PROCESS_WIDE_STORAGE_STATIC MathSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);
    
  /** The parameter "normalize.angle" of the function "normalize" */
  float normalizeAngle;

  /** calculates the decimal input function "sgn" */
  static float getSgn();

  /** calculates the decimal input function "min" */
  static float getMin();

  /** calculates the decimal input function "max" */
  static float getMax();

  /** calculates the decimal input function "abs" */
  static float getAbs();

  /** calculates the decimal input function "random" */
  static float getRandom();

  /** calculates the decimal input function "sin" */
  static float getSin();

  /** calculates the decimal input function "cos" */
  static float getCos();

  /** calculates the decimal input function "normalize" */
  static float getNormalize();

  /** calculates the decimal input function "clip" */
  static float getClip();

  /** calculates the decimal input function "exclude" */
  static float getExclude();

  /** calculates the decimal input function "distance" */
  static float getDistance();

  /** calculates the decimal input function "distance" */
  static float getDistanceAB();

  /** calculates the decimal input function "atan2" */
  static float getAtan2();

  /** The parameter "sgn.value" of the function "sgn" */
  float sgnValue;

  /** The parameter "abs.value" of the function "abs" */
  float absValue;

  /** The parameter "min0.value" of the function "min" */
  float min0Value;

  /** The parameter "min1.value" of the function "min" */
  float min1Value;

  /** The parameter "max.value0" of the function "max" */
  float max0Value;

  /** The parameter "max.value1" of the function "max" */
  float max1Value;

  /** The parameter "*.alpha" of the function "sin" and "cos" */
  float alphaValue;

  float clipValue;
  float clipMin;
  float clipMax;

  float excludeValue;
  float excludeMin;
  float excludeMax;

  float distanceX;
  float distanceY;
  float distanceX2;
  float distanceY2;

  float atan2Y;
  float atan2X;
};

#endif // __MathSymbols_h_

