/** 
* @file MotionSymbols.h
*
* Declaration of class MotionSymbols.
*
* @author Max Risler
*/

#ifndef MotionSymbols_H
#define MotionSymbols_H

#include "Modules/BehaviorControl/Symbols.h"
#include "Representations/MotionControl/MotionRequest.h"
#include "Representations/MotionControl/MotionInfo.h"
#include "Representations/Infrastructure/RobotInfo.h"
#include "Representations/Configuration/RobotName.h"

/**
* The Xabsl symbols that are defined in "motion_symbols.xabsl"
*
* @author Max Risler
*/ 
class MotionSymbols : public Symbols
{
public:
  MotionSymbols(MotionRequest& motionRequest, const MotionInfo& motionInfo, const RobotInfo& robotInfo, const RobotName& robotName) : 
    motionRequest(motionRequest),
    motionInfo(motionInfo),
    robotInfo(robotInfo),
    robotName(robotName),
    lastSpecialAction(SpecialActionRequest::playDead)
  {
    theInstance = this;
  }

private:
  PROCESS_WIDE_STORAGE_STATIC MotionSymbols* theInstance; /**< Points to the only instance of this class in this process or is 0 if there is none. */

  MotionRequest& motionRequest; /**< A reference to the MotionRequest */
  const MotionInfo& motionInfo; /**< A reference to the MotionInfo */  
  const RobotInfo& robotInfo; /**< A reference to the RobotInfo */ 
  const RobotName& robotName; /**< A reference to the RobotName */ 

  SpecialActionRequest::SpecialActionID lastSpecialAction; /**< The last requested not-specialized special action. */

  /** registers the symbols at an engine */
  void registerSymbols(xabsl::Engine& engine);

  void update();

  static void setMotionType(int type);
  static int getMotionType();
  static void setWalkSpeed(float speed);
  static float getWalkSpeed();
  static void setWalkTargetRot(float rot);
  static float getWalkTargetRot();
  static bool getWalkTargetReached();
  static void setSpecialAction(int specialAction);
  static int getSpecialAction();
  static void setSIKE(int sikeId);
  static int getSIKE();
};

#endif // MotionSymbols_H

