/**
* @file Logger.h
* Declaration of module Logger which collects data during operation and
* can write it to a file if wanted.
*
* @author <a href="mailto:Tim.Laue@dfki.de">Tim Laue</a>
*/

#ifndef Logger_H__
#define Logger_H__

#include "Tools/Module/Module.h"
#include "Tools/RingBuffer.h"
#include "Representations/Debugging/LoggerOutput.h"
#include "Representations/Debugging/SimpleXabslInfo.h"
#include "Representations/Infrastructure/GameInfo.h"
#include "Representations/Configuration/RobotName.h"


MODULE(Logger)
  REQUIRES(SimpleXabslInfo)
  REQUIRES(GameInfo)
  REQUIRES(RobotName)
  PROVIDES(LoggerOutput)
END_MODULE


/**
* @class Logger
* A module for logging
*/
class Logger : public LoggerBase
{
public:
  /** Constructor */
  Logger()
  {
    optionsBuffer = new RingBuffer<std::vector<std::string>,CYCLES>;
  }

  /** Destructor */
  ~Logger()
  {
    for(int i=0;i<optionsBuffer->getNumberOfEntries(); ++i)
    {
      (*optionsBuffer)[i].clear();
    }
    delete optionsBuffer;
  }

private:
  /**
  * Triggers the logging.
  * @param loggerOutput A dummy for triggering the logging process, real output goes to a file.
  */
  void update(LoggerOutput& loggerOutput);

  /** Export function generating HTML file*/
  void exportData(const std::string& fileName);

  /** Checks, if a list l contains a given element e */
  bool listContains(const std::vector<std::string>& l, const std::string& e);

  /** Constant for maximum buffer size (current value is for "something longer than one half")*/
  enum {CYCLES = 20000};
  /** A buffer for storing the active options of the last execution cycles */
  RingBuffer<std::vector<std::string>,CYCLES>* optionsBuffer;
};

#endif
