/*
 * LogConverter.cpp
 *
 *  Created on: Jun 5, 2010
 *      Author: reich
 */

#include "LogDataProvider.h"
#include "LogConverter.h"
#include "Representations/Infrastructure/SensorData.h"
#include "Representations/Infrastructure/SensorDataRev5703.h"
#include "Platform/GTAssert.h"
#include "Tools/Streams/OutStreams.h"

LogConverter::LogConverter()
{
  SensorDataRev5703 sensorDataRev5703;
  sizeofSensorDataRev5703 = sizeofRepresentation(sensorDataRev5703);
}

std::size_t LogConverter::sizeofRepresentation(Streamable& streamable)
{
  OutBinarySize outSize;
  outSize << streamable;
  return outSize.getSize();
}

LogConverter::~LogConverter() {}

/**
 * Creates new converted representation
 * @param message the old representation destined for conversion
 * @param representationId id of the representation
 * @returns a newly allocated and converted representation
 */
Streamable* LogConverter::newConvertedRepresentation(InMessage& message, const int representationId)
{
  switch (representationId)
  {
  case idSensorData:
    return newConvertedSensorData(message);
  default:
    ASSERT (false);
  }
  return 0;
}

bool LogConverter::isConvertRequired(InMessage& message, const int representationId)
{
  switch (representationId)
  {
  case idSensorData:
    return isConvertRequiredSensorData(message);
  default:
    return false;
  }
}

bool LogConverter::isConvertRequiredSensorData(InMessage& message)
{
  return (std::size_t)message.getMessageSize() == sizeofSensorDataRev5703;
}

/*
 * @returns newly allocated and converted SensorData representation
 */
Streamable* LogConverter::newConvertedSensorData(InMessage& message)
{
  SensorDataRev5703 sensorDataRev5703;
  SensorData *sensorData = new SensorData;
  message.bin >> sensorDataRev5703;
  message.resetReadPosition();

  memcpy(sensorData->data, sensorDataRev5703.data, sizeof (float) * SensorDataRev5703::us);
  memcpy(&sensorData->data[SensorData::angleX], &sensorDataRev5703.data[SensorDataRev5703::angleX],
      sizeof (float) * (SensorData::numOfSensors - SensorData::angleX));
  memcpy(sensorData->currents, sensorDataRev5703.data, sizeof (sensorData->currents));
  memcpy(sensorData->temperatures, sensorDataRev5703.temperatures, sizeof (sensorData->temperatures));

  sensorData->timeStamp = sensorDataRev5703.timeStamp;
  sensorData->usActuatorMode = (SensorData::UsActuatorMode)sensorDataRev5703.usSensorType;
  sensorData->usTimeStamp = sensorData->timeStamp;

  return sensorData;
}
