/** 
* @file WhiteCorrection.cpp
* This file implements a module that can provide white corrected cameraSettings
* @author Alexander Haertl
*/

#ifndef __WhiteCorrection_h_
#define __WhiteCorrection_h_

#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/Image.h"
#include "Representations/Configuration/CameraSettings.h"

MODULE(WhiteCorrection)
  USES(Image)
  PROVIDES_WITH_MODIFY(CameraSettings)
END_MODULE

class WhiteCorrection : public WhiteCorrectionBase
{
private:
  enum State
  {
    Measure,
    Wait,
    Idle
  } state;

  int accR, accG, accB; /**< accumulators for RGB-components */
  int measurement; /**< counter for measurements in state Measure */
  int framesWaited; /**< counter for frames waited so far in state Wait */

  void update(CameraSettings& cameraSettings);
  
  /**
  * computes the average color of all pixels of the camera image
  * @param color The average color of all image pixels
  */
  void getAverageYCbCr(Image::Pixel& color);
  void getAverageYCbCrMMX(Image::Pixel& color);

public:
  /**
  * Default constructor.
  */
  WhiteCorrection();
};

#endif // __WhiteCorrection_h_
