/**
* @file BIKEParameters.h
* @author <a href="mailto:judy@informatik.uni-bremen.de">Judith Mller</a>
*/

#ifndef __BIKEParameters_H__
#define __BIKEParameters_H__

#define NUM_OF_POINTS 3

#include "Tools/Streams/StreamHandler.h"
#include "Tools/Math/Vector3.h"
#include "Tools/Math/Pose2D.h"
#include "Tools/Streams/InStreams.h"
#include <vector>

class BikeExtraMath
{
public:
  static float factorial(float number)
  {
    if (number<=1.f)
      return 1.f;
    return factorial(number-1.f)*number; 
  }

  static float polyKo(float n, float k)
  {
    if (k == 0.f || k == n) return 1.f;
    return (factorial(n)/(factorial(k)*factorial(n-k)));
  }
};

class Phase: public Streamable
{
public:
  enum Limbs
  {
    leftFootTra,
    leftFootRot,
    rightFootTra,
    rightFootRot,
    leftArmTra,
    rightArmTra, 
    numOfLimbs
  };

  unsigned int duration; 

  Vector3<> controlPoints[6][NUM_OF_POINTS];

  Vector3<> originPos[6], originOffset[6];
  Vector3<> odometryOffset;

  virtual void serialize(In* in, Out* out);

  Phase(){};

  ~Phase(){}
};

class BIKEParameters : public Streamable
{
public:

  int numberOfPhases, preview;
  bool loop;
  char name[260];

  Vector3<> footOrigin, armOrigin; 
  std::vector<Phase> phaseParameters;
  float kpx, kdx, kix, kpy, kdy, kiy;

  enum SupportLegMode
  {
    auto_,
    left,
    right,
    both,
    numOfSupportLegModes,
  };

  SupportLegMode supportLegMode;

  static const char* getSupportLegModeName(SupportLegMode mode)
  {
    switch(mode)
    {
    case auto_:
      return "auto";
    case left:
      return "left";
    case right:
      return "right";
    case both:
      return "both";
    default:
      return "unknown";
    };
  }

  /**
  * The method serializes this object.
  * @param in Points to an In-stream, when currently reading.
  * @param out Points to an Out-stream, when currently writing.
  */
  virtual void serialize(In* in, Out* out);

  Vector3<> getPosition(float phase, int phaseNumber, int limb);
  void initFirstPhase();
  void initFirstPhaseLoop(const Phase& lastParameters);
  void initPhase(int i);
  void init();

  BIKEParameters():
    numberOfPhases(0),
    preview(150),
    loop(false),
    footOrigin(0.f, 0.f, 0.f),
    armOrigin(0.f, 0.f, 0.f),
    phaseParameters(0),
    kpx(0.f),
    kdx(0.f),
    kix(0.f),
    kpy(0.f),
    kdy(0.f),
    kiy(0.f),
    supportLegMode(auto_)
  {
  };
};

#endif // __BIKEParameters_H__
