/** 
* @file Modules/MotionControl/MotionSelector.h
* This file declares a module that is responsible for controlling the motion.
* @author <A href="mailto:Thomas.Roefer@dfki.de">Thomas Rfer</A>
* @author <A href="mailto:allli@tzi.de">Alexander Hrtl</A>
*/

#ifndef __MotionSelector_h_
#define __MotionSelector_h_

#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/MotionControl/SpecialActionsOutput.h"
#include "Representations/MotionControl/WalkingEngineOutput.h"
#include "Representations/MotionControl/WalkingEngineStandOutput.h"
#include "Representations/MotionControl/BikeEngineOutput.h"
#include "Representations/MotionControl/SIKEngineOutput.h"
#include "Representations/MotionControl/MotionRequest.h"
#include "Representations/MotionControl/MotionSelection.h"
#include "Representations/Sensing/GroundContactState.h"

MODULE(MotionSelector)
USES(SpecialActionsOutput)
USES(WalkingEngineOutput)
USES(WalkingEngineStandOutput)
USES(BikeEngineOutput)
USES(SIKEngineOutput)
REQUIRES(FrameInfo)
REQUIRES(MotionRequest)
REQUIRES(GroundContactState)
PROVIDES_WITH_MODIFY(MotionSelection)
END_MODULE

class MotionSelector : public MotionSelectorBase
{
private:
  MotionRequest::Motion lastMotion;
  MotionRequest::Motion prevMotion;
  unsigned lastExecution;
  SpecialActionRequest::SpecialActionID lastActiveSpecialAction;
  void update(MotionSelection& motionSelection);

public:
  /**
  * Default constructor.
  */
  MotionSelector() : lastMotion(MotionRequest::specialAction), prevMotion(MotionRequest::specialAction),
    lastActiveSpecialAction(SpecialActionRequest::playDead) {}
};

#endif // __MotionSelector_h_
