/**
* @file SIKEMotion.h
* author jeff
*/

#ifndef SIKEMotion_H
#define SIKEMotion_H

#include <vector>

#include "Tools/Math/Pose3D.h"
#include "Representations/Configuration/RobotDimensions.h"
#include "Representations/Configuration/JointCalibration.h"
#include "Representations/Sensing/RobotModel.h"

class SIKEMotion
{
  public:
    class KeyFrame
    {
      public:
        KeyFrame mirrored(bool mirror) const;

        unsigned time;

        Pose3D lfoot,
               rfoot,
               lhand,
               rhand;

        float headYaw,
              headPitch;

        static KeyFrame fromRobotModel(const RobotModel &theRobotModel,
                                       const RobotDimensions &theRobotDimensions);
        static KeyFrame interpolateBetweenFrames(const KeyFrame &f1,
                                                 const KeyFrame &f2,
                                                 float ratio);
    };

    bool getMotionAt(unsigned timeSinceMotionStarted,
                     bool mirror,
                     const KeyFrame &startFrame,
                     KeyFrame &result) const;
    bool loadFromFile(std::string filename,
                      const RobotDimensions &theRobotDimensions);
    void saveToFile(std::string filename,
                    const RobotDimensions &theRobotDimensions,
                    const JointCalibration &theJointCalibration);

    std::vector<KeyFrame> frames;
};

#endif //SIKEMotion_H

