/** 
* @file Modules/MotionControl/SIKEngine.h
* S(imple)I(inverse)K(inematic)Engine
* author jeff
*/

#ifndef SIKEngine_H
#define SIKEngine_H

#include <vector>

#include "Tools/Math/Pose3D.h"
#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/Configuration/RobotDimensions.h"
#include "Representations/Configuration/JointCalibration.h"
#include "Representations/Sensing/RobotModel.h"
#include "Representations/MotionControl/SIKEngineOutput.h"
#include "Representations/MotionControl/MotionSelection.h"

#include "SIKEMotion.h"

MODULE(SIKEngine)
  REQUIRES(RobotDimensions)
  REQUIRES(FrameInfo)
  REQUIRES(RobotModel)
  REQUIRES(MotionSelection)
  REQUIRES(JointCalibration)
  REQUIRES(FilteredJointData)
  PROVIDES_WITH_MODIFY(SIKEngineOutput)
END_MODULE

class SIKEngine: public SIKEngineBase
{
  public:
    typedef SIKEMotion::KeyFrame Frame;

    class SIKEState
    {
      public:
        SIKEState() :
          actMotion(SIKERequest::numOfMotions),
          mirror(false),
          actMotionStartTime(0),
          done(true)
        {}

        SIKERequest::Motion actMotion;
        bool mirror;
        unsigned actMotionStartTime;
        bool done;
        Frame actMotionStartFrame;
    };

  private:
    std::vector<SIKEMotion> motions;
    SIKEState state;
    SIKEngineOutput *output;

    void init();
    void update(SIKEngineOutput &sikengineOutput);

    void playMotion(SIKEState &state);
    void setOutput(const Frame &frame);

    void editor(SIKEState &state);

    class EditorState
    {
      public:
        EditorState() :
          frameId(0),
          lastFrameId(-1),
          interpolationTime(0)
        {}

        int frameId,
            lastFrameId,
            interpolationTime;
        Frame lastFrame;
    };

    EditorState editorState;
};

#endif // SIKEngine_H

