/**
* @file RobotPerceptor.h
* @author <a href="afabisch@tzi.de">Alexander Fabisch</a>
* @author <a href="d3ni@tzi.de">Thomas Liebschwager</a>
*/
#ifndef __RobotPerceptor_h_
#define __RobotPerceptor_h_

#include "Tools/Module/Module.h"
#include "Representations/Infrastructure/Image.h"
#include "Representations/Configuration/ColorTable64.h"
#include "Representations/Perception/RegionPercept.h"
#include "Representations/Perception/RobotPercept.h"
#include "PointExplorer.h"

MODULE(RobotPerceptor)
  REQUIRES(Image)
  REQUIRES(ColorTable64)
  REQUIRES(RegionPercept)
  PROVIDES_WITH_MODIFY_AND_DRAW(RobotPercept)
END_MODULE

/**
 * @class RobotPerceptor
 * This class finds indicators for robots on the image.
 */
class RobotPerceptor : public RobotPerceptorBase
{
  /**
   * @class Parameters
   * The parameters for the RobotPerceptor.
   */
  class Parameters : public Streamable
  {
  private:
    void serialize(In* in, Out* out);
  public:
    int teamMarkerRegionMinSize;
    Vector2<> eccentricityBoundaries;
    float aboveWhiteRatio;
    float belowWhiteRatio;
    Vector2<int> axisND;
    float greenGroundRatio;
    /** The distance in pixels between exploring scanlines. */
    int exploreStepSize;
    /** The distance in pixels between neighboring scan lines. */
    int gridStepSize;
    /** The maximum number of pixels to skip when grouping pixels to segments. */
    int skipOffset;
    /** The minimal size/length for a segment in pixels for each color. */
    int minSegSize[ColorClasses::numOfColors];
  };

  typedef std::vector<Vector2<int> >::const_iterator V2CIt;

  Parameters params;
  PointExplorer pointExplorer;

  /** Updates the RobotPercept. */
  void update(RobotPercept& robotPercept);
  /**
   * Prepares the robotPercept for this cycle.
   */
  void prepareRobotPercept(RobotPercept& robotPercept);
  float areaMembership(int area);
  float eccentricityMembership(float eccentricity);
  void principleAxisLength(float& len_min, float& len_max, const RegionPercept::Region* region, const Vector2<int>& cog, const Vector2<>& achse1, const Vector2<>& achse2);
public:
  RobotPerceptor();
};

#endif// __RobotPerceptor_h_

