#ifndef __RobotPerceptorHelper_h_
#define __RobotPerceptorHelper_h_

#include "Tools/Math/Vector2.h"
#include "Tools/ImageProcessing/BresenhamLineScan.h"
#include "Representations/Infrastructure/Image.h"
#include "Representations/Configuration/ColorTable64.h"

inline ColorClasses::Color getColor(const ColorTable64& theColorTable64, const Image& theImage, int x, int y)
{
  unsigned col(theImage.image[y][x].color);
  return (ColorClasses::Color) theColorTable64.colorClasses[col >> 18 & 0x3f][col >> 10 & 0x3f][col >> 26 & 0x3f];
}

int scanline(
  const Image& theImage,
  const ColorTable64& theColorTable64,
  const ColorClasses::Color& teamColor,
  const Vector2<int>& start,
  Vector2<int>& end,
  int& whitePixels)
{
  BresenhamLineScan scan = BresenhamLineScan(start, end);
  scan.init();
  end = start;
  for(int i = 0; i < scan.numberOfPixels; i++)
  {
    scan.getNext(end);
    if(end.x >= 0 && end.x < theImage.cameraInfo.resolutionWidth
       && end.y >= 0 && end.y < theImage.cameraInfo.resolutionHeight)
    {
      if(getColor(theColorTable64, theImage, end.x, end.y) == ColorClasses::white)
      {
        whitePixels++;
        continue;
      }
      else
      {
        unsigned char y = theImage.image[end.y][end.x].yCbCrPadding;
        unsigned char cb = theImage.image[end.y][end.x].cb;
        unsigned char cr = theImage.image[end.y][end.x].cr;
        if(y > 100 && y > cb && y > cr)
          whitePixels++;
      }
    }
    else
    {
      return i;
    }
  }
  return scan.numberOfPixels;
}

inline bool exactLocalisation(const Image& theImage, const Vector2<int>& p)
{
  return theImage.cameraInfo.resolutionHeight != p.y;
}

#endif //__RobotPerceptorHelper_h_

