/**
* @file GroundContactDetector.h
* Declaration of module GroundContactDetector.
* @author Colin Graf
*/

#ifndef GroundContactDetector_H
#define GroundContactDetector_H

#include "Tools/Module/Module.h"
#include "Tools/Range.h"
#include "Representations/Infrastructure/SensorData.h"
#include "Representations/Infrastructure/FrameInfo.h"
#include "Representations/Infrastructure/RobotInfo.h"
#include "Representations/Sensing/GroundContactState.h"
#include "Representations/MotionControl/MotionInfo.h"
#include "Representations/MotionControl/MotionRequest.h"

MODULE(GroundContactDetector)
  REQUIRES(RobotName)
  REQUIRES(SensorData)
  REQUIRES(FrameInfo)
  REQUIRES(MotionRequest)
  REQUIRES(RobotInfo)
  USES(MotionInfo)
  PROVIDES_WITH_MODIFY(GroundContactState)
END_MODULE

/**
* @class GroundContactDetector
* A module for sensor data filtering.
*/
class GroundContactDetector : public GroundContactDetectorBase
{
public:
  /** Default constructor. */
  GroundContactDetector();

private:
  /**
  * A collection of parameters for the ground contact detector.
  */
  class Parameters : public Streamable
  {
  public:
    /** Default constructor. */
    Parameters() {}

    float fsrFactor;
    float loadFactor;
    float threshold;
    unsigned int minCycleContactUnity;
    int safeContactTime;
    int noContactTime;
  private:
    /**
    * The method makes the object streamable.
    * @param in The stream from which the object is read.
    * @param out The stream to which the object is written. 
    */
    virtual void serialize(In* in, Out* out)
    {  
      STREAM_REGISTER_BEGIN();
        STREAM(fsrFactor);
        STREAM(loadFactor);
        STREAM(threshold);
        STREAM(minCycleContactUnity);
        STREAM(safeContactTime);
      STREAM_REGISTER_FINISH();
    }
  };

  Parameters p;


  unsigned int framesWithContact;
  unsigned int framesWithoutContact;
  unsigned int contactStartTime;
  unsigned int noContactStartTime;

  bool contact;
  bool wasContacted;

  void init();

  /**
  * Updates the GroundContactState representation .
  * @param groundContactState The ground contact representation which is updated by this module.
  */
  void update(GroundContactState& groundContactState);
};

#endif
