/**
* @file Platform/Win32/GTAssert.h
* This file contains macros for low level debugging.
* @author Thomas Rfer
*/
#ifndef GTAssert_H
#define GTAssert_H

#undef ASSERT
#undef VERIFY
#undef TRACE

#ifdef RELEASE

#define ASSERT(cond) ((void)0)
#define VERIFY(cond) ((void)(cond))
#define TRACE(...) ((void)0)

#else //RELEASE

class Assert
{
public:
  static void trace(const char* format, ...);
  static void assertFailed() {__asm{int 3};}
  static void verifyFailed() {__asm{int 3};}
};

/**
 * ASSERT prints a message if cond is false and RELEASE is not defined.
 * ASSERT does not evaluate cond if RELEASE is defined.
 * @param c The condition to be checked.
 */
#define ASSERT(c) ((void)((c) ? 0 : (Assert::assertFailed(), 0)))

/**
 * VERIFY prints a message if cond is false and RELEASE is not defined.
 * VERIFY does evaluate cond even if RELEASE is defined.
 * @param c The condition to be checked.
 */
#define VERIFY(c) ((void)((c) ? 0 : (Assert::verifyFailed(), 0)))

/**
 * TRACE prints a message if RELEASE is not defined.
 */
#define TRACE(...) Assert::trace(__VA_ARGS__)

#endif //RELEASE

#endif //GTAssert_H
