/**
* @file Platform/linux/GTAssert.h
* Some helper functions for low level debugging
* @author Colin Graf
*/

#if !defined(RELEASE) && !defined(NOASSERTIONS)

#include "GTAssert.h"

#include <cstdlib>
#include <cstdio>
#include <cstdarg>

void Assert::trace(const char* format, ...)
{
  char data[320];
  va_list ap;
  va_start(ap, format);
  int length = vsnprintf(data, sizeof(data) - 2, format, ap);
  if(length < 0)
    length = sizeof(data) - 2;
  va_end(ap);
  data[length++] = '\n';
  data[length] = '\0';
  fputs(data, stderr);
  fflush(stderr);
}

void Assert::verifyFailed(const char* file, unsigned int line, const char* condition)
{
  trace("%s:%u VERIFY(%s) failed", file, line, condition);
  abort();
}

void Assert::assertFailed(const char* file, unsigned int line, const char* condition)
{
  trace("%s:%u ASSERT(%s) failed", file, line, condition);
  abort();
}

#endif
