/** 
* @file Blackboard.cpp
* Implementation of a class representing the blackboard.
* @author <a href="mailto:Thomas.Roefer@dfki.de">Thomas R�fer</a>
*/

#include "Blackboard.h"
#include <cstring>
#include <cstdlib>

Blackboard::Blackboard() :
// Initialize all representations by themselves:
// Infrastructure
theJointData(theJointData),
theJointRequest(theJointRequest),
theSensorData(theSensorData),
theKeyStates(theKeyStates),
theLEDRequest(theLEDRequest),
theImage(theImage),
theCameraInfo(theCameraInfo),
theFrameInfo(theFrameInfo),
theCognitionFrameInfo(theCognitionFrameInfo),
theRobotInfo(theRobotInfo),
theOwnTeamInfo(theOwnTeamInfo),
theOpponentTeamInfo(theOpponentTeamInfo),
theGameInfo(theGameInfo),
theTeamStatus(theTeamStatus),
theSoundRequest(theSoundRequest),
theSoundOutput(theSoundOutput),
theTeamMateData(theTeamMateData),
theMotionRobotHealth(theMotionRobotHealth),
theRobotHealth(theRobotHealth),
theBoardInfo(theBoardInfo),

// Configuration
theRobotName(theRobotName),
theColorTable64(theColorTable64),
theCameraSettings(theCameraSettings),
theFieldDimensions(theFieldDimensions),
theRobotDimensions(theRobotDimensions),
theJointCalibration(theJointCalibration),
theSensorCalibration(theSensorCalibration),
theCameraCalibration(theCameraCalibration),
theBehaviorConfiguration(theBehaviorConfiguration),
theMassCalibration(theMassCalibration),

// Perception
theCameraMatrix(theCameraMatrix),
theCameraMatrixPrev(theCameraMatrixPrev),
theRobotCameraMatrix(theRobotCameraMatrix),
theRobotCameraMatrixPrev(theRobotCameraMatrixPrev),
theImageCoordinateSystem(theImageCoordinateSystem),
theBallSpots(theBallSpots),
theLineSpots(theLineSpots),
theBallPercept(theBallPercept),
theExtendedBallPercepts(theExtendedBallPercepts),
theLinePercept(theLinePercept),
theRegionPercept(theRegionPercept),
theGoalPercept(theGoalPercept),
theGroundContactState(theGroundContactState),
theBodyContour(theBodyContour),
theRobotPercept(theRobotPercept),
theImageInfo(theImageInfo),

// Modeling
theAnotherObstacleModel(theAnotherObstacleModel),
theFallDownState(theFallDownState),
theBallHypotheses(theBallHypotheses),
theBallModel(theBallModel),
theGroundTruthBallModel(theGroundTruthBallModel),
theObstacleModel(theObstacleModel),
theRobotPose(theRobotPose),
theRobotPoseInfo(theRobotPoseInfo),
thePotentialRobotPose(thePotentialRobotPose),
theGroundTruthRobotPose(theGroundTruthRobotPose),
theRobotPoseHypotheses(theRobotPoseHypotheses),
theSelfLocatorSampleSet(theSelfLocatorSampleSet),
theBallLocatorSampleSet(theBallLocatorSampleSet),
theRobotsModel(theRobotsModel),
theFreePartOfOpponentGoalModel(theFreePartOfOpponentGoalModel),

// BehaviorControl
theBehaviorControlOutput(theBehaviorControlOutput),
theHeadMotionProposal(theHeadMotionProposal),
theBehaviorParameters(theBehaviorParameters),
theBehaviorTactics(theBehaviorTactics),
theBH2010Parameters(theBH2010Parameters),

// Sensing
theFilteredJointData(theFilteredJointData),
theFilteredJointDataPrev(theFilteredJointDataPrev),
theFilteredSensorData(theFilteredSensorData),
theInertiaSensorData(theInertiaSensorData),
theOrientationData(theOrientationData),
theGroundTruthOrientationData(theGroundTruthOrientationData),
theTorsoMatrix(theTorsoMatrix),
theTorsoMatrixPrev(theTorsoMatrixPrev),
theRobotModel(theRobotModel),

// MotionControl
theOdometryData(theOdometryData),
theGroundTruthOdometryData(theGroundTruthOdometryData),
theMotionRequest(theMotionRequest),
theHeadMotionRequest(theHeadMotionRequest),
theHeadJointRequest(theHeadJointRequest),
theMotionSelection(theMotionSelection),
theSpecialActionsOutput(theSpecialActionsOutput),
theWalkingEngineOutput(theWalkingEngineOutput),
theWalkingEngineStandOutput(theWalkingEngineStandOutput),
theBikeEngineOutput(theBikeEngineOutput),
theSIKEngineOutput(theSIKEngineOutput),
theMotionInfo(theMotionInfo),
theUnstableJointRequest(theUnstableJointRequest),

// Debugging
theXabslInfo(theXabslInfo),
theSimpleXabslInfo(theSimpleXabslInfo),
theLoggerOutput(theLoggerOutput)
{
}

void Blackboard::operator=(const Blackboard& other)
{
  memcpy(this, &other, sizeof(Blackboard)); 
}

void* Blackboard::operator new(std::size_t size)
{
  return calloc(1, size);
}

void Blackboard::operator delete(void* p)
{
  return free(p);
}

void Blackboard::distract()
{
}

PROCESS_WIDE_STORAGE Blackboard* Blackboard::theInstance = 0;
