/**
* @file Representations/Configuration/BehaviorParameters.h
* The file declares a class that containts frequently used parameters of the behavior which can be modified.
* @author <a href="mailto:dhonsel@informatik.uni-bremen.de">Daniel Honsel</a>
*/
#ifndef __BehaviorParameters_h_
#define __BehaviorParameters_h_

#include "Tools/Streams/Streamable.h"
#include "Tools/Math/Vector2.h"

class BehaviorParameters : public Streamable  
{
private:
  /**
  * The method makes the object streamable.
  * @param in The stream from which the object is read (if in != 0).
  * @param out The stream to which the object is written (if out != 0).
  */
  virtual void serialize(In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN();
      STREAM(kickDistanceGreatX);
      STREAM(kickDistanceSmallX);
      STREAM(kickDistanceGreatY);
      STREAM(kickDistanceMidY);
      STREAM(kickDistanceSmallY);
      STREAM(distanceWalkPedantic);
      STREAM(distanceKickoffDribble);
      STREAM(kickoffDribbleTolerance);
      STREAM(kickoffDribbleBallOffsetY);
      STREAM(translationStrikerX);
      STREAM(translationStrikerY);
      STREAM(translationStrikerDefensiveX);
      STREAM(translationStrikerDefensiveY);
      STREAM(translationKeeperX);
      STREAM(translationKeeperY);
    STREAM_REGISTER_FINISH();
  }

public:  
  /**
  * These values are used for positioning before kicking. 
  * Added to the radius of the ball they result the distance to the ball.
  */
  float kickDistanceGreatX; /**< Often used great x-distance (kickDistanceGreatX + radius => approx. 250mm) */
  float kickDistanceSmallX; /**< Often used small x-distance (kickDistanceSmallX + radius => approx. 200mm) */
  float kickDistanceGreatY; /**< Often used great y-distance (kickDistanceGreatY + radius => approx. 200mm) */
  float kickDistanceMidY;   /**< Often used mid y-distance (kickDistanceMidY + radius => approx. 100mm) */
  float kickDistanceSmallY; /**< Often used small y-distance (kickDistanceSmallY + radius => approx. 40mm) */

  float distanceWalkPedantic; /**< From this distance the robot walks pedantic to the ball. */

  Vector2<> distanceKickoffDribble; /**< Distance to begin dribbling in kickoff. */
  float kickoffDribbleTolerance; /**< Hysteresis used in kickoff dribble. */
  float kickoffDribbleBallOffsetY; /**< The aimed ball offset in y-direction in kickoff dribble. */
  float translationStrikerX;
  float translationStrikerY;
  float translationStrikerDefensiveX;
  float translationStrikerDefensiveY;
  float translationKeeperX;
  float translationKeeperY;

  /** Default constructor. */
  BehaviorParameters() : 
    kickDistanceGreatX(0), 
    kickDistanceSmallX(0), 
    kickDistanceGreatY(0), 
    kickDistanceMidY(0), 
    kickDistanceSmallY(0), 
    distanceWalkPedantic(0),
    distanceKickoffDribble(0, 0),
    kickoffDribbleTolerance(0),
    kickoffDribbleBallOffsetY(0),
    translationStrikerX(0),
    translationStrikerY(0),
    translationStrikerDefensiveX(0),
    translationStrikerDefensiveY(0),
    translationKeeperX(0),
    translationKeeperY(0) {}
};

#endif //__BehaviorParameters_h_
