/**
 * @file BehaviorTactics.h
 * Declaration of class BehaviorTactics
 * 
 * This class contains three parameters which are
 * used in behavior control for determining which
 * tactic will be used. The values for these parameters
 * are read from the config file "tactics.cfg".
 * 
 * @author Michael Mester
 */

#ifndef __BehaviorTactics_h_
#define __BehaviorTactics_h_

#include <string>
#include "Tools/Streams/Streamable.h"


class BehaviorTactics : public Streamable
{
public:

  BehaviorTactics() : 
      tactic(0),
      dribblingOn(0),
      aggressiveness(0) {}

  enum Tactic{
    defensive,
    single_striker,
    offensive
  };

  int tactic;
  bool dribblingOn;
  float aggressiveness;

private:
  virtual void serialize( In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
      STREAM(tactic);
      STREAM(dribblingOn);
      STREAM(aggressiveness);
    STREAM_REGISTER_FINISH();
  }
};

#endif //__BehaviorTactics_h_
