/**
 * @file RobotDimensions.h
 *
 * Description of the Dimensions of the Kondo Robot
 *
 * @author Cord Niehaus
 */

#ifndef __RobotDimensions_H__
#define __RobotDimensions_H__

#include "Tools/Streams/Streamable.h"
#include "Tools/Math/Vector3.h"

class RobotDimensions : public Streamable
{
private:
  virtual void serialize(In* in, Out* out)
  { 
    STREAM_REGISTER_BEGIN();

    STREAM(lengthBetweenLegs);
    STREAM(upperLegLength);
    STREAM(lowerLegLength);
    STREAM(heightLeg5Joint);
 
    STREAM(zLegJoint1ToHeadPan);
    STREAM(xHeadTiltToCamera);
    STREAM(zHeadTiltToCamera);
    STREAM(headTiltToCameraTilt);

    STREAM(xHeadTiltToUpperCamera);
    STREAM(zHeadTiltToUpperCamera);
    STREAM(headTiltToUpperCameraTilt);

    STREAM(armOffset);
    STREAM(upperArmLength);
    STREAM(lowerArmLength);

    STREAM(motionCycleTime);         
    STREAM(imageRecordingTime);
    STREAM(imageRecordingDelay);

    STREAM_REGISTER_FINISH();
  }

public:

  float lengthBetweenLegs;         //!<length between leg joints LL1 and LR1
  float upperLegLength;            //!<length between leg joints LL2 and LL3 in z-direction
  float lowerLegLength;            //!<length between leg joints LL3 and LL4 in z-direction
  float heightLeg5Joint;           //!<height of leg joints LL4 and LR4 of the ground
 
  float zLegJoint1ToHeadPan;       //!<height offset between LL1 and head pan joint 
  float xHeadTiltToCamera;         //!<forward offset between head tilt joint and lower camera
  float zHeadTiltToCamera;         //!<height offset between head tilt joint lower and 
  float headTiltToCameraTilt;      //!<tilt of lower camera against head tilt

  float xHeadTiltToUpperCamera;    //!<forward offset between head tilt joint and upper camera
  float zHeadTiltToUpperCamera;    //!<height offset between head tilt joint upper and
  float headTiltToUpperCameraTilt; //!<tilt of upper camera against head tilt

  Vector3<> armOffset;             //! The offset of the first left arm joint relative to the middle between the hip joints
  float upperArmLength;            //! The length between the shoulder and the elbow in y-direction
  float lowerArmLength;            //!< height off lower arm starting at arm2/arm3

  float motionCycleTime;           //! Length of one motion cycle in seconds.
  float imageRecordingTime;        //! Time the camera requires to take an image (in s, for motion compensation, may depend on exposure). 
  float imageRecordingDelay;       //! Delay after the camera took an image (in s, for motion compensation).
};

#endif
