/**
* @file Representations/Debugging/SimpleXabslInfo.h
*
* Declaration of class SimpleXabslInfo
*
* @author <a href="mailto:Tim.Laue@dfki.de">Tim Laue</a>
*/

#ifndef __SimpleXabslInfo_h_
#define __SimpleXabslInfo_h_

#include <vector>
#include <string>
#include "Tools/Streams/Streamable.h"


/**
* @class SimpleXabslInfo 
* 
* A class to represent information about a Xabsl behavior.
*
* @author <a href="mailto:Tim.Laue@dfki.de">Tim Laue</a>
*/
class SimpleXabslInfo: public Streamable
{
public:
  /** Serialize function, currently not used */
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
      STREAM(selectedAgentName);
    STREAM_REGISTER_FINISH();
  }

  /** An active option. */
  struct ActiveOption
  {
    std::string name,              /**< The name of the active option. */
                activeState;       /**< The active state. */
    int durationOfActivation,      /**< The duration this option is active. */        
        durationOfStateActivation; /**< The duration the state is active. */
    int depth;                     /**< Depth in option activation tree. */
  };

  std::string selectedAgentName;           /**< The name of the agent. */
  std::vector<ActiveOption> activeOptions; /**< The active options. */
 
  /** Default constructor. */
  SimpleXabslInfo() {reset();}

  /**
  * The function empties the Xabsl info object.
  */
  void reset()
  {
    selectedAgentName = "none";
    activeOptions.clear();
  }
};

#endif //__SimpleXabslInfo_h_
