/**
* @file AnotherObstacleModel.h
* Declaration of class AnotherObstacleModel
* @author Colin Graf
* @author <a href="mailto:Tim.Laue@dfki.de">Tim Laue</a>
*/

#ifndef AnotherObstacleModel_H
#define AnotherObstacleModel_H

#include "Tools/Math/Vector2.h"
#include "Tools/Debugging/DebugDrawings.h"

/**
* @class AnotherObstacleModel
* A class that represents the current state of the robot's environment
*/
class AnotherObstacleModel : public Streamable
{
public:
  class Obstacle : public Streamable
  {
  public:
    Vector2<> leftCorner;
    Vector2<> rightCorner;
    Vector2<> center;
    Vector2<> closestPoint;
   
    Obstacle() {}

    Obstacle(const Vector2<>& leftCorner, const Vector2<>& rightCorner,
      const Vector2<>& center, const Vector2<>& closestPoint) : 
    leftCorner(leftCorner), rightCorner(rightCorner), center(center), closestPoint(closestPoint) {}

  private:
    /**
    * Makes the object streamable
    * @param in The stream from which the object is read 
    * @param out The stream to which the object is written 
    */
    void serialize(In* in, Out* out)
    {
      STREAM_REGISTER_BEGIN();
        STREAM(leftCorner);
        STREAM(rightCorner);
        STREAM(center);
        STREAM(closestPoint);
      STREAM_REGISTER_FINISH();
    }
  };

  std::vector<Obstacle> obstacles;

  /** Default constructor. */
  AnotherObstacleModel() { obstacles.reserve(10); }

  /** Function for drawing */
  void draw() 
  {
    DECLARE_DEBUG_DRAWING("representation:AnotherObstacleModel", "drawingOnField"); 
    COMPLEX_DRAWING("representation:AnotherObstacleModel",
    {
      for(std::vector<Obstacle>::const_iterator it = obstacles.begin(), end = obstacles.end(); it != end; ++it)
      {
        const Obstacle& obstacle = *it;
        const Vector2<>& a = obstacle.leftCorner;
        const Vector2<>& b = obstacle.rightCorner;
        LINE("representation:AnotherObstacleModel", 0, 0, a.x, a.y, 30, Drawings::ps_solid, ColorClasses::black);
        LINE("representation:AnotherObstacleModel", 0, 0, b.x, b.y, 30, Drawings::ps_solid, ColorClasses::black);
        LINE("representation:AnotherObstacleModel", a.x, a.y, b.x, b.y, 30, Drawings::ps_solid, ColorClasses::black);
        CROSS("representation:AnotherObstacleModel", obstacle.center.x, obstacle.center.y, 100, 30, Drawings::ps_solid, ColorClasses::blue);
        CROSS("representation:AnotherObstacleModel", obstacle.closestPoint.x, obstacle.closestPoint.y, 100, 30, Drawings::ps_solid, ColorClasses::red);
      }
    });
  }

private:
  /**
  * Makes the object streamable
  * @param in The stream from which the object is read 
  * @param out The stream to which the object is written 
  */
  void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM_VECTOR(obstacles);
    STREAM_REGISTER_FINISH();
  }
};

#endif // AnotherObstacleModel_H
