/**
* @file RobotsModel.cpp
* 
* Debug drawings for the RobotsModel.
*
* @author <a href="mailto:afabisch@tzi.de">Alexander Fabisch</a>
*/

#include "RobotsModel.h"
#include "Platform/GTAssert.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Representations/Infrastructure/GameInfo.h"

void RobotsModel::draw()
{
  DECLARE_DEBUG_DRAWING("representation:RobotsModel:robots", "drawingOnField");
  DECLARE_DEBUG_DRAWING("representation:RobotsModel:covariance", "drawingOnField");

  for(RIt r(robots.begin()); r != robots.end(); r++)
  {
    COMPLEX_DRAWING("representation:RobotsModel:covariance",
    {
      float s1(r->ellipse.x), s2(r->ellipse.y), theta(r->ellipse.z);

      // error ellipse
      ELLIPSE("representation:RobotsModel:covariance", r->relPosOnField, sqrtf(3.0f)*s1, sqrtf(3.0f)*s2, theta,
        10, Drawings::ps_solid, ColorRGBA(100,100,255,100), Drawings::bs_solid, ColorRGBA(100,100,255,100));
      ELLIPSE("representation:RobotsModel:covariance", r->relPosOnField, sqrtf(2.0f)*s1, sqrtf(2.0f)*s2, theta,
        10, Drawings::ps_solid, ColorRGBA(150,150,100,100), Drawings::bs_solid, ColorRGBA(150,150,100,100));
      ELLIPSE("representation:RobotsModel:covariance", r->relPosOnField, s1, s2, theta,
        10, Drawings::ps_solid, ColorRGBA(255,100,100,100), Drawings::bs_solid, ColorRGBA(255,100,100,100));
    });

    COMPLEX_DRAWING("representation:RobotsModel:robots",
    {
      ASSERT(r->team == TEAM_BLUE || r->team == TEAM_RED);
      ColorClasses::Color color(r->team == TEAM_BLUE ? ColorClasses::robotBlue :
                                r->team == TEAM_RED ? ColorClasses::red : ColorClasses::black);
      CROSS("representation:RobotsModel:robots", r->relPosOnField.x, r->relPosOnField.y,
        50, 20, Drawings::ps_solid, color);
    });
  }
}

