/**
 * @file RobotsModel.h
 *
 * @author <a href="mailto:afabisch@tzi.de>Alexander Fabisch</a>
 */
#ifndef __RobotsModel_h_
#define __RobotsModel_h_

#include "Tools/Streams/Streamable.h"
#include "Tools/Math/Vector2.h"
#include "Tools/Math/Vector3.h"
#include <vector>

/**
 * @class RobotsModel
 * This representation contains all percepted and smoothed robots in relative field coordinates.
 */
class RobotsModel : public Streamable
{
  void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM_VECTOR(robots);
    STREAM_REGISTER_FINISH();
  }
public:
  /**
   * @class Robot
   * A robot on the field.
   */
  class Robot : public Streamable
  {
    void serialize(In* in, Out* out)
    {
      STREAM_REGISTER_BEGIN();
      STREAM(relPosOnField);
      STREAM(team);
      STREAM(standing);
      STREAM(ellipse);
      STREAM_REGISTER_FINISH();
    }

  public:
    Vector2<> relPosOnField;  /**< Relative position of the robot on the field. */
    int team;                 /**< TEAM_BLUE or TEAM_RED? see GameInfo.h */
    bool standing;            /**< Is this robot standing or horizontal? */
    Vector3<> ellipse;        /**< The error ellipse is given by the length of principle axes and the orientation. */

    Robot() {}

    Robot(const Robot& other)
        : relPosOnField(other.relPosOnField),
          team(other.team),
          standing(other.standing),
          ellipse(other.ellipse) {}

    Robot(Vector2<> relPosOnField, int team, bool standing, Vector3<> ellipse)
        : relPosOnField(relPosOnField),
          team(team),
          standing(standing),
          ellipse(ellipse) {}
  };

  typedef std::vector<Robot>::const_iterator RCIt;
  typedef std::vector<Robot>::iterator RIt;

  std::vector<Robot> robots;

  void draw();
};

#endif // __RobotsModel_h_

