/**
* @file Representations/MotionControl/BikeRequest.h
* This file declares a class that represents a bike request.
* @author <A href="mailto:judy@tzi.de">Judith Mller</A>
*/

#ifndef __BikeRequest_H__
#define __BikeRequest_H__

#include <cstring>
#include "Tools/Math/Vector3.h"
#include "Tools/Math/Pose3D.h"

class DynPoint: public Streamable
{
public:
  int limb;
  int phaseNumber;
  Vector3<> translation;
  Vector3<> angle;
  Vector3<> odometryOffset;


  DynPoint(){};

  DynPoint(int limb, int phaseNumber, Vector3<> translation, Vector3<>angle, Vector3<> odometryOffset):
  limb(limb),
    phaseNumber(phaseNumber),
    translation(translation),
    angle(angle),
    odometryOffset(odometryOffset)
  {};


  ~DynPoint(){};

private:
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN( );
    STREAM(limb);
    STREAM(phaseNumber);
    STREAM(translation);
    STREAM(angle);
    STREAM(odometryOffset);
    STREAM_REGISTER_FINISH();
  }
};

/**
* @class BikeRequest
* A class that represents a bike request.
*/
class BikeRequest : public Streamable
{
private:


  virtual void serialize( In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN( );
    STREAM_ENUM(bMotionType, numOfBMotions, getBMotionName);
    STREAM(mirror);
    STREAM(dynamical);
    STREAM_VECTOR(dynPoints);
    STREAM_REGISTER_FINISH();
  }

public:

  /** Hier kommen noch einige Parameter */

  enum BMotionID
  {
    trykick,
    //add new motions before newKick
    newKick, //DebugMotion
    none,
    numOfBMotions
  };

  bool mirror, dynamical;
  BMotionID bMotionType; 
  std::vector<DynPoint> dynPoints;

  /**
  * Assignment operator
  * @param other The other BikeRequest that is assigned to this one
  * @return A reference to this object after the assignment
  */
  BikeRequest& operator=(const BikeRequest& other)
  {
    mirror = other.mirror;
    dynamical = other.dynamical;
    bMotionType = other.bMotionType;
    dynPoints = other.dynPoints;
    return *this;
  }

  static const char* getBMotionName(BMotionID id)
  {
    switch (id)
    {  
    case trykick: return "trykick";
    //Debug Kick for adding new Kicks
    case newKick: return "noName";
    case none: return "none";
    default: return "unknown";
    }
  }

  static BMotionID getBMotionFromName(const char* name)
  {    for(int i = 0; i < numOfBMotions; ++i)
  if(!strcmp(name, getBMotionName(BMotionID(i))))
    return BMotionID(i);
  return numOfBMotions;
  }

  BikeRequest():mirror(false), dynamical(false), bMotionType(none){}; 

};

#endif // __BikeRequest_H__
