/** 
* @file MotionInfo.h
* Definition of class MotionInfo.
* @author Martin Ltzsch
*/

#ifndef MotionInfo_H
#define MotionInfo_H

#include "MotionRequest.h"

/**
* @class MotionInfo
* The executed motion request and additional information about the motions which are executed by the Motion process.
*/
class MotionInfo : public MotionRequest
{
public:
  bool isMotionStable; /**< If true, the motion is stable, leading to a valid torso / camera matrix. */

  /** Default constructor. */
  MotionInfo() : isMotionStable(false) {}
  
private:
  /**
  * The method makes the object streamable.
  * @param in The stream from which the object is read 
  * @param out The stream to which the object is written 
  */
  virtual void serialize(In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN();
    STREAM_BASE(MotionRequest);
    STREAM(isMotionStable);
    STREAM_REGISTER_FINISH();
  }  
};

#endif // MotionInfo_H
