/**
* @file Representations/MotionControl/SIKERequest.h
* This file declares a class that represents a SIKE request.
* author jeff
*/

#ifndef SIKERequest_H
#define SIKERequest_H

#include "Tools/Math/Pose3D.h"

/**
* @class SIKERequest
* A class that represents a SIKE request.
*/
class SIKERequest : public Streamable
{
private:

  virtual void serialize( In* in, Out* out)
  {  
    STREAM_REGISTER_BEGIN( );
    STREAM_ENUM(motion, numOfMotions, getMotionName);
    STREAM(mirror);
    STREAM_REGISTER_FINISH();
  }

public:

  enum Motion
  {
    demoJesus,
    testKick,
    quickKick,
    sitDown2,
    editor,
    numOfMotions,
  };

  bool mirror;
  Motion motion; 

  static const char* getMotionName(Motion id)
  {
    switch (id)
    {  
      case demoJesus: return "demoJesus";
      case testKick: return "testKick";
      case quickKick: return "quickKick";
      case sitDown2: return "sitDown2";
      case editor: return "editor";
      default: return "unknown";
    }
  }

  SIKERequest():mirror(false), motion(numOfMotions) {}; 
};

#endif // __SIKERequest_H__

