/**
* @file RobotPercept.h
* @author <a href="afabisch@tzi.de">Alexander Fabisch</a>
*/
#ifndef __RobotPercept_h_
#define __RobotPercept_h_

#include "Tools/Math/Vector2.h"
#include "Tools/Debugging/DebugDrawings.h"
#include "Representations/Infrastructure/GameInfo.h"
#include "Tools/ColorClasses.h"
#include <vector>

class RobotPercept : public Streamable
{
private:
  void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
    STREAM_VECTOR(robots);
    STREAM_REGISTER_FINISH();
  }
public:
  class Robot : public Streamable
  {
    void serialize(In* in, Out* out)
    {
      STREAM_REGISTER_BEGIN();
      STREAM(lowestPx);
      STREAM(team);
      STREAM(standing);
      STREAM_REGISTER_FINISH();
    }
  public:
    Vector2<int> lowestPx; /**< coordinates of the lowest seen pixel */
    int team; /**< the team color of the seen robot (either TEAM_BLUE or TEAM_RED) */
    bool standing; /**< whether the robot is upright or lying on the field */

    Robot() {}
    Robot(const Vector2<int>& lowestPx, bool standing, ColorClasses::Color color) : lowestPx(lowestPx), standing(standing)
    {
      if(color == ColorClasses::blue || color == ColorClasses::robotBlue)
        team = TEAM_BLUE;
      else if(color == ColorClasses::red)
        team = TEAM_RED;
      else
        team = -1;
    }
  };

  typedef std::vector<Robot>::const_iterator RCIt;
  typedef std::vector<Robot>::iterator RIt;

  std::vector<Robot> robots;
  std::vector<Robot> unlocalizableRobots;

  void draw()
  {
    DECLARE_DEBUG_DRAWING("representation:RobotPercept:pixels", "drawingOnImage");
    COMPLEX_DRAWING("representation:RobotPercept:pixels",
      for(RCIt r(robots.begin()); r != robots.end(); r++)
      {
        ColorClasses::Color color(r->team == TEAM_BLUE ? ColorClasses::robotBlue :
                                  r->team == TEAM_RED ? ColorClasses::red : ColorClasses::black);
        MID_DOT("representation:RobotPercept:pixels", (int) r->lowestPx.x, (int) r->lowestPx.y, color, color);
      }
    );
  }
};

#endif

