/**
* @file InertiaSensorData.h
* Declaration of class InertiaSensorData.
* @author Colin Graf
*/ 

#ifndef InertiaSensorData_H
#define InertiaSensorData_H

#include "Tools/Streams/Streamable.h"
#include "Tools/Math/Vector2.h"
#include "Tools/Math/Vector3.h"
#include "Representations/Infrastructure/SensorData.h"

/**
* @class InertiaSensorData
* Encapsulates calibrated inertia sensor data.
*/
class InertiaSensorData : public Streamable
{
public:
  enum
  {
    off = SensorData::off, /**< A special value to indicate that the sensor is missing. */
  };

  Vector2<float> gyro; /**< The change in orientation around the x- and y-axis. (in radian/s) */
  Vector3<float> acc; /**< The acceleration along the x-, y- and z-axis. (in m/s^2) */

  /** Default constructor. */
  InertiaSensorData() {}
  
private:
  /**
  * The method makes the object streamable.
  * @param in The stream from which the object is read
  * @param out The stream to which the object is written 
  */
  virtual void serialize(In* in, Out* out)
  {
    STREAM_REGISTER_BEGIN();
      STREAM(gyro);
      STREAM(acc);
    STREAM_REGISTER_FINISH();
  }
};

#endif // InertiaSensorData_H
